@@ -1,5 +1,5 @@
 /*
- * $Id: dummy_ci.h,v 1.4 2000/07/27 17:47:03 lange Exp $
+ * $Id: dummy_ci.h,v 1.3 1999/03/20 16:33:34 grubba Exp $
  *
  * Fallback case_info file.
  * Only ISO-8859-1 in this one.
@@ -11,8 +11,6 @@
 { 0x005b, CIM_NONE, 0x0000, },
 { 0x0061, CIM_CASEBIT, 0x0020, },
 { 0x007b, CIM_NONE, 0x0000, },
-{ 0x00b5, CIM_LOWERDELTA, -0x02e7, },
-{ 0x00b6, CIM_NONE, 0x0000, },
 { 0x00c0, CIM_CASEBIT, 0x0020, },
 { 0x00d7, CIM_NONE, 0x0000, },
 { 0x00d8, CIM_CASEBIT, 0x0020, },
@@ -22,4 +20,3 @@
 { 0x00f8, CIM_CASEBIT, 0x0020, },
 { 0x00ff, CIM_LOWERDELTA, -0x0079, },
 { 0x0100, CIM_NONE, 0x0000, },
-#define CASE_INFO_SHIFT0_HIGH 0x000f
