@@ -5,19 +5,18 @@
 \*/
 
 /*
- * $Id: fd_control.h,v 1.6 2000/12/16 05:24:40 marcus Exp $
+ * $Id: fd_control.h,v 1.4 1998/06/08 12:48:26 grubba Exp $
  */
 #ifndef FD_CONTROL_H
 #define FD_CONTROL_H
 
 /* Prototypes begin here */
-PMOD_EXPORT int set_nonblocking(int fd,int which);
-PMOD_EXPORT int query_nonblocking(int fd);
-PMOD_EXPORT int set_close_on_exec(int fd, int which);
+int set_nonblocking(int fd,int which);
+int query_nonblocking(int fd);
+int set_close_on_exec(int fd, int which);
 
 #ifdef HAVE_BROKEN_F_SETFD
 void do_close_on_exec(void);
-void cleanup_close_on_exec(void);
 #endif /* HAVE_BROKEN_F_SETFD */
 /* Prototypes end here */
 
