@@ -1,33 +1,25 @@
-/*
-|| This file is part of Pike. For copyright information see COPYRIGHT.
-|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
-|| for more information.
-|| $Id: clf.c,v 1.16 2004/04/06 13:00:48 nilsson Exp $
-*/
 
-/* MUST BE FIRST */
 #include "global.h"
-RCSID("$Id: clf.c,v 1.16 2004/04/06 13:00:48 nilsson Exp $");
+RCSID("$Id: clf.c,v 1.2 2000/03/27 00:04:48 grubba Exp $");
 #include "fdlib.h"
 #include "stralloc.h"
+#include "pike_macros.h"
+#include "object.h"
 #include "program.h"
 #include "interpret.h"
 #include "builtin_functions.h"
 #include "module_support.h"
-#include "pike_error.h"
-#include "bignum.h"
-#include "pike_security.h"
+#include "error.h"
 
 #include "threads.h"
 #include <stdio.h>
 #include <fcntl.h>
 
-
-#define sp Pike_sp
-
 /** Forward declarations of functions implementing Pike functions **/
 
-static void f_read( INT32 args );
+static void f_read_clf( INT32 args );
+
+extern int fd_from_object(struct object *o);
 
 
 /** Global tables **/
@@ -48,6 +40,7 @@ static char char_class[1<<CHAR_BIT];
 #define CLS_SLASH  7
 #define CLS_COLON  8
 #define CLS_HYPHEN 9
+#define CLS_MINUS  9
 #define CLS_PLUS   10
 
 
@@ -58,7 +51,7 @@ static char char_class[1<<CHAR_BIT];
 
 /* Initialize and start module */
 
-PIKE_MODULE_INIT
+void pike_module_init( void )
 {
   int i;
 
@@ -77,7 +70,7 @@ PIKE_MODULE_INIT
   char_class['-'] = CLS_HYPHEN;
   char_class['+'] = CLS_PLUS;
 
-  add_function_constant( "read", f_read,
+  add_function_constant( "read", f_read_clf,
 			 "function(function(array(string|int),int|void:void),"
 			 "string|object,int|void:int)", 0 );
 }
@@ -85,78 +78,23 @@ PIKE_MODULE_INIT
 
 /* Restore and exit module */
 
-PIKE_MODULE_EXIT
+void pike_module_exit( void )
 {
 }
 
 
 /** Functions implementing Pike functions **/
 
-/*! @module CommonLog
- *!
- *! The CommonLog module is used to parse the lines in a www server's logfile,
- *! which must be in "common log" format -- such as used by default for the
- *! access log by Roxen, Caudium, Apache et al.
- */
+/* CommonLog.read() */
 
-/*! @decl int read(function(array(int|string), int : void ) callback,@
- *!            Stdio.File|string logfile, void|int offset)
- *!
- *! Reads the log file and calls the callback function for every parsed line.
- *! For lines that fails to be parsed the callback is not called not is any
- *! error thrown. The number of bytes read are returned.
- *!
- *! @param callback
- *! The callbacks first argument is an array with the different parts of the
- *! log entry.
- *! @array
- *!   @elem string remote_host
- *!
- *!   @elem int(0..0)|string ident_user
- *!
- *!   @elem int(0..0)|string auth_user
- *!
- *!   @elem int year
- *!
- *!   @elem int month
- *!
- *!   @elem int day
- *!
- *!   @elem int hours
- *!
- *!   @elem int minutes
- *!
- *!   @elem int seconds
- *!
- *!   @elem int timezone
- *!
- *!   @elem int(0..0)|string method
- *!     One of "GET", "POST", "HEAD" etc.
- *!   @elem int(0..0)|string path
- *!
- *!   @elem string protocol
- *!     E.g. "HTTP/1.0"
- *!   @elem int reply_code
- *!     One of 200, 404 etc.
- *!   @elem int bytes
- *! @endarray
- *!
- *! The second callback argument is the current offset to the end of the
- *! current line.
- *!
- *! @param offset
- *! The position in the file where the parser should begin.
- */
-
-static void f_read( INT32 args )
+static void f_read_clf( INT32 args )
 {
   char *read_buf;
   struct svalue *logfun, *file;
   FD f = -1;
   int cls, c, my_fd=1, state=0, tzs=0;
   char *char_pointer;
-  INT32 v=0, yy=0, mm=0, dd=0, h=0, m=0, s=0, tz=0;
-  ptrdiff_t offs0=0, len=0;
+  INT32 v=0, yy=0, mm=0, dd=0, h=0, m=0, s=0, tz=0, offs0=0, len=0;
   struct svalue *old_sp;
   /* #define DYNAMIC_BUF */
 #ifdef DYNAMIC_BUF
@@ -177,76 +115,29 @@ static void f_read( INT32 args )
 
   get_all_args("CommonLog.read", args, "%*%*", &logfun, &file);
   if(logfun->type != T_FUNCTION)
-    SIMPLE_BAD_ARG_ERROR("CommonLog.read", 1, "function");
+    error("Bad argument 1 to CommonLog.read, expected function.\n");
 
   if(file->type == T_OBJECT)
   {
     f = fd_from_object(file->u.object);
     
     if(f == -1)
-      Pike_error("CommonLog.read: File is not open.\n");
+      error("CommonLog.read: File is not open.\n");
     my_fd = 0;
   } else if(file->type == T_STRING &&
 	    file->u.string->size_shift == 0) {
-#ifdef PIKE_SECURITY
-      if(!CHECK_SECURITY(SECURITY_BIT_SECURITY))
-      {
-	if(!CHECK_SECURITY(SECURITY_BIT_CONDITIONAL_IO))
-	  Pike_error("Permission denied.\n");
-	push_text("read");
-	push_int(0);
-	ref_push_string(file->u.string);
-	push_text("r");
-	push_int(00666);
-
-	safe_apply(OBJ2CREDS(CURRENT_CREDS)->user,"valid_open",5);
-	switch(Pike_sp[-1].type)
-	{
-	case PIKE_T_INT:
-	  switch(Pike_sp[-1].u.integer)
-	  {
-	  case 0: /* return 0 */
-	    errno=EPERM;
-	    Pike_error("CommonLog.read(): Failed to open file for reading (errno=%d).\n",
-		       errno);
-
-	  case 2: /* ok */
-	    pop_stack();
-	    break;
-
-	  case 3: /* permission denied */
-	    Pike_error("CommonLog.read: permission denied.\n");
-
-	  default:
-	    Pike_error("Error in user->valid_open, wrong return value.\n");
-	  }
-	  break;
-
-	default:
-	  Pike_error("Error in user->valid_open, wrong return type.\n");
-
-	case PIKE_T_STRING:
-	  /*	  if(Pike_sp[-1].u.string->shift_size) */
-	  /*	    file=Pike_sp[-1]; */
-	  pop_stack();
-	}
-
-      }
-#endif
+    THREADS_ALLOW();
     do {
-      THREADS_ALLOW();
       f=fd_open((char *)STR0(file->u.string), fd_RDONLY, 0);
-      THREADS_DISALLOW();
-      if (f >= 0 || errno != EINTR) break;
-      check_threads_etc();
-    } while (1);
-
+    } while(f < 0 && errno == EINTR);
+    THREADS_DISALLOW();
+    
     if(errno < 0)
-      Pike_error("CommonLog.read(): Failed to open file for reading (errno=%d).\n",
+      error("CommonLog.read(): Failed to open file for reading (errno=%d).\n",
 	    errno);
-  } else
-    SIMPLE_BAD_ARG_ERROR("CommonLog.read", 2, "string|Stdio.File");
-
+  } else 
+    error("Bad argument 1 to CommonLog.read, expected string or object .\n");
+  
 #ifdef HAVE_LSEEK64
   lseek64(f, offs0, SEEK_SET);
 #else
@@ -257,13 +148,11 @@ static void f_read( INT32 args )
   buf = malloc(bufsize);
 #endif
   while(1) {
+    THREADS_ALLOW();
     do {
-      THREADS_ALLOW();
       len = fd_read(f, read_buf, CLF_BLOCK_SIZE);
-      THREADS_DISALLOW();
-      if (len >= 0 || errno != EINTR) break;
-      check_threads_etc();
-    } while (1);
+    } while(len < 0 && errno == EINTR);
+    THREADS_DISALLOW();
     if(len == 0)
       break; /* nothing more to read. */
     if(len < 0)
@@ -286,7 +175,7 @@ static void f_read( INT32 args )
       case CLS_RBRACK: fprintf(stderr, "CLS_RBRACK"); break;
       case CLS_SLASH: fprintf(stderr, "CLS_SLASH"); break;
       case CLS_COLON: fprintf(stderr, "CLS_COLON"); break;
-      case CLS_HYPHEN: fprintf(stderr, "CLS_HYPHEN"); break;
+      case CLS_HYPHEN: fprintf(stderr, "CLS_HYPHEN/CLS_MINUS"); break;
       case CLS_PLUS: fprintf(stderr, "CLS_PLUS"); break;
       default: fprintf(stderr, "???");
       }
@@ -297,7 +186,7 @@ static void f_read( INT32 args )
 	if(sp != old_sp) {
 	  if(sp == old_sp+15) {
 	    f_aggregate(15);
-	    push_int64(offs0);
+	    push_int(offs0);
 	    apply_svalue(logfun, 2);
 	    pop_stack();
 	  } else
@@ -644,7 +533,7 @@ static void f_read( INT32 args )
 	  state = (cls == CLS_CRLF? 0:14);
 	break;
       case 28:
-	if(cls>=CLS_HYPHEN) {
+	if(cls>=CLS_MINUS) {
 	  state = 29;
 	  tzs = cls!=CLS_PLUS;
 	  tz = 0;
@@ -813,7 +702,7 @@ static void f_read( INT32 args )
   }
   if(sp == old_sp + 15) {
     f_aggregate(15);
-    push_int64(offs0);
+    push_int(offs0);
     apply_svalue(logfun, 2);
     pop_stack();
   }
@@ -827,8 +716,9 @@ static void f_read( INT32 args )
      */
     fd_close(f);
   pop_n_elems(sp-old_sp+args);  
-  push_int64(offs0);
+  push_int(offs0);
 }
 
-/*! @endmodule
- */
+
+
+
