@@ -1,37 +1,29 @@
-/*
-|| This file is part of Pike. For copyright information see COPYRIGHT.
-|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
-|| for more information.
-|| $Id: xcf.c,v 1.49 2005/05/18 12:36:54 mast Exp $
-*/
-
 #include "global.h"
-RCSID("$Id: xcf.c,v 1.49 2005/05/18 12:36:54 mast Exp $");
+RCSID("$Id: xcf.c,v 1.14 2000/04/10 21:58:23 grubba Exp $");
 
 #include "image_machine.h"
 
 #include "pike_macros.h"
 #include "object.h"
+#include "constants.h"
 #include "module_support.h"
 #include "interpret.h"
 #include "object.h"
 #include "svalue.h"
 #include "threads.h"
+#include "array.h"
 #include "interpret.h"
 #include "svalue.h"
 #include "mapping.h"
-#include "pike_error.h"
+#include "error.h"
 #include "stralloc.h"
 #include "builtin_functions.h"
-#include "bignum.h"
+#include "operators.h"
+#include "dynamic_buffer.h"
 
 #include "image.h"
 #include "colortable.h"
 
-
-#define sp Pike_sp
-#define fp Pike_fp
-
 extern struct program *image_colortable_program;
 extern struct program *image_program;
 
@@ -51,158 +43,10 @@ extern struct program *image_program;
 
 struct buffer
 {
-  struct pike_string *s;
-  ptrdiff_t base_offset;
-  ptrdiff_t base_len;
-  size_t len;
+  unsigned int len;
   unsigned char *str;
 };
 
-struct substring
-{
-  struct pike_string *s;
-  ptrdiff_t offset;
-  ptrdiff_t len;
-};
-
-static struct program *substring_program;
-#define SS(X) ((struct substring*)X->storage)
-
-static void f_substring_cast( INT32 args )
-{
-  /* FIXME: assumes string */
-  struct substring *s = SS(fp->current_object);
-  pop_n_elems( args );
-  push_string( make_shared_binary_string( (((char *)s->s->str)+s->offset),
-                                          s->len ) );
-}
-
-static void f_substring_index( INT32 args )
-{
-  ptrdiff_t i = sp[-1].u.integer;
-  struct substring *s = SS(fp->current_object);
-  pop_n_elems( args );
-
-  if( i < 0 ) i = s->len + i;
-  if( i >= s->len ) {
-    Pike_error("Index out of bounds, %"PRINTPTRDIFFT"d > %"PRINTPTRDIFFT"d\n",
-	       i, s->len-1);
-  }
-  push_int( ((unsigned char *)s->s->str)[s->offset+i] );
-}
-
-static void f_substring__sprintf( INT32 args )
-{
-  int x;
-  struct substring *s = SS(fp->current_object);
-
-  if (args != 2 )
-    SIMPLE_TOO_FEW_ARGS_ERROR("_sprintf",2);
-  if (sp[-2].type!=T_INT)
-    SIMPLE_BAD_ARG_ERROR("_sprintf",0,"integer");
-  if (sp[-1].type!=T_MAPPING)
-    SIMPLE_BAD_ARG_ERROR("_sprintf",1,"mapping");
-  x = sp[-2].u.integer;
-  pop_n_elems( args );
-
-  switch( x )
-  {
-   case 't':
-     push_constant_text("SubString");
-     return;
-   case 'O':
-     push_constant_text("SubString( %O /* [+%d .. %d] */ )" );
-     push_text("string"); f_substring_cast( 1 );
-
-     push_int64( s->len );
-     push_int64( s->offset );
-     f_sprintf( 4 );
-     return;
-   default: 
-     push_int(0);
-     return;
-  }
-}
-
-static void f_substring_get_int( INT32 args )
-{
-  struct substring *s = SS(fp->current_object);
-  int res;
-  unsigned char *p;
-  int x = sp[-1].u.integer;
-  if( x > s->len>>2 )
-    Pike_error("Index %d out of range.\n", x );
-
-  p = ((unsigned char *)s->s->str) + s->offset + x*4;
-  res = (p[0]<<24) | (p[1]<<16) | (p[2]<<8) | p[3];
-  push_int( res );
-}
-
-
-static void f_substring_get_uint( INT32 args )
-{
-  struct substring *s = SS(fp->current_object);
-  unsigned int res;
-  unsigned char *p;
-  int x = sp[-1].u.integer;
-  if( x > s->len>>2 )
-    Pike_error("Index %d out of range.\n", x );
-
-  p = ((unsigned char *)s->s->str) + s->offset + x*4;
-  res = (p[0]<<24) | (p[1]<<16) | (p[2]<<8) | p[3];
-  push_int64( res );
-}
-
-static void f_substring_get_ushort( INT32 args )
-{
-  struct substring *s = SS(fp->current_object);
-  unsigned short res;
-  unsigned char *p;
-  int x = sp[-1].u.integer;
-  if( x > s->len>>1 )
-    Pike_error("Index %d out of range.\n", x );
-
-  p = ((unsigned char *)s->s->str) + s->offset + x*2;
-  res = (p[2]<<8) | p[3];
-  push_int( res );
-}
-
-static void f_substring_get_short( INT32 args )
-{
-  struct substring *s = SS(fp->current_object);
-  short res;
-  unsigned char *p;
-  int x = sp[-1].u.integer;
-  if( x > s->len>>1 )
-    Pike_error("Index %d out of range.\n", x );
-
-  p = ((unsigned char *)s->s->str) + s->offset + x*2;
-  res = (p[2]<<8) | p[3];
-  push_int( res );
-}
-
-static void push_substring( struct pike_string *s, 
-                            ptrdiff_t offset,
-                            ptrdiff_t len )
-{
-  struct object *o = clone_object( substring_program, 0 );
-  struct substring *ss = SS(o);
-  ss->s = s;
-  s->refs++;
-  ss->offset = offset;
-  ss->len = len;
-  push_object( o );
-}
-
-static void free_substring(struct object *o)
-{
-  if( SS(fp->current_object)->s )
-  {
-    free_string( SS(fp->current_object)->s );
-    SS(fp->current_object)->s = 0;
-  }
-}
-
 
 typedef enum
 {
@@ -303,24 +147,23 @@ static unsigned int read_uint( struct buffer *from )
 {
   unsigned int res;
   if(from->len < 4)
-    Pike_error("Not enough space for 4 bytes (uint32)\n");
-  res = (from->str[0]<<24)|(from->str[1]<<16)|(from->str[2]<<8)|from->str[3];
+    error("Not enough space for 4 bytes (uint32)\n");
+  res = from->str[0]<<24|from->str[1]<<16|from->str[2]<<8|from->str[3];
   from->str += 4;
   from->len -= 4;
   return res;
 }
 
-static int xcf_read_int( struct buffer *from )
+static int read_int( struct buffer *from )
 {
   return (int)read_uint( from );
 }
 
-static char *read_data( struct buffer *from, size_t len )
+static char *read_data( struct buffer * from, unsigned int len )
 {
   char *res;
   if( from->len < len )
-    Pike_error("Not enough space for %lu bytes\n",
-	  DO_NOT_WARN((unsigned long)len));
+    error("Not enough space for %ud bytes\n", len);
   res = (char *)from->str;
   from->str += len;
   from->len -= len;
@@ -329,19 +172,18 @@ static char *read_data( struct buffer *from, size_t len )
 
 static struct buffer read_string( struct buffer *data )
 {
-  struct buffer res = *data;
-  res.len = xcf_read_int( data );
-  res.base_offset = (data->base_offset+(data->base_len-data->len));
+  struct buffer res;
+  res.len = read_int( data );
   res.str = (unsigned char *)read_data( data, res.len );
-  if(res.len > 0)  res.len--;  /* len includes ending \0 */
-  res.base_len = res.len;
+  if(res.len > 0) res.len--; /* len includes ending \0 */
   if(!res.str)
-    Pike_error("String read failed\n");
+    error("String read failed\n");
   return res;
 }
 
 static struct property read_property( struct buffer * data )
 {
+  int i;
   struct property res;
   res.type = read_uint( data );
   if(res.type == PROP_COLORMAP)
@@ -350,16 +192,10 @@ static struct property read_property( struct buffer * data )
     read_uint(data); /* bogus 'len'... */
     foo = read_uint( data );
     res.data.len = foo*3;
-    res.data.base_offset = data->base_offset+(data->base_len-data->len);
-    res.data.base_len = res.data.len;
     res.data.str = (unsigned char *)read_data( data,foo*3 );
-    res.data.s   = data->s;
   } else {
     res.data.len = read_uint( data );
-    res.data.base_offset = data->base_offset+(data->base_len-data->len);
-    res.data.base_len = res.data.len;
     res.data.str = (unsigned char *)read_data( data,res.data.len );
-    res.data.s   = data->s;
   }
   res.next = NULL;
   return res;
@@ -516,7 +352,7 @@ static struct level read_level( struct buffer *buff,
   ONERROR err;
   int offset;
   struct tile *last_tile = NULL;
-/*   int all_tiles_eq = 0; */
+  int all_tiles_eq = 0;
   MEMSET(&res, 0, sizeof(res));
   res.width = read_uint( buff );
   res.height = read_uint( buff );
@@ -533,9 +369,22 @@ static struct level read_level( struct buffer *buff,
       last_tile->next = tile;
     last_tile = tile;
     if(!res.first_tile)
+    {
       res.first_tile = tile;
+      if(offset2)
+        all_tiles_eq = offset2-offset;
+    }
+    if(all_tiles_eq && offset2 && offset2-offset != all_tiles_eq)
+      all_tiles_eq = 0;
+    if(all_tiles_eq)
+      ob.len = all_tiles_eq;
+    else if(offset2)
+      ob.len = offset2-offset;
+    else
+      ob.len = MINIMUM((TILE_WIDTH*TILE_HEIGHT*5),ob.len);
     tile->data = ob;
     tile->next = NULL;
+/* fprintf(stderr, "tile, o=%d; o2=%d; l=%d\n", offset, offset2,ob.len); */
     offset = offset2;
   }
   UNSET_ONERROR( err );
@@ -567,6 +416,7 @@ static struct layer_mask read_layer_mask( struct buffer *buff,
   ONERROR err;
   int offset;
   struct property tmp;
+  struct buffer ob;
 
   MEMSET(&res, 0, sizeof(res));
   res.width = read_uint( buff );
@@ -604,6 +454,7 @@ static struct channel read_channel( struct buffer *buff,
   ONERROR err;
   int offset;
   struct property tmp;
+  struct buffer ob;
 
   MEMSET(&res, 0, sizeof(res));
   res.width = read_uint( buff );
@@ -646,7 +497,7 @@ static struct layer read_layer( struct buffer *buff, struct buffer *initial )
   SET_ONERROR( err, free_layer, &res );
   res.width = read_uint( buff );
   res.height = read_uint( buff );
-  res.type = xcf_read_int( buff );
+  res.type = read_int( buff );
   res.name = read_string( buff );
 
 
@@ -662,8 +513,8 @@ static struct layer read_layer( struct buffer *buff, struct buffer *initial )
     }
   } while( tmp.type );
 
-  h_offset = xcf_read_int( buff );
-  lm_offset = xcf_read_int( buff );
+  h_offset = read_int( buff );
+  lm_offset = read_int( buff );
 
   if(lm_offset)
   {
@@ -698,7 +549,7 @@ static struct gimp_image read_image( struct buffer * data )
   MEMSET(&res, 0, sizeof(res));
   initial = *data;
   if(data->len < 34)
-    Pike_error("This is not an xcf file (to little data)\n");
+    error("This is not an xcf file (to little data)\n");
   if(!(data->str[0] == 'g' &&
        data->str[1] == 'i' &&
        data->str[2] == 'm' &&
@@ -706,15 +557,15 @@ static struct gimp_image read_image( struct buffer * data )
        data->str[4] == ' '))
   {
     if(strlen((char *)data->str) == 13)
-      Pike_error("This is not an xcf file (%s)\n", data->str);
+      error("This is not an xcf file (%s)\n", data->str);
     else
-      Pike_error("This is not an xcf file\n");
+      error("This is not an xcf file\n");
   }
   data->str+=14; data->len-=14;
 
   res.width = read_uint( data );
   res.height = read_uint( data );
-  res.type = xcf_read_int( data );
+  res.type = read_int( data );
 
   SET_ONERROR( err, free_image, &res );
 
@@ -765,8 +616,7 @@ static struct gimp_image read_image( struct buffer * data )
 
 static void push_buffer( struct buffer *b )
 {
-  push_substring( b->s, b->base_offset+(b->base_len-b->len), b->len );
-/* push_string( make_shared_binary_string( (char *)b->str, b->len ) );*/
+  push_string( make_shared_binary_string( (char *)b->str, b->len ) );
 }
 
 static void push_properties( struct property *p )
@@ -779,7 +629,7 @@ static void push_properties( struct property *p )
     f_aggregate_mapping( 4 );
     p = p->next;
   }
-  f_aggregate(DO_NOT_WARN((INT32)(sp - osp)));
+  f_aggregate( sp-osp );
 }
 
 static void push_tile( struct tile *t )
@@ -793,7 +643,7 @@ static void push_hierarchy( struct hierarchy * h )
   struct svalue *osp = sp, *tsp;
   if(h->level.width != h->width ||
      h->level.height != h->height)
-    Pike_error("Illegal hierarchy level sizes!\n");
+    error("Illegal hierarchy level sizes!\n");
 
   ref_push_string( s_width );  push_int( h->width );
   ref_push_string( s_height ); push_int( h->height );
@@ -806,8 +656,8 @@ static void push_hierarchy( struct hierarchy * h )
     push_tile( t );
     t=t->next;
   }
-  f_aggregate(DO_NOT_WARN((INT32)(sp - tsp)));
-  f_aggregate_mapping(DO_NOT_WARN((INT32)(sp - osp)));
+  f_aggregate( sp-tsp );
+  f_aggregate_mapping( sp-osp );
 }
 
 static void push_layer_mask(struct layer_mask *i)
@@ -820,7 +670,7 @@ static void push_layer_mask(struct layer_mask *i)
   ref_push_string( s_name );  push_buffer( &i->name );
   ref_push_string( s_image_data );
   push_hierarchy( &i->image_data );
-  f_aggregate_mapping(DO_NOT_WARN((INT32)(sp-osp)));
+  f_aggregate_mapping( sp-osp );
 }
 
 static void push_channel(struct channel *i)
@@ -833,7 +683,7 @@ static void push_channel(struct channel *i)
   ref_push_string( s_name );  push_buffer( &i->name );
   ref_push_string( s_image_data );
   push_hierarchy( &i->image_data );
-  f_aggregate_mapping(DO_NOT_WARN((INT32)(sp-osp)));
+  f_aggregate_mapping( sp-osp );
 }
 
 static void push_layer(struct layer *i)
@@ -852,7 +702,7 @@ static void push_layer(struct layer *i)
     ref_push_string( s_mask );
     push_layer_mask( i->mask );
   }
-  f_aggregate_mapping(DO_NOT_WARN((INT32)(sp - osp)));
+  f_aggregate_mapping( sp-osp );
 }
 
 
@@ -887,7 +737,7 @@ static void push_image( struct gimp_image *i )
     c = c->next;
   }
   f_aggregate( nitems );
-  f_aggregate_mapping(DO_NOT_WARN((INT32)(sp-osp)));
+  f_aggregate_mapping( sp-osp );
 }
 
 
@@ -899,14 +749,9 @@ static void image_xcf____decode( INT32 args )
   ONERROR err;
   get_all_args( "___decode", args, "%S", &s );
   if(args > 1)
-    Pike_error("Too many arguments to Image.XCF.___decode()\n");
-
-  b.s = s;
-  b.base_offset = 0;
-  b.base_len = s->len;
-  b.len = s->len;
-  b.str = (unsigned char *)s->str;
+    error("Too many arguments to Image.XCF.___decode()\n");
 
+  b.str = (unsigned char *)s->str; b.len = s->len;
   res = read_image( &b );
   SET_ONERROR( err, free_image, &res );
   push_image( &res );
@@ -917,7 +762,7 @@ static void image_xcf____decode( INT32 args )
 }
 
 
-static unsigned char read_char( struct buffer *from )
+unsigned char read_char( struct buffer *from )
 {
   unsigned char res = 0;
   if(from->len)
@@ -929,9 +774,74 @@ static unsigned char read_char( struct buffer *from )
   return res;
 }
 
+
+void image_xcf_f__rle_decode( INT32 args )
+{
+  struct pike_string *t;
+  struct buffer s;
+  struct buffer od, d;
+  int bpp, xsize, ysize, i;
+  get_all_args( "_rle_decode", args, "%S%d%d%d", &t, &bpp, &xsize, &ysize);
+
+  s.len = t->len;
+  s.str = (unsigned char *)t->str;
+  od.len = xsize*ysize*bpp;  /* Max and only size, really */
+  od.str = (unsigned char *)xalloc( xsize*ysize*bpp );
+  d = od;
+  for(i=0; i<bpp; i++)
+  {
+    int nelems = xsize*ysize;
+    int length;
+    while(nelems)
+    {
+      unsigned char val = read_char( &s );
+      if(!s.len)
+      {
+        break; /* Hm. This is actually rather fatal */
+      }
+      length = val;
+      if( length >= 128 )
+      {
+        length = 255-(length-1);
+        if (length == 128)
+          length = (read_char( &s )<<8) + read_char( &s );
+        nelems -= length;
+        while(length--)
+        {
+          if(d.len < 1)
+            break;
+          d.str[0] = read_char( &s );
+          d.str++;
+          d.len--;
+        }
+      } else {
+        length++;
+        if(length == 128)
+          length = (read_char( &s )<<8) + read_char( &s );
+        nelems -= length;
+        val = read_char( &s );
+        while(length--)
+        {
+          if(d.len < 1)
+            break;
+          d.str[0] = val;
+          d.str++;
+          d.len--;
+        }
+      }
+    }
+  }
+
+  pop_n_elems(args);
+    /* fprintf(stderr, "%d bytes of source data used out of %d bytes\n" */
+    /*         "%d bytes decoded data generated\n",  */
+    /*         t->len-s.len,t->len,od.len); */
+  push_string(make_shared_binary_string((char *)od.str,od.len));
+  free(od.str);
+}
+
 /*
 **! method array(object) decode_layers( string data )
-**! method array(object) decode_layers( string data, mapping options )
 **!     Decodes a XCF image to an array of Image.Layer objects
 **!
 **!     The layer object have the following extra variables (to be queried
@@ -939,9 +849,6 @@ static unsigned char read_char( struct buffer *from )
 **!
 **!      image_xres, image_yres, image_colormap, image_guides, image_parasites,
 **!      name, parasites, visible, active
-**!
-**!	Takes the same argument mapping as <ref>_decode</ref>,
-**!	note especially "draw_all_layers":1.
 */
 
 /*
@@ -1181,38 +1088,29 @@ static unsigned char read_char( struct buffer *from )
 
 void image_xcf_f__decode_tiles( INT32 args )
 {
+  extern void check_signals();
   struct object *io,*ao, *cmapo;
   struct array *tiles;
-  struct image *i=NULL, *a=NULL;
+  struct image *i, *a=NULL;
   struct neo_colortable *cmap = NULL;
-  INT32 rxs, rys;
   rgb_group *colortable=NULL;
-  rgb_group pix = {0,0,0};
-  rgb_group apix= {255,255,255}; /* avoid may use uninitialized warnings */
-
-  INT_TYPE rle, bpp, span, shrink;
+  int rle, bpp, span;
   unsigned int l, x=0, y=0, cx, cy;
-  get_all_args( "_decode_tiles", args, "%o%O%a%i%i%O%i%d%d",
-                &io, &ao, &tiles, &rle, &bpp, &cmapo, &shrink, &rxs, &rys);
-
-
+  get_all_args( "_decode_tiles", args, "%o%O%a%i%i%O",
+                &io, &ao, &tiles, &rle, &bpp, &cmapo);
   if( !(i = (struct image *)get_storage( io, image_program )))
-    Pike_error("Wrong type object argument 1 (image)\n");
-
+    error("Wrong type object argument 1 (image)\n");
   if(ao && !(a = (struct image *)get_storage( ao, image_program )))
-    Pike_error("Wrong type object argument 2 (image)\n");
-
+    error("Wrong type object argument 2 (image)\n");
   if( cmapo &&
       !(cmap=(struct neo_colortable *)get_storage(cmapo,
                                                   image_colortable_program)))
-    Pike_error("Wrong type object argument 4 (colortable)\n");
-
+    error("Wrong type object argument 4 (colortable)\n");
   for(l=0; l<(unsigned int)tiles->size; l++)
-    if(tiles->item[l].type != T_OBJECT)
-      Pike_error("Wrong type array argument 3 (tiles)\n");
-
-  if(a && ((i->xsize != a->xsize) || (i->ysize != a->ysize)))
-    Pike_error("Image and alpha objects are not identically sized.\n");
+    if(tiles->item[l].type != T_STRING)
+      error("Wrong type array argument 3 (tiles)\n");
+  if(a && (i->xsize != a->xsize ||i->ysize != a->ysize))
+    error("Image and alpha objects are not identically sized.\n");
 
   if(cmap)
   {
@@ -1220,191 +1118,116 @@ void image_xcf_f__decode_tiles( INT32 args )
     image_colortable_write_rgb( cmap, (unsigned char *)colortable );
   }
 
-  x=y=0;
 
-  THREADS_ALLOW();
+/*   switch(bpp) */
+/*   { */
+/*    case 1: */
+/*    case 3: */
+/*      if(ao) */
+/*      { */
+/*        destruct( ao ); */
+/*        a=0; */
+/*        ao=0; */
+/*      } */
+/*      break; */
+/*   } */
+
+  x=y=0;
   for(l=0; l<(unsigned)tiles->size; l++)
   {
-    struct object *to = tiles->item[l].u.object;
-    struct substring *tile_ss = SS(to);
-    struct buffer tile;
-    char *df = 0;
+    struct pike_string *tile = tiles->item[l].u.string;
     unsigned int eheight, ewidth;
     unsigned char *s;
+    ewidth = MINIMUM(TILE_WIDTH, i->xsize-x);
+    eheight = MINIMUM(TILE_HEIGHT, i->ysize-y);
+    add_ref(tile);
 
-    if(!tile_ss)
-      continue;
-
-    tile.str = (unsigned char *)(tile_ss->s->str + tile_ss->offset);
-    tile.len = tile_ss->len;
-
-    ewidth = MINIMUM(TILE_WIDTH, (rxs-x));
-    eheight = MINIMUM(TILE_HEIGHT, (rys-y));
+/*     fprintf(stderr, "       tile %d/%d [%dx%d]  %dbpp      \n", */
+/*             l+1, tiles->size, ewidth, eheight,bpp); */
 
     if(rle)
     {
-      struct buffer s = tile, od, d;
-      int i;
-      od.len = eheight*ewidth*bpp;  /* Max and only size, really */
-      df = (char *)(od.str = (unsigned char *)xalloc( eheight*ewidth*bpp+1 ));
-      d = od;
+      push_string( tile );
+      push_int( bpp );
+      push_int( ewidth );
+      push_int( eheight );
+      image_xcf_f__rle_decode( 4 );
+      tile = (struct pike_string *)debug_malloc_pass(sp[-1].u.string);
+      if(sp[-1].type != T_STRING)
+        fatal("Internal disaster in XCF module\n");
+      sp--;
+    }
 
-      for(i=0; i<bpp; i++)
+    if( (unsigned)(tile->len) < (unsigned)(eheight * ewidth * bpp ))
+      error("Too small tile, was %d bytes, I really need %d\n",
+            tile->len, eheight*ewidth * bpp);
+
+    s = (unsigned char *)tile->str;
+
+    check_signals(); /* Allow ^C */
+
+    for(cy=0; cy<eheight; cy++)
+    {
+      for(cx=0; cx<ewidth; cx++)
       {
-        int nelems = ewidth*eheight;
-        int length;
-        while(nelems)
+        rgb_group pix;
+        rgb_group apix;
+        int ind = (cx+cy*ewidth);
+
+        if(rle)
+          span = ewidth*eheight;
+        else
+          span = 1;
+        if(cx+x > (unsigned)i->xsize)  continue;
+        if(cy+y > (unsigned)i->ysize)  continue;
+
+        switch( bpp )
         {
-          unsigned char val = read_char( &s );
-          if(!s.len)
-          {
-            break; /* Hm. This is actually rather fatal */
-          }
-          length = val;
-          if( length >= 128 )
-          {
-            length = 255-(length-1);
-            if (length == 128)
-              length = (read_char( &s )<<8) + read_char( &s );
-            nelems -= length;
-            while(length--)
-            {
-              if(d.len < 1)
-                break;
-              d.str[0] = read_char( &s );
-              d.str++;
-              d.len--;
-            }
-          } else {
-            length++;
-            if(length == 128)
-              length = (read_char( &s )<<8) + read_char( &s );
-            nelems -= length;
-            val = read_char( &s );
-            while(length--)
-            {
-              if(d.len < 1)
-                break;
-              d.str[0] = val;
-              d.str++;
-              d.len--;
-            }
-          }
+         case 1: /* indexed or grey */
+           if(colortable)
+             pix = colortable[s[ind]];
+           else
+             pix.r = pix.g = pix.b = s[ind];
+           break;
+         case 2: /* indexed or grey with alpha */
+           if(colortable)
+             pix = colortable[s[ind]];
+           else
+             pix.r = pix.g = pix.b = s[ind];
+           apix.r = apix.g = apix.b = s[ind+span];
+           break;
+         case 3: /* rgb */
+           pix.r = s[ind];
+           pix.g = s[ind+span];
+           pix.b = s[ind+span*2];
+           break;
+         case 4: /* rgb with alpha */
+           pix.r = s[ind];
+           pix.g = s[ind+span*1];
+           pix.b = s[ind+span*2];
+           apix.r = apix.g = apix.b = s[ind+span*3];
+           break;
         }
+        ind = i->xsize*(cy+y)+(cx+x);
+        i->img[ind] = pix;
+        if(a) a->img[ind] = apix;
       }
-      tile = od;
-    }
-
-    if( (size_t)(tile.len) < (size_t)(eheight * ewidth * bpp ))
-    {
-      if( df ) free( df ); df=0;
-      continue;
-    }
-    s = (unsigned char *)tile.str;
-
-
-#define LOOP_INIT() {\
-    int ix, iy=y/shrink;                                        \
-    for(cy=0; cy<eheight; cy+=shrink,iy=((cy+y)/shrink))        \
-    {                                                           \
-      rgb_group *ip, *ap = NULL;                                \
-      int ind=cy*ewidth;                                        \
-      int ds = 0;                                               \
-      if(iy >= i->ysize)  continue;                             \
-      ix = x/shrink;                                            \
-      ip = i->img + (i->xsize*iy);                              \
-      if( a ) ap = a->img + (i->xsize*iy);                      \
-      for(cx=0; cx<ewidth; cx+=shrink,ind+=shrink,ix++)         \
-      {                                                         \
-        if(ix >= i->xsize)  continue
-
-#define LOOP_EXIT()                                             \
-        ip[ix] = pix;                                           \
-      }                                                         \
-    }}
-
-    if(rle)
-      span = ewidth*eheight;
-    else
-      span = 1;
-
-    switch(bpp)
-    {
-     case 1: /* indexed or grey */
-       if(colortable)
-       {
-         LOOP_INIT();
-         pix = colortable[s[ind]];
-         LOOP_EXIT();
-       } 
-       else
-       {
-         LOOP_INIT();
-         pix.r = pix.g = pix.b = s[ind];
-         LOOP_EXIT();
-       }
-       break;
-     case 2: /* indexed or grey with alpha */
-       if(colortable)
-       {
-         LOOP_INIT();
-         pix = colortable[ s[ ind ] ];
-	 if( a )
-	 {
-	   apix.r = apix.g = apix.b = s[ind+span];
-	   ap[ix] = apix;
-	 }
-         LOOP_EXIT();
-       }
-       else
-       {
-         LOOP_INIT();
-         pix.r  =  pix.g =  pix.b = s[ind];
-	 if( a )
-	 {
-	   apix.r = apix.g = apix.b = s[ind+span];
-	   ap[ix] = apix;
-	 }
-         LOOP_EXIT();
-       }
-       break;
-     case 3: /* rgb */
-       LOOP_INIT();
-       pix.r = s[ind];
-       pix.g = s[ind+span];
-       pix.b = s[ind+span*2];
-       LOOP_EXIT();
-       break;
-     case 4: /* rgba */
-       LOOP_INIT();
-       pix.r = s[ind];
-       pix.g = s[ind+span];
-       pix.b = s[ind+span*2];
-       if(a)
-       {
-	 apix.r = apix.b = apix.g = s[ind+span*3];
-	 ap[ix] = apix;
-       }
-       LOOP_EXIT();
-       break;
-    }
-
-    if( df )
-    { 
-      free(df); 
-      df=0; 
     }
     x += TILE_WIDTH;
-
-    if( (int)x >= (int)rxs )
+    if( x >= (unsigned)i->xsize )
     {
       x = 0;
       y += TILE_HEIGHT;
     }
+    if(y>=(unsigned)i->ysize)
+    {
+      free_string(tile);
+      if(colortable) free( colortable );
+      return;
+    }
+    free_string(tile);
   }
-  THREADS_DISALLOW();
-  if(colortable) 
-    free( colortable );
+  if(colortable) free( colortable );
 
   pop_n_elems(args);
   push_int(0);
@@ -1493,20 +1316,6 @@ void init_image_xcf()
 #define STRING(X) s_##X = make_shared_binary_string(#X,sizeof( #X )-sizeof(""))
 #include "xcf_constant_strings.h"
 #undef STRING
-
-  start_new_program();
-  ADD_STORAGE( struct substring );
-  add_function("cast", f_substring_cast, "function(string:mixed)",0);
-  add_function("`[]", f_substring_index, "function(int:int)",0);
-  add_function("get_short", f_substring_get_short, "function(int:int)", 0 );
-  add_function("get_ushort", f_substring_get_ushort, "function(int:int)", 0 );
-  add_function("get_int", f_substring_get_int, "function(int:int)", 0 );
-  add_function("get_uint", f_substring_get_uint, "function(int:int)", 0 );
-  add_function("_sprintf",f_substring__sprintf,
-               "function(int,mapping:mixed)", 0);
-/*   set_init_callback(init_substring); */
-  set_exit_callback(free_substring);
-  substring_program = end_program();  
 }
 
 
@@ -1515,5 +1324,4 @@ void exit_image_xcf()
 #define STRING(X) free_string(s_##X)
 #include "xcf_constant_strings.h"
 #undef STRING
-  free_program( substring_program );
 }
