@@ -1,7 +1,7 @@
 /*
 **! module Image
 **! note
-**!	$Id: poly.c,v 1.12 2000/12/05 21:08:26 per Exp $
+**!	$Id: poly.c,v 1.3 1999/07/28 09:26:20 mirar Exp $
 **! class Poly
 **!
 */
@@ -18,7 +18,7 @@ another?
 
 #include "global.h"
 
-RCSID("$Id: poly.c,v 1.12 2000/12/05 21:08:26 per Exp $");
+RCSID("$Id: poly.c,v 1.3 1999/07/28 09:26:20 mirar Exp $");
 
 #include "image_machine.h"
 
@@ -38,15 +38,12 @@ RCSID("$Id: poly.c,v 1.12 2000/12/05 21:08:26 per Exp $");
 #include "operators.h"
 #include "module_support.h"
 #include "opcodes.h"
-#include "pike_error.h"
+#include "error.h"
 
 #include "image.h"
 #include "colortable.h"
 #include "initstuff.h"
 
-/* This must be included last! */
-#include "module_magic.h"
-
 #ifdef PFLOAT
 #undef PFLOAT
 #endif
@@ -55,8 +52,8 @@ RCSID("$Id: poly.c,v 1.12 2000/12/05 21:08:26 per Exp $");
 #ifdef THIS
 #undef THIS
 #endif
-#define THIS ((struct poly *)(Pike_fp->current_storage))
-#define THISOBJ (Pike_fp->current_object)
+#define THIS ((struct poly *)(fp->current_storage))
+#define THISOBJ (fp->current_object)
 
 struct poly
 {
@@ -214,12 +211,11 @@ void vertices_dump(struct poly *this,char *desc)
 	 fprintf(stderr,", down");
 	 while (l)
 	 {
-	    fprintf(stderr," %ld:%g,%g",
-		    DO_NOT_WARN((long)(l->down - this->vertex)),
-		    l->down->x, l->down->y);
+	    fprintf(stderr," %d:%g,%g",
+		    l->down-this->vertex,l->down->x,l->down->y);
 	    if (l->up!=this->vertex+i)
-	       fprintf(stderr,"(wrong up: %ld)",
-		       DO_NOT_WARN((long)(l->up - this->vertex + i)));
+	       fprintf(stderr,"(wrong up: %d)",
+		       l->up-this->vertex+i);
 	    l=l->nextdown;
 	 }
       }
@@ -228,12 +224,10 @@ void vertices_dump(struct poly *this,char *desc)
 	 fprintf(stderr,", up");
 	 while (l)
 	 {
-	    fprintf(stderr, " %ld:%g,%g",
-		    DO_NOT_WARN((long)(l->up - this->vertex)),
-		    l->up->x, l->up->y);
+	    fprintf(stderr," %d:%g,%g",l->up-this->vertex,l->up->x,l->up->y);
 	    if (l->down!=this->vertex+i)
-	       fprintf(stderr,"(wrong down: %ld)",
-		       DO_NOT_WARN((long)(l->down - this->vertex + i)));
+	       fprintf(stderr,"(wrong down: %d)",
+		       l->down-this->vertex+i);
 	    l=l->nextup;
 	 }
       }
@@ -373,12 +367,11 @@ static struct vertex *vertex_find_or_insert(struct poly *p,
       j=(a+b)/2;
       if (y<p->vertex[j].y) b=j-1;
       else if (y>p->vertex[j].y) a=j+1;
-      else if (y==p->vertex[j].y) {
+      else if (y==p->vertex[j].y) 
 	 if (x<p->vertex[j].x) b=j-1;
 	 else if (x>p->vertex[j].x) a=j+1;
 	 else
 	    return p->vertex+j;
-      }
    }
    
    if (p->nallocatedvertex==p->nvertex)
@@ -416,7 +409,7 @@ static void mend_crossed_lines(struct poly *p)
    struct line **active,*l;
    int nactive;
    int from,i,j=0;
-   double f;
+   float f;
    int new;
    PFLOAT x,y;
 
@@ -457,7 +450,7 @@ static void mend_crossed_lines(struct poly *p)
 	       v=vertex_find_or_insert(p,x,y);
 
 	       if (v-p->vertex<from) 
-		  Pike_error("internal error: unexpected v-p->vertex<from\n");
+		  error("internal error: unexpected v-p->vertex<from\n");
 
 	       v1=active[i]->down;
 	       v2=active[new]->down;
@@ -564,7 +557,7 @@ static void image_poly_create(INT32 args)
    struct array *a;
 
    if (THIS->nvertex || THIS->nline) 
-      Pike_error("Poly: create called on initialised object\n");
+      error("Poly: create called on initialised object\n");
 
    /* this is to get the correct error message */
    for (i=0; i<args; i++)
@@ -708,17 +701,17 @@ static void image_poly_cast(INT32 args)
 		  {
 		     struct line *lk;
 
-		     fprintf(stderr," %ld %ld:%g,%g - ",
-			     DO_NOT_WARN((long)(l - THIS->line)),
-			     DO_NOT_WARN((long)(v - THIS->vertex)),
+		     fprintf(stderr," %d %d:%g,%g - ",
+			     l-THIS->line,
+			     v-THIS->vertex,
 			     v->x,v->y);
 #endif
 
 		     v=down?l->down:l->up;
 
 #if 1
-		     fprintf(stderr,"%ld:%g,%g: ",
-			     DO_NOT_WARN((long)(v - THIS->vertex)),
+		     fprintf(stderr,"%d:%g,%g: ",
+			     v-THIS->vertex,
 			     v->x,v->y);
 
 		     if ((lk=v->firstdown))
@@ -726,9 +719,8 @@ static void image_poly_cast(INT32 args)
 			fprintf(stderr,", down");
 			while (lk)
 			{
-			   fprintf(stderr," %ld[%c]:%g,%g",
-				   DO_NOT_WARN((long)(lk - THIS->line)),
-				   mark[lk-THIS->line]?'x':' ',
+			   fprintf(stderr," %d[%c]:%g,%g",
+				   lk-THIS->line,mark[lk-THIS->line]?'x':' ',
 				   lk->down->x,lk->down->y);
 			   lk=lk->nextdown;
 			}
@@ -738,9 +730,8 @@ static void image_poly_cast(INT32 args)
 			fprintf(stderr,", up");
 			while (lk)
 			{
-			   fprintf(stderr," %ld[%c]:%g,%g",
-				   DO_NOT_WARN((long)(lk - THIS->line)),
-				   mark[lk-THIS->line]?'x':' ',
+			   fprintf(stderr," %d[%c]:%g,%g",
+				   lk-THIS->line,mark[lk-THIS->line]?'x':' ',
 				   lk->up->x,l->up->y);
 			   lk=lk->nextup;
 			}
@@ -777,7 +768,7 @@ static void image_poly_cast(INT32 args)
 	 free(mark);
 
 	 if (ni!=THIS->nline) 
-	    Pike_error("Poly: internal error; ni!=nline\n");
+	    error("Poly: internal error; ni!=nline\n");
 
 	 f_aggregate(na);
 
