@@ -1,11 +1,6 @@
 /*
-|| This file is part of Pike. For copyright information see COPYRIGHT.
-|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
-|| for more information.
-|| $Id: mime.c,v 1.36 2002/11/25 15:35:32 marcus Exp $
-*/
-
-/*
+ * $Id: mime.c,v 1.21 1999/08/17 18:38:32 marcus Exp $
+ *
  * RFC1521 functionality for Pike
  *
  * Marcus Comstedt 1996-1999
@@ -15,15 +10,14 @@
 
 #include "config.h"
 
-RCSID("$Id: mime.c,v 1.36 2002/11/25 15:35:32 marcus Exp $");
-#include "module.h"
+RCSID("$Id: mime.c,v 1.21 1999/08/17 18:38:32 marcus Exp $");
 #include "stralloc.h"
 #include "pike_macros.h"
 #include "object.h"
 #include "program.h"
 #include "interpret.h"
 #include "builtin_functions.h"
-#include "pike_error.h"
+#include "error.h"
 
 #ifdef __CHAR_UNSIGNED__
 #define SIGNED signed
@@ -32,8 +26,6 @@ RCSID("$Id: mime.c,v 1.36 2002/11/25 15:35:32 marcus Exp $");
 #endif
 
 
-#define sp Pike_sp
-
 /** Forward declarations of functions implementing Pike functions **/
 
 static void f_decode_base64( INT32 args );
@@ -71,17 +63,12 @@ unsigned char rfc822ctype[1<<CHAR_BIT];
 
 /** Externally available functions **/
 
-/*! @module MIME
- */
-
 /* Initialize and start module */
 
-PIKE_MODULE_INIT
+void pike_module_init( void )
 {
   int i;
 
-  Pike_compiler->new_program->id = PROG_MODULE_MIME_ID;
-
   /* Init reverse base64 mapping */
   memset( base64rtab, -1, sizeof(base64rtab) );
   for (i = 0; i < 64; i++)
@@ -142,29 +129,23 @@ PIKE_MODULE_INIT
 
 /* Restore and exit module */
 
-PIKE_MODULE_EXIT
+void pike_module_exit( void )
 {
 }
 
 
 /** Functions implementing Pike functions **/
 
-/*! @decl string decode_base64(string encoded_data)
- *!
- *! This function decodes data encoded using the @tt{base64@}
- *! transfer encoding.
- *!
- *! @seealso
- *! @[MIME.encode_base64()], @[MIME.decode()]
- */
+/* MIME.decode_base64() */
+
 static void f_decode_base64( INT32 args )
 {
   if(args != 1)
-    Pike_error( "Wrong number of arguments to MIME.decode_base64()\n" );
+    error( "Wrong number of arguments to MIME.decode_base64()\n" );
   else if (sp[-1].type != T_STRING)
-    Pike_error( "Wrong type of argument to MIME.decode_base64()\n" );
+    error( "Wrong type of argument to MIME.decode_base64()\n" );
   else if (sp[-1].u.string->size_shift != 0)
-    Pike_error( "Char out of range for MIME.decode_base64()\n" );
+    error( "Char out of range for MIME.decode_base64()\n" );
   else {
 
     /* Decode the string in sp[-1].u.string.  Any whitespace etc
@@ -174,8 +155,7 @@ static void f_decode_base64( INT32 args )
 
     struct string_builder buf;
     SIGNED char *src;
-    ptrdiff_t cnt;
-    INT32 d = 1;
+    INT32 cnt, d = 1;
     int pads = 0;
 
     init_string_builder( &buf, 0 );
@@ -215,7 +195,7 @@ static void f_decode_base64( INT32 args )
 /*  Convenience function for encode_base64();  Encode groups*3 bytes from
  *  *srcp into groups*4 bytes at *destp.
  */
-static int do_b64_encode( ptrdiff_t groups, unsigned char **srcp, char **destp,
+static int do_b64_encode( INT32 groups, unsigned char **srcp, char **destp,
 			  int insert_crlf )
 {
   unsigned char *src = *srcp;
@@ -246,32 +226,24 @@ static int do_b64_encode( ptrdiff_t groups, unsigned char **srcp, char **destp,
   return g;
 }
 
-/*! @decl string encode_base64(string data, void|int no_linebreaks)
- *!
- *! This function encodes data using the @tt{base64@} transfer encoding.
- *!
- *! If a nonzero value is passed as @[no_linebreaks], the result string
- *! will not contain any linebreaks.
- *!
- *! @seealso
- *! @[MIME.decode_base64()], @[MIME.encode()]
- */
+/* MIME.encode_base64() */
+
 static void f_encode_base64( INT32 args )
 {
   if(args != 1 && args != 2)
-    Pike_error( "Wrong number of arguments to MIME.encode_base64()\n" );
+    error( "Wrong number of arguments to MIME.encode_base64()\n" );
   else if(sp[-args].type != T_STRING)
-    Pike_error( "Wrong type of argument to MIME.encode_base64()\n" );
+    error( "Wrong type of argument to MIME.encode_base64()\n" );
   else if (sp[-args].u.string->size_shift != 0)
-    Pike_error( "Char out of range for MIME.encode_base64()\n" );
+    error( "Char out of range for MIME.encode_base64()\n" );
   else {
 
     /* Encode the string in sp[-args].u.string.  First, we need to know
        the number of 24 bit groups in the input, and the number of
        bytes actually present in the last group. */
 
-    ptrdiff_t groups = (sp[-args].u.string->len+2)/3;
-    ptrdiff_t last = (sp[-args].u.string->len-1)%3+1;
+    INT32 groups = (sp[-args].u.string->len+2)/3;
+    int last = (sp[-args].u.string->len-1)%3+1;
 
     int insert_crlf = !(args == 2 && sp[-1].type == T_INT &&
 			sp[-1].u.integer != 0);
@@ -314,22 +286,16 @@ static void f_encode_base64( INT32 args )
   }
 }
 
-/*! @decl string decode_qp(string encoded_data)
- *!
- *! This function decodes data encoded using the @tt{quoted-printable@}
- *! (a.k.a. quoted-unreadable) transfer encoding.
- *!
- *! @seealso
- *! @[MIME.encode_qp()], @[MIME.decode()]
- */
+/* MIME.decode_qp() */
+
 static void f_decode_qp( INT32 args )
 {
   if(args != 1)
-    Pike_error( "Wrong number of arguments to MIME.decode_qp()\n" );
+    error( "Wrong number of arguments to MIME.decode_qp()\n" );
   else if(sp[-1].type != T_STRING)
-    Pike_error( "Wrong type of argument to MIME.decode_qp()\n" );
+    error( "Wrong type of argument to MIME.decode_qp()\n" );
   else if (sp[-1].u.string->size_shift != 0)
-    Pike_error( "Char out of range for MIME.decode_qp()\n" );
+    error( "Char out of range for MIME.decode_qp()\n" );
   else {
 
     /* Decode the string in sp[-1].u.string.  We have absolutely no idea
@@ -338,7 +304,7 @@ static void f_decode_qp( INT32 args )
 
     struct string_builder buf;
     SIGNED char *src;
-    ptrdiff_t cnt;
+    INT32 cnt;
 
     init_string_builder(&buf, 0);
 
@@ -373,29 +339,16 @@ static void f_decode_qp( INT32 args )
   }
 }
 
-/*! @decl string encode_qp(string data, void|int no_linebreaks)
- *!
- *! This function encodes data using the @tt{quoted-printable@}
- *! (a.k.a. quoted-unreadable) transfer encoding.
- *!
- *! If a nonzero value is passed as @[no_linebreaks], the result
- *! string will not contain any linebreaks.
- *!
- *! @note
- *! Please do not use this function.  QP is evil, and there's no
- *! excuse for using it.
- *!
- *! @seealso
- *! @[MIME.decode_qp()], @[MIME.encode()]
- */
+/* MIME.encode_qp() */
+
 static void f_encode_qp( INT32 args )
 {
   if (args != 1 && args != 2)
-    Pike_error( "Wrong number of arguments to MIME.encode_qp()\n" );
+    error( "Wrong number of arguments to MIME.encode_qp()\n" );
   else if (sp[-args].type != T_STRING)
-    Pike_error( "Wrong type of argument to MIME.encode_qp()\n" );
+    error( "Wrong type of argument to MIME.encode_qp()\n" );
   else if (sp[-args].u.string->size_shift != 0)
-    Pike_error( "Char out of range for MIME.encode_qp()\n" );
+    error( "Char out of range for MIME.encode_qp()\n" );
   else {
 
     /* Encode the string in sp[-args].u.string.  We don't know how
@@ -404,7 +357,7 @@ static void f_encode_qp( INT32 args )
 
     struct string_builder buf;
     unsigned char *src = (unsigned char *)sp[-args].u.string->str;
-    ptrdiff_t cnt;
+    INT32 cnt;
     int col = 0;
     int insert_crlf = !(args == 2 && sp[-1].type == T_INT &&
 			sp[-1].u.integer != 0);
@@ -440,22 +393,14 @@ static void f_encode_qp( INT32 args )
 
 /* MIME.decode_uue() */
 
-/*! @decl string decode_uue(string encoded_data)
- *!
- *! This function decodes data encoded using the @tt{x-uue@} transfer encoding.
- *! It can also be used to decode generic UUEncoded files.
- *!
- *! @seealso
- *! @[MIME.encode_uue()], @[MIME.decode()]
- */
 static void f_decode_uue( INT32 args )
 {
   if (args != 1)
-    Pike_error( "Wrong number of arguments to MIME.decode_uue()\n" );
+    error( "Wrong number of arguments to MIME.decode_uue()\n" );
   else if(sp[-1].type != T_STRING)
-    Pike_error( "Wrong type of argument to MIME.decode_uue()\n" );
+    error( "Wrong type of argument to MIME.decode_uue()\n" );
   else if (sp[-1].u.string->size_shift != 0)
-    Pike_error( "Char out of range for MIME.decode_uue()\n" );
+    error( "Char out of range for MIME.decode_uue()\n" );
   else {
 
     /* Decode string in sp[-1].u.string.  This is done much like in
@@ -463,7 +408,7 @@ static void f_decode_uue( INT32 args )
 
     struct string_builder buf;
     char *src;
-    ptrdiff_t cnt;
+    INT32 cnt;
 
     init_string_builder( &buf, 0 );
 
@@ -537,24 +482,22 @@ static void f_decode_uue( INT32 args )
 /*  Convenience function for encode_uue();  Encode groups*3 bytes from
  *  *srcp into groups*4 bytes at *destp, and reserve space for last more.
  */
-static void do_uue_encode(ptrdiff_t groups, unsigned char **srcp, char **destp,
-			  ptrdiff_t last )
+static void do_uue_encode( INT32 groups, unsigned char **srcp, char **destp,
+			   INT32 last )
 {
   unsigned char *src = *srcp;
   char *dest = *destp;
 
   while (groups || last) {
     /* A single line can hold at most 15 groups */
-    ptrdiff_t g = (groups >= 15? 15 : groups);
+    int g = (groups >= 15? 15 : groups);
 
     if (g<15) {
       /* The line isn't filled completely.  Add space for the "last" bytes */
-      *dest++ = ' ' +
-	DO_NOT_WARN((char)(3*g + last));
+      *dest++ = ' ' + (3*g + last);
       last = 0;
     } else
-      *dest++ = ' ' + 
-	DO_NOT_WARN((char)(3*g));
+      *dest++ = ' ' + (3*g);
 
     groups -= g;
 
@@ -584,30 +527,18 @@ static void do_uue_encode(ptrdiff_t groups, unsigned char **srcp, char **destp,
 
 /* MIME.encode_uue() */
 
-/*! @decl string encode_uue(string encoded_data,  void|string filename)
- *!
- *! This function encodes data using the @tt{x-uue@} transfer encoding.
- *!
- *! The optional argument @[filename] specifies an advisory filename to include
- *! in the encoded data, for extraction purposes.
- *!
- *! This function can also be used to produce generic UUEncoded files.
- *!
- *! @seealso
- *! @[MIME.decode_uue()], @[MIME.encode()]
- */
 static void f_encode_uue( INT32 args )
 {
   if (args != 1 && args != 2)
-    Pike_error( "Wrong number of arguments to MIME.encode_uue()\n" );
+    error( "Wrong number of arguments to MIME.encode_uue()\n" );
   else if (sp[-args].type != T_STRING ||
 	   (args == 2 && sp[-1].type != T_VOID && sp[-1].type != T_STRING &&
 	    sp[-1].type != T_INT))
-    Pike_error( "Wrong type of argument to MIME.encode_uue()\n" );
+    error( "Wrong type of argument to MIME.encode_uue()\n" );
   else if (sp[-args].u.string->size_shift != 0 ||
 	   (args == 2 && sp[-1].type == T_STRING &&
 	    sp[-1].u.string->size_shift != 0))
-    Pike_error( "Char out of range for MIME.encode_uue()\n" );
+    error( "Char out of range for MIME.encode_uue()\n" );
   else {
 
     /* Encode string in sp[-args].u.string.  If args == 2, there may be
@@ -618,8 +549,8 @@ static void f_encode_uue( INT32 args )
     struct pike_string *str;
     unsigned char *src = (unsigned char *) sp[-args].u.string->str;
     /* Calculate number of 24 bit groups, and actual # of bytes in last grp */
-    ptrdiff_t groups = (sp[-args].u.string->len + 2)/3;
-    ptrdiff_t last= (sp[-args].u.string->len - 1)%3 + 1;
+    INT32 groups = (sp[-args].u.string->len + 2)/3;
+    int last= (sp[-args].u.string->len - 1)%3 + 1;
 
     /* Get the filename if provided */
     if (args == 2 && sp[-1].type == T_STRING)
@@ -682,6 +613,7 @@ static void f_encode_uue( INT32 args )
   }
 }
 
+/* MIME.tokenize() */
 
 static void low_tokenize( INT32 args, int mode )
 {
@@ -693,8 +625,7 @@ static void low_tokenize( INT32 args, int mode )
   unsigned char *src = (unsigned char *)sp[-args].u.string->str;
   struct array *arr;
   struct pike_string *str;
-  ptrdiff_t cnt = sp[-args].u.string->len;
-  INT32 n = 0, l, e, d;
+  INT32 cnt = sp[-args].u.string->len, n = 0, l, e, d;
   char *p;
 
   while (cnt>0)
@@ -878,7 +809,7 @@ static void low_tokenize( INT32 args, int mode )
 	cnt = 0;
 	break;
       }
-      Pike_error( "Invalid character in header field\n" );
+      error( "Invalid character in header field\n" );
     }
 
   /* Create the resulting array and push it */
@@ -887,99 +818,30 @@ static void low_tokenize( INT32 args, int mode )
   push_array( arr );
 }
 
-/*! @decl array(string|int) tokenize(string header)
- *!
- *! A structured header field, as specified by RFC822, is constructed from
- *! a sequence of lexical elements.
- *!
- *! These are:
- *! @dl
- *!   @item
- *!     individual special characters
- *!   @item
- *!     quoted-strings
- *!   @item
- *!     domain-literals
- *!   @item
- *!     comments
- *!   @item
- *!     atoms
- *! @enddl
- *!
- *! This function will analyze a string containing the header value,
- *! and produce an array containing the lexical elements.
- *!
- *! Individual special characters will be returned as characters (i.e.
- *! @tt{int@}s).
- *!
- *! Quoted-strings, domain-literals and atoms will be decoded and returned
- *! as strings.
- *!
- *! Comments are not returned in the array at all.
- *!
- *! @note
- *! As domain-literals are returned as strings, there is no way to tell the
- *! domain-literal @tt{[127.0.0.1]@} from the quoted-string
- *! @tt{"[127.0.0.1]"@}. Hopefully this won't cause any problems.
- *! Domain-literals are used seldom, if at all, anyway...
- *! 
- *! The set of special-characters is the one specified in RFC1521
- *! (i.e. @tt{"<", ">", "@@", ",", ";", ":", "\", "/", "?", "="@}),
- *! and not the set specified in RFC822.
- *!
- *! @seealso
- *!   @[MIME.quote()], @[tokenize_labled()],
- *!   @[decode_words_tokenized_remapped()].
- */
 static void f_tokenize( INT32 args )
 {
   if (args != 1)
-    Pike_error( "Wrong number of arguments to MIME.tokenize()\n" );
+    error( "Wrong number of arguments to MIME.tokenize()\n" );
 
   if (sp[-1].type != T_STRING)
-    Pike_error( "Wrong type of argument to MIME.tokenize()\n" );
+    error( "Wrong type of argument to MIME.tokenize()\n" );
 
   if (sp[-1].u.string->size_shift != 0)
-    Pike_error( "Char out of range for MIME.tokenize()\n" );
+    error( "Char out of range for MIME.tokenize()\n" );
 
   low_tokenize( args, 0 );
 }
 
-/*! @decl array(array(string|int)) tokenize_labled(string header)
- *!
- *! Similar to @[tokenize()], but labels the contents, by making
- *! arrays with two elements; the first a label, and the second
- *! the value that @[tokenize()] would have put there, except
- *! for that comments are kept.
- *!
- *! The following labels exist:
- *! @string
- *!   @value "encoded-word"
- *!     Word encoded according to =?...
- *!   @value "special"
- *!     Special character.
- *!   @value "word"
- *!     Word.
- *!   @value "domain-literal"
- *!     Domain literal.
- *!   @value "comment"
- *!     Comment.
- *! @endstring
- *!
- *! @seealso
- *!   @[MIME.quote()], @[tokenize()],
- *!   @[decode_words_tokenized_labled_remapped()]
- */
 static void f_tokenize_labled( INT32 args )
 {
   if (args != 1)
-    Pike_error( "Wrong number of arguments to MIME.tokenize_labled()\n" );
+    error( "Wrong number of arguments to MIME.tokenize_labled()\n" );
 
   if (sp[-1].type != T_STRING)
-    Pike_error( "Wrong type of argument to MIME.tokenize_labled()\n" );
+    error( "Wrong type of argument to MIME.tokenize_labled()\n" );
 
   if (sp[-1].u.string->size_shift != 0)
-    Pike_error( "Char out of range for MIME.tokenize_labled()\n" );
+    error( "Char out of range for MIME.tokenize_labled()\n" );
 
   low_tokenize( args, 1 );
 }
@@ -988,7 +850,7 @@ static void f_tokenize_labled( INT32 args )
 /*  Convenience function for quote() which determines if a sequence of
  *  characters can be stored as an atom.
  */
-static int check_atom_chars( unsigned char *str, ptrdiff_t len )
+static int check_atom_chars( unsigned char *str, INT32 len )
 {
   /* Atoms must contain at least 1 character... */
   if (len < 1)
@@ -1007,7 +869,7 @@ static int check_atom_chars( unsigned char *str, ptrdiff_t len )
 
 /*  This one check is a sequence of charactes is actually an encoded word.
  */
-static int check_encword( unsigned char *str, ptrdiff_t len )
+static int check_encword( unsigned char *str, INT32 len )
 {
   int q = 0;
 
@@ -1032,22 +894,6 @@ static int check_encword( unsigned char *str, ptrdiff_t len )
 
 /* MIME.quote() */
 
-/*! @decl string quote(array(string|int) lexical_elements);
- *!
- *! This function is the inverse of the @[MIME.tokenize] function.
- *!
- *! A header field value is constructed from a sequence of lexical elements.
- *! Characters (@tt{int@}s) are taken to be special-characters, whereas
- *! strings are encoded as atoms or quoted-strings, depending on whether
- *! they contain any special characters.
- *!
- *! @note
- *!   There is no way to construct a domain-literal using this function.
- *!   Neither can it be used to produce comments.
- *!
- *! @seealso
- *!   @[MIME.tokenize()]
- */
 static void f_quote( INT32 args )
 {
   struct svalue *item;
@@ -1056,9 +902,9 @@ static void f_quote( INT32 args )
   int prev_atom = 0;
 
   if (args != 1)
-    Pike_error( "Wrong number of arguments to MIME.quote()\n" );
+    error( "Wrong number of arguments to MIME.quote()\n" );
   else if (sp[-1].type != T_ARRAY)
-    Pike_error( "Wrong type of argument to MIME.quote()\n" );
+    error( "Wrong type of argument to MIME.quote()\n" );
 
   /* Quote array in sp[-1].u.array.  Once again we'll rely on a
      string_builder to collect the output string. */
@@ -1077,12 +923,12 @@ static void f_quote( INT32 args )
 
       /* Neither int or string.  Too bad... */
       free_string_builder( &buf );
-      Pike_error( "Wrong type of argument to MIME.quote()\n" );
+      error( "Wrong type of argument to MIME.quote()\n" );
 
     } else if (item->u.string->size_shift != 0) {
 
       free_string_builder( &buf );
-      Pike_error( "Char out of range for MIME.quote()\n" );
+      error( "Char out of range for MIME.quote()\n" );
 
     } else {
 
@@ -1105,7 +951,7 @@ static void f_quote( INT32 args )
       } else {
 
 	/* Have to use quoted-string */
-	ptrdiff_t len = str->len;
+	INT32 len = str->len;
 	char *src = str->str;
 	string_builder_putchar( &buf, '"' );
 	while(len--) {
@@ -1128,13 +974,6 @@ static void f_quote( INT32 args )
   push_string( finish_string_builder( &buf ) );
 }
 
-/*! @decl string quote_labled(array(array(string|int)) tokens)
- *!
- *! This function performs the reverse operation of @[tokenize_labled()].
- *!
- *! @seealso
- *!   @[MIME.quote()], @[MIME.tokenize_labled()]
- */
 static void f_quote_labled( INT32 args )
 {
   struct svalue *item;
@@ -1143,9 +982,9 @@ static void f_quote_labled( INT32 args )
   int prev_atom = 0;
 
   if (args != 1)
-    Pike_error( "Wrong number of arguments to MIME.quote_labled()\n" );
+    error( "Wrong number of arguments to MIME.quote_labled()\n" );
   else if (sp[-1].type != T_ARRAY)
-    Pike_error( "Wrong type of argument to MIME.quote_labled()\n" );
+    error( "Wrong type of argument to MIME.quote_labled()\n" );
 
   /* Quote array in sp[-1].u.array.  Once again we'll rely on a
      string_builder to collect the output string. */
@@ -1157,14 +996,14 @@ static void f_quote_labled( INT32 args )
     if (item->type != T_ARRAY || item->u.array->size<2 ||
 	item->u.array->item[0].type != T_STRING) {
       free_string_builder( &buf );
-      Pike_error( "Wrong type of argument to MIME.quote_labled()\n" );
+      error( "Wrong type of argument to MIME.quote_labled()\n" );
     }
 
     if (c_compare_string( item->u.array->item[0].u.string, "special", 7 )) {
 
       if(item->u.array->item[1].type != T_INT) {
 	free_string_builder( &buf );
-	Pike_error( "Wrong type of argument to MIME.quote_labled()\n" );
+	error( "Wrong type of argument to MIME.quote_labled()\n" );
       }
 
       /* Single special character */
@@ -1175,12 +1014,12 @@ static void f_quote_labled( INT32 args )
 
       /* All the remaining lexical items require item[1] to be a string */
       free_string_builder( &buf );
-      Pike_error( "Wrong type of argument to MIME.quote_labled()\n" );
+      error( "Wrong type of argument to MIME.quote_labled()\n" );
 
     } else if (item->u.array->item[1].u.string->size_shift != 0) {
 
       free_string_builder( &buf );
-      Pike_error( "Char out of range for MIME.quote_labled()\n" );
+      error( "Char out of range for MIME.quote_labled()\n" );
 
     } else if (c_compare_string( item->u.array->item[0].u.string, "word", 4 )){
 
@@ -1203,7 +1042,7 @@ static void f_quote_labled( INT32 args )
       } else {
 
 	/* Have to use quoted-string */
-	ptrdiff_t len = str->len;
+	INT32 len = str->len;
 	char *src = str->str;
 	string_builder_putchar( &buf, '"' );
 	while(len--) {
@@ -1234,7 +1073,7 @@ static void f_quote_labled( INT32 args )
       struct pike_string *str = item->u.array->item[1].u.string;
 
       /* Encode comment */
-      ptrdiff_t len = str->len;
+      INT32 len = str->len;
       char *src = str->str;
       string_builder_putchar( &buf, '(' );
       while(len--) {
@@ -1253,12 +1092,12 @@ static void f_quote_labled( INT32 args )
       struct pike_string *str = item->u.array->item[1].u.string;
 
       /* Encode domain-literal */
-      ptrdiff_t len = str->len;
+      INT32 len = str->len;
       char *src = str->str;
 
       if (len<2 || src[0] != '[' || src[len-1] != ']') {
 	free_string_builder( &buf );
-	Pike_error( "Illegal domain-literal passed to MIME.quote_labled()\n" );
+	error( "Illegal domain-literal passed to MIME.quote_labled()\n" );
       }
 
       len -= 2;
@@ -1279,7 +1118,7 @@ static void f_quote_labled( INT32 args )
 
       /* Unknown label.  Too bad... */
       free_string_builder( &buf );
-      Pike_error( "Unknown label passed to MIME.quote_labled()\n" );
+      error( "Unknown label passed to MIME.quote_labled()\n" );
 
     }
 
@@ -1289,6 +1128,3 @@ static void f_quote_labled( INT32 args )
   pop_n_elems( 1 );
   push_string( finish_string_builder( &buf ) );
 }
-
-/*! @endmodule
- */
