@@ -1,13 +1,15 @@
 /*
-|| This file is part of Pike. For copyright information see COPYRIGHT.
-|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
-|| for more information.
-|| $Id: msqlmod.c,v 1.25 2003/04/07 17:23:48 nilsson Exp $
-*/
+ * This code is (C) Francesco Chemolli, 1997.
+ * You may use, modify and redistribute it freely under the terms
+ * of the GNU General Public License, version 2.
+ * $Id: msqlmod.c,v 1.11 2000/04/30 21:16:55 kinkie Exp $
+ *
+ * This version is intended for Pike/0.5 and later.
+ * It won't compile under older versions of the Pike interpreter.
+ */
 
 /* All this code is pretty useless if we don't have a msql library...*/
 #include "global.h"
-#include "module.h"
 #include "msql_config.h"
 #ifdef HAVE_MSQL
 
@@ -33,11 +35,11 @@
 #include "operators.h"
 #include "multiset.h"
 
-RCSID("$Id: msqlmod.c,v 1.25 2003/04/07 17:23:48 nilsson Exp $");
+RCSID("$Id: msqlmod.c,v 1.11 2000/04/30 21:16:55 kinkie Exp $");
 #include "version.h"
 
 #ifdef _REENTRANT
-MUTEX_T pike_msql_mutex STATIC_MUTEX_INIT;
+MUTEX_T pike_msql_mutex;
 #define MSQL_LOCK() mt_lock(&pike_msql_mutex)
 #define MSQL_UNLOCK() mt_unlock(&pike_msql_mutex)
 #else
@@ -52,52 +54,6 @@ MUTEX_T pike_msql_mutex STATIC_MUTEX_INIT;
 
 #include <msql.h>
 
-#define sp Pike_sp
-
-/*! @module Msql
- *! This is an interface to the mSQL database server.
- *! This module may or may not be availible on your Pike, depending
- *! whether the appropriate include and library files (msql.h and libmsql.a
- *! respectively) could be found at compile-time. Note that you @b{do not@}
- *! need to have a mSQL server running on your host to use this module:
- *! you can connect to the database over a TCP/IP socket
- *!
- *! Please notice that unless you wish to specifically connect to a mSQL
- *! server, you'd better use the @[Sql.Sql] program instead. Using @[Sql.Sql]
- *! ensures that your Pike applications will run with any supported SQL
- *! server without changing a single line of code.
- *!
- *! Also notice that some functions may be mSQL/2.0-specific, and thus missing
- *! on hosts running mSQL/1.0.*
- *!
- *! @note
- *!  The mSQL C API has some extermal dependencies.
- *!  They take the form of certain environment variables
- *!  which, if defined in the environment of the pike interpreter, influence
- *!  the interface's behavior. Those are "MSQL_TCP_PORT" which forces the
- *!  server to connect to a port other than the default, "MSQL_UNIX_PORT", same
- *!  as above, only referring to the UNIX domain sockets. If you built your
- *!  mSQL server with the default setttings, you shouldn't worry about these.
- *!  The variable MINERVA_DEBUG can be used to debug the mSQL API (you
- *!  shouldn't worry about this either). Refer to the mSQL documentation
- *!  for further details.
- *!
- *!  Also note that THIS MODULE USES BLOCKING I/O to connect to the server.
- *!  mSQL should be reasonably fast, but you might want to consider this
- *!  particular aspect. It is thread-safe, and so it can be used
- *!  in a multithread environment.
- *!
- *! @fixme
- *! Although it seems that mSQL/2.0 has some support for server statistics,
- *! it's really VERY VERY primitive, so it won't be added for now.
- *!
- *! @seealso
- *!   @[Sql.Sql]
- */
-
-/*! @class msql
- */
-
 static char * decode_msql_type (int msql_type)
 {
 	switch (msql_type) {
@@ -128,7 +84,7 @@ struct msql_my_data
 #endif
 };
 
-#define THIS ((struct msql_my_data *) Pike_fp->current_storage)
+#define THIS ((struct msql_my_data *) fp->current_storage)
 
 static void msql_object_created (struct object *o)
 {
@@ -175,17 +131,13 @@ static void do_select_db(char * dbname)
 	{
 		THIS->db_selected=0;
 		report_error();
-		Pike_error("Could not select database.\n");
+		error("Could not select database.\n");
 	}
 	THIS->db_selected=1;
 	return;
 }
 
-/*! @decl void shutdown()
- *!
- *! This function shuts a SQL-server down.
- */
-
+/* void shutdown() */
 static void do_shutdown (INT32 args)
 /* Notice: the msqlShutdown() function is undocumented. I'll have to go
 	 through the source to find how to report errors.*/
@@ -195,7 +147,7 @@ static void do_shutdown (INT32 args)
 	pop_n_elems(args);
 
 	if (!THIS->connected)
-		Pike_error ("Not connected to any server.\n");
+		error ("Not connected to any server.\n");
 
 	THREADS_ALLOW();
 	MSQL_LOCK();
@@ -206,25 +158,13 @@ static void do_shutdown (INT32 args)
 	THREADS_DISALLOW();
 	if (status<0) {
 		report_error();
-		Pike_error ("Error while shutting down the DBserver, connection not closed.\n");
+		error ("Error while shutting down the DBserver, connection not closed.\n");
 	}
 	THIS->connected=0;
 	THIS->db_selected=0;
 }
 
-/*! @decl void reload_acl()
- *!
- *! This function forces a server to reload its ACLs.
- *!
- *! @note
- *!  This function is @b{not@} part of the standard interface, so it is
- *!  @b{not@} availible through the @[Sql.Sql] interface, but only through
- *!  @[Sql.msql] and @[Msql.msql] programs.
- *!
- *! @seealso
- *!   @[create]
- */
-
+/* void reload_acl() */
 static void do_reload_acl (INT32 args)
 /* Undocumented mSQL function. */
 {
@@ -232,7 +172,7 @@ static void do_reload_acl (INT32 args)
 	check_all_args("Msql->reload_acl",args,0);
 	pop_n_elems(args);
 	if (!THIS->connected)
-		Pike_error ("Not connected to any server.\n");
+		error ("Not connected to any server.\n");
 
 	socket=THIS->socket;
 	THREADS_ALLOW();
@@ -242,38 +182,12 @@ static void do_reload_acl (INT32 args)
 	THREADS_DISALLOW();
 	if (status<0) {
 		report_error();
-		Pike_error ("Could not reload ACLs.\n");
+		error ("Could not reload ACLs.\n");
 	}
 }
 
-/*! @decl void create (void|string dbserver, void|string dbname,@
- *!                    void|string username, void|string passwd)
- *! With one argument, this function
- *! tries to connect to the specified (use hostname or IP address) database
- *! server. To connect to a server running on the local host via UNIX domain
- *! sockets use @expr{"localhost"@}. To connect to the local host via TCP/IP
- *! sockets
- *! you have to use the IP address @expr{"127.0.0.1"@}.
- *! With two arguments it also selects a database to use on the server.
- *! With no arguments it tries to connect to the server on localhost, using
- *! UNIX sockets.
- *!
- *! @throws
- *! You need to have a database selected before using the sql-object,
- *! otherwise you'll get exceptions when you try to query it.
- *! Also notice that this function @b{can@} raise exceptions if the db
- *! server doesn't respond, if the database doesn't exist or is not accessible
- *! by you.
- *!
- *! @note
- *! You don't need bothering about syncronizing the connection to the database:
- *! it is automatically closed (and the database is sync-ed) when the msql
- *! object is destroyed.
- *!
- *! @seealso
- *!   @[select_db]
- */
-
+/* void create (void|string dbserver, void|string dbname,
+	void|string username, void|string passwd) */
 static void msql_mod_create (INT32 args)
 {
 	struct pike_string * arg1=NULL, *arg2=NULL;
@@ -284,12 +198,12 @@ static void msql_mod_create (INT32 args)
 			BIT_STRING|BIT_VOID,BIT_STRING|BIT_VOID,BIT_STRING|BIT_VOID,
 			BIT_STRING|BIT_VOID,0);
 
-	if (args)
-	  if (sp[-args].u.string->len)
-	    arg1=sp[-args].u.string;
-	if (args >1)
-	  if (sp[1-args].u.string->len)
-	    arg2=sp[1-args].u.string;
+  if (args)
+    if (sp[-args].u.string->len)
+      arg1=sp[-args].u.string;
+  if (args >1)
+    if (sp[1-args].u.string->len)
+      arg2=sp[1-args].u.string;
 
 	/*Okay. We had one or two arguments, and we must connect to a server
 	and if needed select a database.
@@ -302,21 +216,18 @@ static void msql_mod_create (INT32 args)
 		THIS->db_selected=0;
 	}
 
+	THREADS_ALLOW();
 	/* msql won' support specifying a port number to connect to. 
 	 * As far as I know, ':' is not a legal character in an hostname,
 	 * so we'll silently ignore it.
 	 */
+
 	if (arg1) {
 		colon=strchr(arg1->str,':');
-		if (colon) {
-		  arg1 = make_shared_binary_string(arg1->str,
-						   colon - arg1->str);
-		  free_string(sp[-args].u.string);
-		  sp[-args].u.string = arg1;
-		}
+		if (colon)
+			*colon='\0';
 	}
 
-	THREADS_ALLOW();
 	MSQL_LOCK();
 	/* Warning! If there were no args, we're deferencing a NULL pointer!*/
 	if (!arg1 || !strcmp (arg1->str,"localhost"))
@@ -330,23 +241,17 @@ static void msql_mod_create (INT32 args)
 		THIS->db_selected=0;
 		THIS->connected=0;
 		report_error();
-		Pike_error("Error while connecting to mSQL server.\n");
+		error("Error while connecting to mSQL server.\n");
 	}
 	THIS->socket=sock;
 	THIS->connected=1;
-	if (arg2)
-	  do_select_db(arg2->str);
+	if (!arg2)
+		return;
+	do_select_db(arg2->str);
 	pop_n_elems(args);
 }
 
-/*! @decl array list_dbs(void|string wild)
- *!
- *! Returns an array containing the names of all databases availible on
- *! the system. Will throw an exception if there is no server connected.
- *! If an argument is specified, it will return only those databases
- *! whose name matches the given glob.
- */
-
+/* array list_dbs(void|string wild) */
 static void do_list_dbs (INT32 args)
 {
 	m_result * result;
@@ -356,7 +261,7 @@ static void do_list_dbs (INT32 args)
 	check_all_args("Msql->list_dbs",args,BIT_STRING|BIT_VOID,0);
 
 	if (!THIS->connected)
-		Pike_error ("Not connected.\n");
+		error ("Not connected.\n");
 	if (args>0 && sp[-args].u.string->len)
 		/* We have a glob. We should pop the arg and push it again for a later
 		 * call to glob() */
@@ -388,15 +293,7 @@ static void do_list_dbs (INT32 args)
 	return;
 }
 
-/*! @decl array list_tables(void|string wild)
- *!
- *! Returns an array containing the names of all the tables in the currently
- *! selected database. Will throw an exception if we aren't connected to
- *! a database.
- *! If an argument is specified, it will return only those tables
- *! whose name matches the given glob.
- */
-
+/* array list_tables(void|string wild) */
 static void do_list_tables (INT32 args)
 	/* ARGH! There's much code duplication here... but the subtle differences
 	 * between the various functions make it impervious to try to generalize..*/
@@ -408,7 +305,7 @@ static void do_list_tables (INT32 args)
 	check_all_args ("Msql->list_tables",args,BIT_STRING|BIT_VOID,0);
 
 	if (!THIS->db_selected)
-		Pike_error ("No database selected.\n");
+		error ("No database selected.\n");
 
 	if (args>0 && sp[-args].u.string->len)
 		/* We have a glob. We should pop the arg and push it again for a later
@@ -441,22 +338,7 @@ static void do_list_tables (INT32 args)
 	return;
 }
 
-/*! @decl void select_db(string dbname)
- *!
- *! Before querying a database you have to select it. This can be accomplished
- *! in two ways: the first is calling the @[create] function with two arguments,
- *! another is calling it with one or no argument and then calling @[select_db].
- *! You can also use this function to change the database you're querying,
- *! as long as it is on the same server you are connected to.
- *!
- *! @throws
- *! This function CAN raise exceptions in case something goes wrong
- *! (for example: unexistant database, insufficient permissions, whatever).
- *!
- *! @seealso
- *!  @[create], @[error]
- */
-
+/* void select_db(string dbname) */
 static void select_db(INT32 args)
 {
 	struct pike_string * arg;
@@ -464,46 +346,13 @@ static void select_db(INT32 args)
 
 	check_all_args("Msql->select_db",args,BIT_STRING,0);
 	if (!THIS->connected)
-		Pike_error ("Not connected.\n");
+		error ("Not connected.\n");
 	arg=sp[-args].u.string;
 	do_select_db(arg->str);
 	pop_n_elems(args);
 }
 
-/*! @decl array(mapping(string:mixed)) query (string sqlquery)
- *!
- *! This is all you need to query the database. It takes as argument an SQL
- *! query string (i.e.: "SELECT foo,bar FROM baz WHERE name like '%kinkie%'"
- *! or "INSERT INTO baz VALUES ('kinkie','made','this')")
- *! and returns a data structure containing the returned values.
- *! The structure is an array (one entry for each returned row) of mappings
- *! which have the column name as index and the column contents as data.
- *! So to access a result from the first example you would have to do
- *! "results[0]->foo".
- *!
- *! A query which returns no data results in an empty array (and NOT in a 0).
- *! Also notice that when there is a column name clash (that is: when you
- *! join two or more tables which have columns with the same name), the
- *! clashing columns are renamed to <tablename>+"."+<column name>.
- *! To access those you'll have to use the indexing operator '[]
- *! (i.e.: results[0]["foo.bar"]).
- *!
- *! @throws
- *! Errors (both from the interface and the SQL server) are reported via
- *! exceptions, and you definitely want to catch them. Error messages are
- *! not particularly verbose, since they account only for errors inside the
- *! driver. To get server-related error messages, you have to use the
- *! @[error] function.
- *!
- *! @note
- *! Note that if the query is NOT a of SELECT type, but UPDATE or
- *! MODIFY, the returned value is an empty array.
- *! @b{This is not an error@}. Errors are reported @b{only@} via exceptions.
- *!
- *! @seealso
- *!   @[error]
- */
-
+/* array(mapping(string:mixed)) query (string sqlquery) */
 static void do_query (INT32 args)
 {
 	int status, num_fields,num_rows,j,k,tmp_socket,*duplicate_names_map;
@@ -516,9 +365,9 @@ static void do_query (INT32 args)
 	check_all_args("Msql->query",args,BIT_STRING,0);
 
 	if (!THIS->connected)
-		Pike_error("Must connect to database server before querying it.\n");
+		error("Must connect to database server before querying it.\n");
 	if (!THIS->db_selected)
-		Pike_error("Must select database before querying it.\n");
+		error("Must select database before querying it.\n");
 
 	tmp_socket=THIS->socket;
 	query=sp[-args].u.string->str;
@@ -535,7 +384,7 @@ static void do_query (INT32 args)
 
 	if (status==-1) {
 		report_error();
-		Pike_error("Error in SQL query.\n");
+		error("Error in SQL query.\n");
 	}
 
 #ifdef MSQL_VERSION_2
@@ -560,7 +409,7 @@ static void do_query (INT32 args)
 	duplicate_names_map = malloc (sizeof(int)*num_fields);
 	names = malloc (sizeof(char*)*num_fields);
 	if (!duplicate_names_map || !names)
-		Pike_error ("Memory exhausted.\n");
+		error ("Memory exhausted.\n");
 
 	/* initialize support structure for duplicate column names */
 	for (j=0;j<num_fields;j++)
@@ -585,7 +434,7 @@ static void do_query (INT32 args)
 	for (j=0;j<num_fields;j++) {
 		current_field=msqlFetchField(result);
 		if (!current_field)
-			Pike_error ("Huh? Weird! Null field returned at index %d.\n",j);
+			error ("Huh? Weird! Null field returned at index %d.\n",j);
 		if (duplicate_names_map[j]) { 
 			/* If we have a clashing column name we need to prepend "tablename."*/
 			push_text (current_field->table);
@@ -627,15 +476,7 @@ static void do_query (INT32 args)
 	msqlFreeResult(result);
 }
 
-/*! @decl string server_info()
- *!
- *! This function returns a string describing the server we are talking
- *! to. It has the form "servername/serverversion" (like the HTTP protocol
- *! description) and is most useful in conjunction with the generic SQL-server
- *! module.
- *!
- */
-
+/* string server_info() */
 static void do_info (INT32 args)
 {
 	char * info;
@@ -643,7 +484,7 @@ static void do_info (INT32 args)
 	check_all_args("Msql->info",args,0);
 	pop_n_elems(args);
 	if (!THIS->connected)
-		Pike_error ("Not connected.\n");
+		error ("Not connected.\n");
 	push_text("msql/");
 	THREADS_ALLOW();
 	MSQL_LOCK();
@@ -655,35 +496,20 @@ static void do_info (INT32 args)
 	return;
 }
 
-/*! @decl string host_info()
- *!
- *! This function returns a string describing what host are we talking to,
- *! and how (TCP/IP or UNIX sockets).
- */
-
+/* string host_info() */
 static void do_host_info (INT32 args)
 {
 	check_all_args("Msql->host_info",args,0);
 	pop_n_elems(args);
 	if (!THIS->connected)
-		Pike_error ("Not connected.\n");
+		error ("Not connected.\n");
 	/*it's local to the client library. Not even worth allowing
 	 context switches*/
 	push_text(msqlGetHostInfo()); 
 	return;
 }
 
-/*! @decl string error()
- *!
- *! This function returns the textual description of the last server-related
- *! error. Returns 0 if no error has occurred yet. It is not cleared upon
- *! reading (can be invoked multiple times, will return the same result
- *! until a new error occurs).
- *!
- *! @seealso
- *!   @[query]
- */
-
+/* string error() */
 static void do_error (INT32 args)
 {
 	check_all_args("Msql->error",args,0);
@@ -695,15 +521,7 @@ static void do_error (INT32 args)
 	return;
 }
 
-/*! @decl void create_db(string dbname)
- *!
- *! This function creates a new database with the given name (assuming we
- *! have enough permissions to do this).
- *!
- *! @seealso
- *!   @[drop_db]
- */
-
+/* void create_db (string dbname) */
 static void do_create_db (INT32 args)
 {
 	int dbresult;
@@ -713,7 +531,7 @@ static void do_create_db (INT32 args)
 	check_all_args("Msql->create_db",args,BIT_STRING,0);
 
 	if (!THIS->connected)
-		Pike_error("Not connected.\n");
+		error("Not connected.\n");
 	dbname = sp[-args].u.string->str;
 	socket=THIS->socket;
 	THREADS_ALLOW();
@@ -723,20 +541,12 @@ static void do_create_db (INT32 args)
 	THREADS_DISALLOW();
 	if (dbresult==-1) {
 		report_error();
-		Pike_error ("Could not create database.\n");
+		error ("Could not create database.\n");
 	}
 	pop_n_elems(args);
 }
 
-/*! @decl void drop_db(string dbname)
- *!
- *! This function destroys a database and all the data it contains (assuming
- *! we have enough permissions to do so). USE WITH CAUTION!
- *!
- *! @seealso
- *!   @[create_db]
- */
-
+/* void drop_db (string dbname) */
 static void do_drop_db (INT32 args)
 {
 	int dbresult;
@@ -746,7 +556,7 @@ static void do_drop_db (INT32 args)
 	check_all_args("Msql->drop_db",args,BIT_STRING,0);
 
 	if (!THIS->connected)
-		Pike_error("Not connected.\n");
+		error("Not connected.\n");
 	dbname = sp[-args].u.string->str;
 	socket=THIS->socket;
 	THREADS_ALLOW();
@@ -756,44 +566,13 @@ static void do_drop_db (INT32 args)
 	THREADS_DISALLOW();
 	if (dbresult==-1) {
 		report_error();
-		Pike_error ("Could not drop database.\n");
+		error ("Could not drop database.\n");
 	}
 	pop_n_elems(args);
 	return;
 }
 
-/*! @decl mapping(string:mapping(string:mixed)) list_fields(string table, void|string glob)
- *!
- *! Returns a mapping describing the fields of a table in the database.
- *! The returned value is a mapping, indexed on the column name,
- *! of mappings.The glob argument, if present, filters out the fields
- *! not matching the glob. These contain currently the fields:
- *!
- *! @mapping
- *!   @member string "type"
- *!      Describes the field's mSQL data type ("char","integer",...)
- *!   @member int "length"
- *!      It describes the field's length. It is only interesting for
- *!      char() fields, in fact.  Also
- *!      notice that the strings returned by msql->query() have the correct length.
- *!      This field only specifies the _maximum_ length a "char()" field can have.
- *!   @member string "table"
- *!      The table this field is in. Added only for interface compliancy.
- *!   @member multiset(string) "flags"
- *!      It's a multiset containing textual
- *!      descriptions of the server's flags associated with the current field.
- *!      Currently it can be empty, or contain "unique" or "not null".
- *! @endmapping
- *!
- *! @note
- *!   The version of this function in the Msql.msql() program is @b{not@}
- *!   sql-interface compliant (this is the main reason why using that program
- *!   directly is deprecated). Use @[Sql.Sql] instead.
- *!
- *! @seealso
- *!   @[query]
- */
-
+/* mapping(string:array(string)) list_fields (string table) */
 static void do_list_fields (INT32 args)
 {
 	m_result * result;
@@ -803,9 +582,9 @@ static void do_list_fields (INT32 args)
 
 	check_all_args("Msql->list_fields",args,BIT_STRING,0);
 	if (!THIS->connected)
-		Pike_error ("Not connected.\n");
+		error ("Not connected.\n");
 	if (!THIS->db_selected)
-		Pike_error ("Must select a db first.\n");
+		error ("Must select a db first.\n");
 	table=sp[-args].u.string->str;
 #ifdef MSQL_DEBUG
 	printf ("list_fields: table=%s(%d)\n",sp[-args].u.string->str,sp[-args].u.string->len);
@@ -820,20 +599,19 @@ static void do_list_fields (INT32 args)
 
 	if (!result) {
 		report_error();
-		Pike_error ("No fields information.\n");
+		error ("No fields information.\n");
 	}
 	
 	fields = msqlNumFields(result);
 	if (!fields)
-		Pike_error ("No such table.\n");
+		error ("No such table.\n");
 
 	for (j=0;j<fields;j++)
 	{
 		int flagsnum=0;
 		field=msqlFetchField(result);
-    
-    push_text("name");
 		push_text(field->name);
+	
 		push_text("type");
 		push_text(decode_msql_type(field->type));
 		push_text("length");
@@ -860,31 +638,16 @@ static void do_list_fields (INT32 args)
 		}
 #endif
 		f_aggregate_multiset(flagsnum);
-		f_aggregate_mapping(10);
+		f_aggregate_mapping(8); /*must aggregate on the fields above, except name*/
 	}
-  f_aggregate(fields);
+	f_aggregate_mapping(fields*2);
 
 	msqlFreeResult(result);
 	return;
 }
 
 #ifdef MSQL_VERSION_2
-
-/*! @decl int affected_rows()
- *!
- *! This function returns how many rows in the database were affected by
- *! our last SQL query.
- *!
- *! @note
- *!  This function is availible only if you're using mSQL version 2
- *!  or later. (That means: if the includes and library of version 2 of mSQL
- *!  were availible when the module was compiled).
- *!
- *!  This function is @b{not@} part of the standard interface, so it is @b{not@}
- *!  availible through the @[Sql.Sql] interface, but only through @[Sql.msql] and
- *!  @[Msql.msql] programs
- */
-
+/* int affected_rows() */
 static void do_affected_rows (INT32 args)
 {
 	check_all_args("Msql->affected_rows",args,0);
@@ -893,24 +656,7 @@ static void do_affected_rows (INT32 args)
 	return;
 }
 
-/*! @decl array list_index(string tablename, string indexname)
- *!
- *! This function returns an array describing the index structure for the
- *! given table and index name, as defined by the non-standard SQL query
- *! 'create index' (see the mSQL documentation for further informations).
- *! More than one index can be created for a table. There's currently NO way
- *! to have a listing of the indexes defined for a table (blame it on
- *! the mSQL API).
- *!
- *! @note
- *!  This function is availible if you're using mSQL version 2
- *!  or later.
- *!
- *!  This function is @b{not@} part of the standard interface, so it is @b{not@}
- *!  availible through the @[Sql.Sql] interface, but only through @[Sql.msql] and
- *!  @[Msql.msql] programs.
- */
-
+/* array list_index(string tablename, string indexname) */
 static void do_list_index (INT32 args)
 {
 	char * arg1, *arg2;
@@ -920,7 +666,7 @@ static void do_list_index (INT32 args)
 
 	check_all_args("Msql->list_index",args,BIT_STRING,BIT_STRING,0);
 	if (!THIS->db_selected)
-		Pike_error ("No database selected.\n");
+		error ("No database selected.\n");
 	arg1=sp[-args].u.string->str;
 	arg2=sp[1-args].u.string->str;
 	sock=THIS->socket;
@@ -946,19 +692,7 @@ static void do_list_index (INT32 args)
 }
 #endif
 
-/*! @endclass
- */
-
-/*! @decl constant version
- *!
- *! Should you need to report a bug to the author, please submit along with
- *! the report the driver version number, as returned by this call.
- */
-
-/*! @endmodule
- */
-
-PIKE_MODULE_INIT
+void pike_module_init(void)
 {
 	start_new_program();
 	ADD_STORAGE(struct msql_my_data);
@@ -1009,7 +743,7 @@ PIKE_MODULE_INIT
 	/* Lists the tables contained in the selected database. */
 
 	/* function(string:mapping(string:array(mixed))) */
-  ADD_FUNCTION("list_fields", do_list_fields,tFunc(tStr,tArr(tMap(tStr,tMix))),
+  ADD_FUNCTION("list_fields", do_list_fields,tFunc(tStr,tMap(tStr,tArr(tMix))),
 		OPT_RETURN|OPT_EXTERNAL_DEPEND);
 	/* Returns information on the the fields of the given table of the current
 	  database */
@@ -1070,7 +804,7 @@ PIKE_MODULE_INIT
 }
 
 #else /*HAVE_MSQL*/
-PIKE_MODULE_INIT {}
+void pike_module_init(void) {}
 #endif /*HAVE_MSQL*/
 
-PIKE_MODULE_EXIT { }
+void pike_module_exit (void) { }
