@@ -1,10 +1,3 @@
-/*
-|| This file is part of Pike. For copyright information see COPYRIGHT.
-|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
-|| for more information.
-|| $Id: initstuff.h,v 1.5 2004/02/23 17:40:40 per Exp $
-*/
-
 #ifndef PARSER_INITER
 #define PARSER_CLASS(a,b,c,what,id) extern struct program *what;
 #define PARSER_SUBMODULE(a,b,c) 
@@ -15,8 +8,6 @@
 PARSER_CLASS("HTML", init_parser_html, exit_parser_html, 
 	    parser_html_program, PROG_PARSER_HTML_ID )
 
-PARSER_SUBMODULE("_RCS", init_parser_rcs, exit_parser_rcs )
-
    /*
 for documentation purpose:
 
