@@ -1,9 +1,6 @@
 /*
-|| This file is part of Pike. For copyright information see COPYRIGHT.
-|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
-|| for more information.
-|| $Id: acconfig.h,v 1.3 2002/10/11 01:39:53 nilsson Exp $
-*/
+ * $Id: acconfig.h,v 1.1 2000/09/11 17:48:19 grubba Exp $
+ */
 
 #ifndef IMAGE_GIF_CONFIG_H
 #define IMAGE_GIF_CONFIG_H
