@@ -1,5 +1,5 @@
 /*
- * $Id: crypto.c,v 1.41 2000/12/05 21:08:32 per Exp $
+ * $Id: crypto.c,v 1.31 2000/03/28 12:16:39 grubba Exp $
  *
  * A pike module for getting access to some common cryptos.
  *
@@ -37,13 +37,10 @@
 /* Prototypes */
 #include "crypto.h"
 
-/* THIS MUST BE INCLUDED LAST */
-#include "module_magic.h"
-
 struct pike_crypto {
   struct object *object;
-  ptrdiff_t block_size;
-  ptrdiff_t backlog_len;
+  INT32 block_size;
+  INT32 backlog_len;
   unsigned char *backlog;
 };
 
@@ -61,7 +58,7 @@ static const char *crypto_functions[] = {
 };
 
 #undef THIS
-#define THIS	((struct pike_crypto *)(Pike_fp->current_storage))
+#define THIS	((struct pike_crypto *)(fp->current_storage))
 
 static struct program *pike_crypto_program;
 
@@ -91,7 +88,7 @@ static void check_functions(struct object *o, const char **requiered)
   struct program *p;
 
   if (!o) {
-    Pike_error("Crypto: internal error -- no object\n");
+    error("/precompiled/crypto: internal error -- no object\n");
   }
   if (!requiered) {
     return;
@@ -101,7 +98,7 @@ static void check_functions(struct object *o, const char **requiered)
 
   while (*requiered) {
     if (find_identifier( (char *) *requiered, p) < 0) {
-      Pike_error("Crypto: Object is missing identifier \"%s\"\n",
+      error("/precompiled/crypto: Object is missing identifier \"%s\"\n",
 	    *requiered);
     }
     requiered++;
@@ -124,10 +121,10 @@ static void f_string_to_hex(INT32 args)
   INT32 i;
 
   if (args != 1) {
-    Pike_error("Wrong number of arguments to string_to_hex()\n");
+    error("Wrong number of arguments to string_to_hex()\n");
   }
   if (sp[-1].type != T_STRING) {
-    Pike_error("Bad argument 1 to string_to_hex()\n");
+    error("Bad argument 1 to string_to_hex()\n");
   }
 
   s = begin_shared_string(2 * sp[-1].u.string->len);
@@ -147,13 +144,13 @@ static void f_hex_to_string(INT32 args)
   INT32 i;
 
   if (args != 1) {
-    Pike_error("Wrong number of arguments to hex_to_string()\n");
+    error("Wrong number of arguments to hex_to_string()\n");
   }
   if (sp[-1].type != T_STRING) {
-    Pike_error("Bad argument 1 to hex_to_string()\n");
+    error("Bad argument 1 to hex_to_string()\n");
   }
   if (sp[-1].u.string->len & 1) {
-    Pike_error("Bad string length to hex_to_string()\n");
+    error("Bad string length to hex_to_string()\n");
   }
 
   s = begin_shared_string(sp[-1].u.string->len/2);
@@ -171,7 +168,7 @@ static void f_hex_to_string(INT32 args)
       break;
     default:
       free_string(end_shared_string(s));
-      Pike_error("hex_to_string(): Illegal character (0x%02x) in string\n",
+      error("hex_to_string(): Illegal character (0x%02x) in string\n",
 	    sp[-1].u.string->str[i*2] & 0xff);
     }
     switch (sp[-1].u.string->str[i*2+1])
@@ -186,7 +183,7 @@ static void f_hex_to_string(INT32 args)
       break;
     default:
       free_string(end_shared_string(s));
-      Pike_error("hex_to_string(): Illegal character (0x%02x) in string\n",
+      error("hex_to_string(): Illegal character (0x%02x) in string\n",
 	    sp[-1].u.string->str[i*2+1] & 0xff);
     }
   }
@@ -207,10 +204,10 @@ static void f_des_parity(INT32 args)
   struct pike_string *s;
   int i;
   if (args != 1) {
-    Pike_error("Wrong number of arguments to des_parity()\n");
+    error("Wrong number of arguments to des_parity()\n");
   }
   if (sp[-1].type != T_STRING) {
-    Pike_error("Bad argument 1 to des_parity()\n");
+    error("Bad argument 1 to des_parity()\n");
   }
 
   s = begin_shared_string(sp[-1].u.string->len);
@@ -223,24 +220,24 @@ static void f_des_parity(INT32 args)
 }
 
 /*
- * Crypto
+ * /precompiled/crypto
  */
 
 /* void create(program|object, ...) */
 static void f_create(INT32 args)
 {
   if (args < 1) {
-    Pike_error("Too few arguments to crypto->create()\n");
+    error("Too few arguments to crypto->create()\n");
   }
   if ((sp[-args].type != T_PROGRAM) &&
       (sp[-args].type != T_OBJECT)) {
-    Pike_error("Bad argument 1 to crypto->create()\n");
+    error("Bad argument 1 to crypto->create()\n");
   }
   if (sp[-args].type == T_PROGRAM) {
     THIS->object = clone_object(sp[-args].u.program, args-1);
   } else {
     if (args != 1) {
-      Pike_error("Too many arguments to crypto->create()\n");
+      error("Too many arguments to crypto->create()\n");
     }
     add_ref(THIS->object = sp[-args].u.object);
   }
@@ -251,7 +248,7 @@ static void f_create(INT32 args)
   safe_apply(THIS->object, "query_block_size", 0);
 
   if (sp[-1].type != T_INT) {
-    Pike_error("crypto->create(): query_block_size() didn't return an int\n");
+    error("crypto->create(): query_block_size() didn't return an int\n");
   }
   THIS->block_size = sp[-1].u.integer;
 
@@ -259,8 +256,7 @@ static void f_create(INT32 args)
 
   if ((!THIS->block_size) ||
       (THIS->block_size > 4096)) {
-    Pike_error("crypto->create(): Bad block size %ld\n",
-	  DO_NOT_WARN((long)THIS->block_size));
+    error("crypto->create(): Bad block size %d\n", THIS->block_size);
   }
 
   THIS->backlog = (unsigned char *)xalloc(THIS->block_size);
@@ -272,7 +268,7 @@ static void f_create(INT32 args)
 static void f_query_block_size(INT32 args)
 {
   pop_n_elems(args);
-  push_int(DO_NOT_WARN(THIS->block_size));
+  push_int(THIS->block_size);
 }
 
 /* int query_key_length(void) */
@@ -288,7 +284,7 @@ static void f_set_encrypt_key(INT32 args)
     MEMSET(THIS->backlog, 0, THIS->block_size);
     THIS->backlog_len = 0;
   } else {
-    Pike_error("crypto->set_encrypt_key(): Object has not been created yet\n");
+    error("crypto->set_encrypt_key(): Object has not been created yet\n");
   }
   safe_apply(THIS->object, "set_encrypt_key", args);
   pop_stack();
@@ -302,7 +298,7 @@ static void f_set_decrypt_key(INT32 args)
     MEMSET(THIS->backlog, 0, THIS->block_size);
     THIS->backlog_len = 0;
   } else {
-    Pike_error("crypto->set_decrypt_key(): Object has not been created yet\n");
+    error("crypto->set_decrypt_key(): Object has not been created yet\n");
   }
   safe_apply(THIS->object, "set_decrypt_key", args);
   pop_stack();
@@ -310,21 +306,21 @@ static void f_set_decrypt_key(INT32 args)
 }
 
 /* string crypt(string) */
-static void f_crypto_crypt(INT32 args)
+static void f_crypt(INT32 args)
 {
   unsigned char *result;
-  ptrdiff_t roffset = 0;
-  ptrdiff_t soffset = 0;
-  ptrdiff_t len;
+  INT32 roffset = 0;
+  INT32 soffset = 0;
+  INT32 len;
 
   if (args != 1) {
-    Pike_error("Wrong number of arguments to crypto->crypt()\n");
+    error("Wrong number of arguments to crypto->crypt()\n");
   }
   if (sp[-1].type != T_STRING) {
-    Pike_error("Bad argument 1 to crypto->crypt()\n");
+    error("Bad argument 1 to crypto->crypt()\n");
   }
   if (!(result = alloca(sp[-1].u.string->len + THIS->block_size))) {
-    Pike_error("crypto->crypt(): Out of memory\n");
+    error("crypto->crypt(): Out of memory\n");
   }
   if (THIS->backlog_len) {
     if (sp[-1].u.string->len >=
@@ -338,11 +334,11 @@ static void f_crypto_crypt(INT32 args)
 					    THIS->block_size));
       safe_apply(THIS->object, "crypt_block", 1);
       if (sp[-1].type != T_STRING) {
-	Pike_error("crypto->crypt(): crypt_block() did not return string\n");
+	error("crypto->crypt(): crypt_block() did not return string\n");
       }
       if (sp[-1].u.string->len != THIS->block_size) {
-	Pike_error("crypto->crypt(): Unexpected string length %ld\n",
-	      DO_NOT_WARN((long)sp[-1].u.string->len));
+	error("crypto->crypt(): Unexpected string length %d\n",
+	      sp[-1].u.string->len);
       }
 	
       MEMCPY(result, sp[-1].u.string->str, THIS->block_size);
@@ -369,11 +365,11 @@ static void f_crypto_crypt(INT32 args)
     safe_apply(THIS->object, "crypt_block", 1);
 
     if (sp[-1].type != T_STRING) {
-      Pike_error("crypto->crypt(): crypt_block() did not return string\n");
+      error("crypto->crypt(): crypt_block() did not return string\n");
     }
     if (sp[-1].u.string->len != len) {
-      Pike_error("crypto->crypt(): Unexpected string length %ld\n",
-	    DO_NOT_WARN((long)sp[-1].u.string->len));
+      error("crypto->crypt(): Unexpected string length %d\n",
+	    sp[-1].u.string->len);
     }
 	
     MEMCPY(result + roffset, sp[-1].u.string->str, len);
@@ -396,17 +392,17 @@ static void f_crypto_crypt(INT32 args)
 /* string pad(void) */
 static void f_pad(INT32 args)
 {
-  ptrdiff_t i;
+  int i;
+  int len;
   
   if (args) {
-    Pike_error("Too many arguments to crypto->pad()\n");
+    error("Too many arguments to crypto->pad()\n");
   }
 
   for (i = THIS->backlog_len; i < THIS->block_size - 1; i++) 
     THIS->backlog[i] = my_rand() & 0xff;
   
-  THIS->backlog[THIS->block_size - 1] =
-    DO_NOT_WARN((unsigned char)(7 - THIS->backlog_len));
+  THIS->backlog[THIS->block_size - 1] = 7 - THIS->backlog_len;
 
   push_string(make_shared_binary_string((const char *)THIS->backlog,
 					THIS->block_size));
@@ -420,25 +416,25 @@ static void f_pad(INT32 args)
 /* string unpad(string) */
 static void f_unpad(INT32 args)
 {
-  ptrdiff_t len;
+  int len;
   struct pike_string *str;
 
   if (args != 1) 
-    Pike_error("Wrong number of arguments to crypto->unpad()\n");
+    error("Wrong number of arguments to crypto->unpad()\n");
   
   if (sp[-1].type != T_STRING) 
-    Pike_error("Bad argument 1 to crypto->unpad()\n");
+    error("Bad argument 1 to crypto->unpad()\n");
   
   str = sp[-1].u.string;
   len = str->len;
 
   if (str->str[len - 1] > (THIS->block_size - 1))
-    Pike_error("crypto->unpad(): Invalid padding\n");
+    error("crypto->unpad(): Invalid padding\n");
 
   len -= (str->str[len - 1] + 1);
 
   if (len < 0) 
-    Pike_error("crypto->unpad(): String to short to unpad\n");
+    error("crypto->unpad(): String to short to unpad\n");
   
   add_ref(str);
   pop_stack();
@@ -486,7 +482,7 @@ void pike_crypto_init(void)
   /* function(string:object) */
   ADD_FUNCTION("set_decrypt_key", f_set_decrypt_key,tFunc(tStr,tObj), 0);
   /* function(string:string) */
-  ADD_FUNCTION("crypt", f_crypto_crypt,tFunc(tStr,tStr), 0);
+  ADD_FUNCTION("crypt", f_crypt,tFunc(tStr,tStr), 0);
 
   /* function(void:string) */
   ADD_FUNCTION("pad", f_pad,tFunc(tVoid,tStr), 0);
@@ -510,7 +506,6 @@ void pike_module_init(void)
   /* function(string:string) */
   ADD_FUNCTION("des_parity", f_des_parity,tFunc(tStr,tStr), 0);
 
-  pike_md2_init();
   pike_md5_init();
   pike_sha_init();
   pike_crypto_init();
@@ -523,7 +518,6 @@ void pike_module_init(void)
 
 void pike_module_exit(void)
 {
-  pike_md2_exit();
   pike_md5_exit();
   pike_sha_exit();
   pike_crypto_exit();
