@@ -1,9 +1,12 @@
+/*\
+||| This file a part of Pike, and is copyright by Fredrik Hubinette
+||| Pike is distributed as GPL (General Public License)
+||| See the files COPYING and DISCLAIMER for more information.
+\*/
+
 /*
-|| This file is part of Pike. For copyright information see COPYRIGHT.
-|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
-|| for more information.
-|| $Id: file.h,v 1.36 2004/04/05 21:51:02 mast Exp $
-*/
+ * $Id: file.h,v 1.15 2000/02/03 19:05:37 grubba Exp $
+ */
 
 #ifndef FILE_H
 #define FILE_H
@@ -25,25 +28,23 @@
 #endif
 #endif
 
-#include "pike_netlib.h"
-#include "backend.h"
-
 struct my_file
 {
-  struct fd_callback_box box;	/* Must be first. */
-  /* The box is hooked in whenever box.backend is set. */
-
-  struct svalue event_cbs[4];
-  /* Callbacks can be set without having the corresponding bits in
-   * box.events, but not the other way around. */
-
   short open_mode;
   short flags;
+  FD fd;
   int my_errno;
+  struct svalue read_callback;
+  struct svalue write_callback;
+#ifdef WITH_OOB
+  struct svalue read_oob_callback;
+  struct svalue write_oob_callback;
+#endif /* WITH_OOB */
 
-#if defined(HAVE_FD_FLOCK) || defined(HAVE_FD_LOCKF)
+#if defined(HAVE_FD_FLOCK) || defined(HAVE_FD_LOCKF) 
   struct object *key;
 #endif
+  struct object *myself;
 };
 
 #ifdef _REENTRANT
@@ -51,7 +52,7 @@ struct my_file
 #ifndef HAVE_STRUCT_IOVEC
 struct iovec {
   void *iov_base;
-  size_t iov_len;
+  int iov_len;
 };
 #endif /* !HAVE_STRUCT_IOVEC */
 
@@ -60,7 +61,7 @@ struct pike_sendfile
 {
   struct object *self;
 
-  LONGEST sent;
+  int sent;
 
   struct array *headers;
   struct array *trailers;
@@ -68,7 +69,6 @@ struct pike_sendfile
   struct object *from_file;
   struct object *to_file;
 
-  struct callback *backend_callback;
   struct svalue callback;
   struct array *args;
 
@@ -78,8 +78,8 @@ struct pike_sendfile
   struct my_file *from;
   struct my_file *to;
 
-  LONGEST offset;
-  LONGEST len;
+  INT_TYPE offset;
+  INT_TYPE len;
 
   struct iovec *hd_iov;
   struct iovec *tr_iov;
@@ -89,7 +89,7 @@ struct pike_sendfile
 
   struct iovec *iovs;
   char *buffer;
-  ptrdiff_t buf_size;
+  int buf_size;
 };
 
 #endif /* _REENTRANT */
@@ -97,7 +97,13 @@ struct pike_sendfile
 extern struct program *file_program;
 extern struct program *file_ref_program;
 
-extern int get_inet_addr(PIKE_SOCKADDR *addr,char *name,char *service, INT_TYPE port, int udp);
+extern void get_inet_addr(struct sockaddr_in *addr,char *name);
+
+#define CBFUNCS(X) \
+static void PIKE_CONCAT(file_,X) (int fd, void *data);		\
+static void PIKE_CONCAT(file_set_,X) (INT32 args);		\
+static void PIKE_CONCAT(file_query_,X) (INT32 args);		\
+
 
 #ifdef _REENTRANT
 void low_do_sendfile(struct pike_sendfile *);
@@ -106,12 +112,9 @@ void low_do_sendfile(struct pike_sendfile *);
 /* Prototypes begin here */
 void my_set_close_on_exec(int fd, int to);
 void do_set_close_on_exec(void);
-
-#define CBFUNCS(X) \
-static void PIKE_CONCAT(file_set_,X) (INT32 args); \
-static void PIKE_CONCAT(file_query_,X) (INT32 args);
 CBFUNCS(read_callback)
 CBFUNCS(write_callback)
+
 CBFUNCS(read_oob_callback)
 CBFUNCS(write_oob_callback)
 
@@ -123,17 +126,13 @@ struct new_thread_data;
 void file_proxy(INT32 args);
 void create_proxy_pipe(struct object *o, int for_reading);
 struct file_lock_key_storage;
+void pike_module_exit(void);
 void mark_ids(struct callback *foo, void *bar, void *gazonk);
+void pike_module_init(void);
 int pike_make_pipe(int *fds);
 int fd_from_object(struct object *o);
-void f_strerror(INT32 args);
-void push_stat(PIKE_STAT_T *s);
 /* Prototypes end here */
 
-/* open_mode
- *
- * Note: The lowest 8 bits are reserved for the fd_* flags from "fdlib.h".
- */
 #define FILE_READ               0x1000
 #define FILE_WRITE              0x2000
 #define FILE_APPEND             0x4000
@@ -145,6 +144,7 @@ void push_stat(PIKE_STAT_T *s);
 #define FILE_SET_CLOSE_ON_EXEC  0x0800
 
 /* flags */
+#define FILE_HAS_INTERNAL_REF   0x0001
 #define FILE_NO_CLOSE_ON_DESTRUCT 0x0002
 #define FILE_LOCK_FD		0x0004
 #define FILE_NOT_OPENED         0x0010
