@@ -43,7 +43,7 @@
 #include "threads.h"
 #include "operators.h"
 
-RCSID("$Id: spider.c,v 1.105 2001/07/05 20:00:45 david%hedbor.org Exp $");
+RCSID("$Id: spider.c,v 1.95 2001/07/05 20:03:09 david%hedbor.org Exp $");
 
 #ifdef HAVE_PWD_H
 #include <pwd.h>
@@ -86,10 +86,6 @@ RCSID("$Id: spider.c,v 1.105 2001/07/05 20:00:45 david%hedbor.org Exp $");
 
 #include "dmalloc.h"
 
-
-/* This must be included last! */
-#include "module_magic.h"
-
 #define MAX_PARSE_RECURSE 102
 
 void do_html_parse(struct pike_string *ss,
@@ -103,19 +99,64 @@ void do_html_parse_lines(struct pike_string *ss,
 			 struct array *extra_args,
 			 int line);
 
+
+void f_nice(INT32 args)
+{
+#ifdef HAVE_NICE
+  int ta = sp[-1].u.integer;
+  if(!args) error("You must supply an argument to nice(int)!\n");
+  pop_n_elems(args);
+  push_int(nice(ta));
+#endif
+}
+
+void f_http_decode_string(INT32 args)
+{
+   int proc;
+   char *foo,*bar,*end;
+   struct pike_string *newstr;
+
+   if (!args || sp[-args].type != T_STRING)
+     error("Invalid argument to http_decode_string(STRING);\n");
+
+   foo=bar=sp[-args].u.string->str;
+   end=foo+sp[-args].u.string->len;
+
+   /* count '%' characters */
+   for (proc=0; foo<end; ) if (*foo=='%') { proc++; foo+=3; } else foo++;
+
+   if (!proc) { pop_n_elems(args-1); return; }
+
+   /* new string len is (foo-bar)-proc*2 */
+   newstr=begin_shared_string((foo-bar)-proc*2);
+   foo=newstr->str;
+   for (proc=0; bar<end; foo++)
+      if (*bar=='%')
+      {
+        if (bar<end-2)
+          *foo=(((bar[1]<'A')?(bar[1]&15):((bar[1]+9)&15))<<4)|
+            ((bar[2]<'A')?(bar[2]&15):((bar[2]+9)&15));
+        else
+          *foo=0;
+        bar+=3;
+      }
+      else { *foo=*(bar++); }
+   pop_n_elems(args);
+   push_string(end_shared_string(newstr));
+}
+
 void f_parse_accessed_database(INT32 args)
 {
-  ptrdiff_t cnum = 0, i;
-  int num = 0;
+  int cnum=0, i, num=0;
   struct array *arg;
   struct mapping *m;
 
   if(!args) {
-    Pike_error("Wrong number of arguments to parse_accessed_database(string).\n");
+    error("Wrong number of arguments to parse_accessed_database(string).\n");
   }
 
   if ((sp[-args].type != T_STRING) || (sp[-args].u.string->size_shift)) {
-    Pike_error("Bad argument 1 to parse_accessed_database(string(8)).\n");
+    error("Bad argument 1 to parse_accessed_database(string(8)).\n");
   }
 
   /* Pop all but the first argument */
@@ -125,7 +166,7 @@ void f_parse_accessed_database(INT32 args)
   f_divide(2);
 
   if (sp[-1].type != T_ARRAY) {
-    Pike_error("Expected array as result of string-division.\n");
+    error("Expected array as result of string-division.\n");
   }
 
   /* The initial string is gone, but the array is there now. */
@@ -135,7 +176,7 @@ void f_parse_accessed_database(INT32 args)
 
   for(i = 0; i < arg->size; i++)
   {
-    ptrdiff_t j=0,k=0;
+    int j=0,k=0;
     char *s=0;
     s=(char *)(ITEM(arg)[i].u.string->str);
     k=(ITEM(arg)[i].u.string->len);
@@ -146,14 +187,14 @@ void f_parse_accessed_database(INT32 args)
       k=atoi(s+j);
       if(k>cnum)
 	cnum=k;
-      push_int(DO_NOT_WARN(k));
+      push_int(k);
       mapping_insert(m, sp-2, sp-1);
       pop_n_elems(2);
     }
   }
   stack_swap();
   pop_stack();
-  push_int(DO_NOT_WARN(cnum));
+  push_int(cnum);
   f_aggregate(2);
 }
 
@@ -169,7 +210,7 @@ void f_parse_html(INT32 args)
       sp[-args].type!=T_STRING||
       sp[1-args].type!=T_MAPPING||
       sp[2-args].type!=T_MAPPING)
-    Pike_error("Bad argument(s) to parse_html.\n");
+    error("Bad argument(s) to parse_html.\n");
 
   ss=sp[-args].u.string;
   if(!ss->len)
@@ -229,7 +270,7 @@ void f_parse_html_lines(INT32 args)
       sp[-args].type!=T_STRING||
       sp[1-args].type!=T_MAPPING||
       sp[2-args].type!=T_MAPPING)
-    Pike_error("Bad argument(s) to parse_html_lines.\n");
+    error("Bad argument(s) to parse_html_lines.\n");
 
   ss=sp[-args].u.string;
   if(!ss->len)
@@ -260,16 +301,13 @@ void f_parse_html_lines(INT32 args)
   SET_ONERROR(sserr, do_free_string, ss);
   strings=0;
   do_html_parse_lines(ss,cont,single,&strings,MAX_PARSE_RECURSE,extra_args,1);
-
   UNSET_ONERROR(sserr);
   UNSET_ONERROR(cerr);
   UNSET_ONERROR(serr);
-
   if(extra_args) {
     UNSET_ONERROR(eerr);
     free_array(extra_args);
   }
-
   free_mapping(cont);
   free_mapping(single);
   if(strings > 1)
@@ -285,14 +323,14 @@ char end_quote_character = '\000';
 void f_set_end_quote(INT32 args)
 {
   if(args < 1 || sp[-1].type != T_INT)
-    Pike_error("Wrong argument to set_end_quote(int CHAR)\n");
+    error("Wrong argument to set_end_quote(int CHAR)\n");
   end_quote_character = sp[-1].u.integer;
 }
 
 void f_set_start_quote(INT32 args)
 {
   if(args < 1 || sp[-1].type != T_INT)
-    Pike_error("Wrong argument to set_start_quote(int CHAR)\n");
+    error("Wrong argument to set_start_quote(int CHAR)\n");
   start_quote_character = sp[-1].u.integer;
 }
 
@@ -308,11 +346,11 @@ void f_set_start_quote(INT32 args)
 #define STARTQUOTE(C) do{PUSH();j=i+1;inquote = 1;endquote=(C);}while(0)
 #define ENDQUOTE() do{PUSH();j++;inquote=0;endquote=0;}while(0)
 
-ptrdiff_t extract_word(char *s, ptrdiff_t i, ptrdiff_t len, int is_SSI_tag)
+int extract_word(char *s, int i, int len, int is_SSI_tag)
 {
   int inquote = 0;
   char endquote = 0;
-  ptrdiff_t j;      /* Start character for this word.. */
+  int j;      /* Start character for this word.. */
   int strs = 0;
 
   SKIP_SPACE();
@@ -381,9 +419,9 @@ done:
 #undef SKIP_SPACE
 
 
-ptrdiff_t push_parsed_tag(char *s, ptrdiff_t len)
+int push_parsed_tag(char *s,int len)
 {
-  ptrdiff_t i=0;
+  int i=0;
   struct svalue *oldsp;
   int is_SSI_tag;
 
@@ -396,7 +434,7 @@ ptrdiff_t push_parsed_tag(char *s, ptrdiff_t len)
 
   while (i<len && s[i]!='>')
   {
-    ptrdiff_t oldi;
+    int oldi;
     oldi = i;
     i = extract_word(s, i, len, is_SSI_tag);
     f_lower_case(1);            /* Since SGML wants us to... */
@@ -415,21 +453,18 @@ ptrdiff_t push_parsed_tag(char *s, ptrdiff_t len)
     }
     if(oldi == i) break;
   }
-  f_aggregate_mapping(DO_NOT_WARN(sp - oldsp));
+  f_aggregate_mapping(sp-oldsp);
   if(i<len) i++;
 
   return i;
 }
 
-INLINE int tagsequal(char *s, char *t, ptrdiff_t len, char *end)
+INLINE int tagsequal(char *s, char *t, int len, char *end)
 {
   if(s+len >= end)  return 0;
 
-  while(len--) {
-    if(tolower(*((unsigned char *)(t++))) !=
-       tolower(*((unsigned char *)(s++))))
-      return 0;
-  }
+  while(len--) if(tolower(*(t++)) != tolower(*(s++)))
+    return 0;
 
   switch(*s) {
   case '>':
@@ -443,12 +478,11 @@ INLINE int tagsequal(char *s, char *t, ptrdiff_t len, char *end)
   }
 }
 
-ptrdiff_t find_endtag(struct pike_string *tag, char *s, ptrdiff_t len,
-		      ptrdiff_t *aftertag)
+int find_endtag(struct pike_string *tag, char *s, int len, int *aftertag)
 {
-  ptrdiff_t num=1;
+  int num=1;
 
-  ptrdiff_t i,j;
+  int i,j;
 
   for (i=j=0; i < len; i++)
   {
@@ -482,7 +516,7 @@ void do_html_parse(struct pike_string *ss,
 		   int *strings,int recurse_left,
 		   struct array *extra_args)
 {
-  ptrdiff_t i,j,k,l,m,len,last;
+  int i,j,k,l,m,len,last;
   char *s;
   struct svalue sval1,sval2;
   struct pike_string *ss2;
@@ -508,7 +542,7 @@ void do_html_parse(struct pike_string *ss,
   {
     if (s[i]=='<')
     {
-      ptrdiff_t n;
+      int n;
       /* skip all spaces */
       i++;
       for (n=i;n<len && ISSPACE(((unsigned char *)s)[n]); n++);
@@ -773,7 +807,7 @@ void do_html_parse_lines(struct pike_string *ss,
 			 struct array *extra_args,
 			 int line)
 {
-  ptrdiff_t i,j,k,l,m,len,last;
+  int i,j,k,l,m,len,last;
   char *s;
   struct svalue sval1,sval2;
   struct pike_string *ss2;
@@ -959,17 +993,29 @@ void do_html_parse_lines(struct pike_string *ss,
   }
 }
 
+#ifndef HAVE_INT_TIMEZONE
+int _tz;
+#else
+extern long int timezone;
+#endif
+
+void f_timezone(INT32 args)
+{
+  pop_n_elems(args);
+#ifndef HAVE_INT_TIMEZONE
+  push_int(_tz);
+#else
+  push_int(timezone);
+#endif
+}
+
 void f_get_all_active_fd(INT32 args)
 {
-  int i,fds,q, ne;
+  int i,fds,q;
   struct stat foo;
 
-  ne = fds_size;
-  if( MAX_OPEN_FILEDESCRIPTORS > ne )
-    ne = MAX_OPEN_FILEDESCRIPTORS;
-
   pop_n_elems(args);
-  for (i=fds=0; i<ne; i++)
+  for (i=fds=0; i<MAX_OPEN_FILEDESCRIPTORS; i++)
   {
     int q;
     THREADS_ALLOW();
@@ -992,7 +1038,7 @@ void f_fd_info(INT32 args)
 
   if (args<1||
       sp[-args].type!=T_INT)
-    Pike_error("Illegal argument to fd_info\n");
+    error("Illegal argument to fd_info\n");
   i=sp[-args].u.integer;
   pop_n_elems(args);
   if (fd_fstat(i,&foo))
@@ -1008,6 +1054,56 @@ void f_fd_info(INT32 args)
   push_string(make_shared_string(buf));
 }
 
+struct pike_string *fd_marks[MAX_OPEN_FILEDESCRIPTORS];
+
+void f_mark_fd(INT32 args)
+{
+  int fd;
+  struct pike_string *s;
+  if (args<1
+      || sp[-args].type!=T_INT
+      || (args>2 && sp[-args+1].type!=T_STRING))
+    error("Illegal argument(s) to mark_fd(int,void|string)\n");
+  fd=sp[-args].u.integer;
+  if(fd>MAX_OPEN_FILEDESCRIPTORS || fd < 0)
+    error("Fd must be in the range 0 to %d\n", MAX_OPEN_FILEDESCRIPTORS);
+  if (args<2)
+  {
+    int len;
+    char *tmp;
+    char buf[20];
+    struct stat fs;
+
+
+    pop_stack();
+    if(!fd_fstat(fd,&fs))
+    {
+      if(fd_marks[fd])
+      {
+	ref_push_string(fd_marks[fd]);
+      } else {
+	push_text("");
+      }
+      return;
+    } else {
+      if(fd_marks[fd])
+      {
+	free_string(fd_marks[fd]);
+	fd_marks[fd]=0;
+      }
+      push_int(0);
+      return;
+    }
+  }
+
+  add_ref(s=sp[-args+1].u.string);
+  if(fd_marks[fd])
+    free_string(fd_marks[fd]);
+  fd_marks[fd]=s;
+  pop_n_elems(args);
+  push_int(0);
+}
+
 static void program_name(struct program *p)
 {
   char *f;
@@ -1061,6 +1157,151 @@ void f__dump_obj_table(INT32 args)
 #define MIN(A,B) ((A)<(B)?(A):(B))
 #endif
 
+#ifdef ENABLE_STREAMED_PARSER
+#include "streamed_parser.h"
+
+static struct program *streamed_parser;
+
+#endif /* ENABLE_STREAMED_PARSER */
+
+extern void init_udp(void);
+extern void init_xml(void);
+extern void exit_xml(void);
+
+
+/* Hohum. Here we go. This is try number three for a more optimized Roxen. */
+
+#ifdef _REENTRANT
+#define BUFFER (8192)
+
+struct thread_args
+{
+  struct thread_args *next;
+  struct object *from;
+  struct object *to;
+  int to_fd, from_fd;
+  struct svalue cb;
+  struct svalue args;
+  int len;
+  int sent;
+  char buffer[BUFFER];
+};
+
+MUTEX_T done_lock;
+struct thread_args *done;
+
+/* WARNING! This function is running _without_ any stack etc. */
+
+#define MY_MIN(a,b) ((a)<(b)?(a):(b))
+void do_shuffle(void *_a)
+{
+  struct thread_args *a = (struct thread_args *)_a;
+
+#ifdef DIRECTIO_ON
+  if(a->len > (65536*2))
+    directio(a->from_fd, DIRECTIO_ON);
+#endif
+
+  while(a->len)
+  {
+    int nread, written=0;
+    nread = fd_read(a->from_fd, a->buffer, MY_MIN(BUFFER,a->len));
+    if(nread <= 0) {
+      if (!nread)
+	break;
+      if(errno == EINTR)
+	continue;
+      else
+	break;
+    }
+
+    while(nread)
+    {
+      int nsent = fd_write(a->to_fd, a->buffer+written, nread);
+      if(nsent < 0) {
+	if(errno != EINTR)
+	  goto end;
+	else
+	  continue;
+      }
+      written += nsent;
+      a->sent += nsent;
+      nread -= nsent;
+      a->len -= nsent;
+    }
+  }
+
+  /* We are done. It is up to the backend callback to call the
+   * finish function
+   */
+ end:
+  mt_lock(&done_lock);
+  a->next = done;
+  done = a;
+  mt_unlock(&done_lock);
+  wake_up_backend();
+}
+
+static int num_shuffles = 0;
+static struct callback *my_callback;
+
+void finished_p(struct callback *foo, void *b, void *c)
+{
+  while(done)
+  {
+    struct thread_args *d;
+
+    mt_lock(&done_lock);
+    d = done;
+    done = d->next;
+    mt_unlock(&done_lock);
+
+    num_shuffles--;
+
+    push_int( d->sent );
+    *(sp++) = d->args;
+    push_object( d->from );
+    push_object( d->to );
+    apply_svalue( &d->cb, 4 );
+    free_svalue( &d->cb );
+    pop_stack();
+    free(d);
+  }
+
+  if(!num_shuffles)
+  {
+    remove_callback( foo );
+    my_callback = 0;
+  }
+}
+
+void f_shuffle(INT32 args)
+{
+  struct thread_args *a = malloc(sizeof(struct thread_args));
+  struct svalue *q, *w;
+  get_all_args("shuffle", args, "%o%o%*%*%d", &a->from, &a->to,&q,&w,&a->len);
+  a->sent = 0;
+
+  num_shuffles++;
+  apply(a->to, "query_fd", 0);
+  apply(a->from, "query_fd", 0);
+  get_all_args("shuffle", 2, "%d%d", &a->to_fd, &a->from_fd);
+
+  add_ref(a->from);
+  add_ref(a->to);
+
+  assign_svalue_no_free(&a->cb, q);
+  assign_svalue_no_free(&a->args, w);
+
+  th_farm(do_shuffle, (void *)a);
+
+  if(!my_callback)
+    my_callback = add_backend_callback( finished_p, 0, 0 );
+
+  pop_n_elems(args+2);
+}
+#endif
+
 
 void pike_module_init(void)
 {
@@ -1068,8 +1309,17 @@ void pike_module_init(void)
   empty_string = sp[-1];
   pop_stack();
 
+
+#ifdef _REENTRANT
+  /* function(object,object,function,mixed,int:void) */
+  ADD_FUNCTION("shuffle", f_shuffle,tFunc(tObj tObj tFunction tMix tInt,tVoid), 0);
+#endif
   ADD_EFUN("_low_program_name", f__low_program_name,tFunc(tProgram,tStr),0);
 
+/* function(string:string) */
+  ADD_EFUN("http_decode_string",f_http_decode_string,tFunc(tStr,tStr),
+	   OPT_TRY_OPTIMIZE);
+
 
 /* function(int:int) */
   ADD_EFUN("set_start_quote",f_set_start_quote,tFunc(tInt,tInt),OPT_EXTERNAL_DEPEND);
@@ -1101,7 +1351,7 @@ void pike_module_init(void)
 				       tOr(tStr,tArr(tStr))))),
 		  tMix,
 		  tStr),
-	   OPT_SIDE_EFFECT);
+	   0);
 
 
   ADD_EFUN("parse_html_lines",f_parse_html_lines,
@@ -1129,25 +1379,77 @@ void pike_module_init(void)
 /* function(int,void|int:int) */
   ADD_EFUN("stardate", f_stardate,tFunc(tInt tOr(tVoid,tInt),tInt), 0);
 
+/* function(:int) */
+  ADD_EFUN("timezone", f_timezone,tFunc(tNone,tInt), 0);
+
 /* function(:array(int)) */
   ADD_EFUN("get_all_active_fd", f_get_all_active_fd,tFunc(tNone,tArr(tInt)),
 	   OPT_EXTERNAL_DEPEND);
 
+/* function(int:int) */
+  ADD_EFUN("nice", f_nice,tFunc(tInt,tInt),
+	   OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
+
 /* function(int:string) */
   ADD_EFUN("fd_info", f_fd_info,tFunc(tInt,tStr), OPT_EXTERNAL_DEPEND);
+
+/* function(int,void|mixed:mixed) */
+  ADD_EFUN("mark_fd", f_mark_fd,tFunc(tInt tOr(tVoid,tMix),tMix),
+	   OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
+
+  /* timezone() needs */
   {
-    extern void init_xml();
-    init_xml();
+    time_t foo = (time_t)0;
+    struct tm *g;
+
+    g = localtime(&foo);
+#ifndef HAVE_INT_TIMEZONE
+    _tz = g->tm_gmtoff;
+#endif
   }
+
+#ifdef ENABLE_STREAMED_PARSER
+  start_new_program();
+  add_storage( sizeof (struct streamed_parser) );
+  /* function(mapping(string:function(string,mapping(string:string),mixed:mixed)),mapping(string:function(string,mapping(string:string),string,mixed:mixed)),mapping(string:function(string,mixed:mixed)):void) */
+  ADD_FUNCTION( "init", streamed_parser_set_data,tFunc(tMap(tStr,tFunc(tStr tMap(tStr,tStr) tMix,tMix)) tMap(tStr,tFunc(tStr tMap(tStr,tStr) tStr tMix,tMix)) tMap(tStr,tFunc(tStr tMix,tMix)),tVoid), 0 );
+  /* function(string,mixed:string) */
+  ADD_FUNCTION( "parse", streamed_parser_parse,tFunc(tStr tMix,tStr), 0 );
+  /* function(void:string) */
+  ADD_FUNCTION( "finish", streamed_parser_finish,tFunc(tVoid,tStr), 0 );
+  set_init_callback( streamed_parser_init );
+  set_exit_callback( streamed_parser_destruct );
+
+  streamed_parser = end_program();
+  add_program_constant("streamed_parser", streamed_parser,0);
+#endif /* ENABLE_STREAMED_PARSER */
+
+  init_xml();
 }
 
 
 void pike_module_exit(void)
 {
   int i;
+
   free_string(empty_string.u.string);
+
+#ifdef ENABLE_STREAMED_PARSER
+  if(streamed_parser)
   {
-    extern void exit_xml();
-    exit_xml();
+    free_program(streamed_parser);
+    streamed_parser=0;
   }
+#endif /* ENABLE_STREAMED_PARSER */
+
+  for(i=0; i<MAX_OPEN_FILEDESCRIPTORS; i++)
+  {
+    if(fd_marks[i])
+    {
+      free_string(fd_marks[i]);
+      fd_marks[i]=0;
+    }
+  }
+
+  exit_xml();
 }
