@@ -1,9 +1,8 @@
-/*
-|| This file is part of Pike. For copyright information see COPYRIGHT.
-|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
-|| for more information.
-|| $Id: sprintf.c,v 1.118 2004/03/21 17:14:47 nilsson Exp $
-*/
+/*\
+||| This file a part of Pike, and is copyright by Fredrik Hubinette
+||| Pike is distributed as GPL (General Public License)
+||| See the files COPYING and DISCLAIMER for more information.
+\*/
 
 /* TODO: use ONERROR to cleanup fsp */
 
@@ -16,7 +15,55 @@
   deallocated at next call of sprintf. Most operators doesn't need this
   feature though as they allocate their buffers with alloca() or simply use
   pointers into other strings.
-  It also has a lot more features.
+  It also has a lot more features:
+
+  Modifiers:
+    0  Zero pad numbers (implies right justification)
+    !  Toggle truncation
+       pad positive integers with a space
+    +  pad positive integers with a plus sign
+    -  left adjusted within field size (default is right)
+    |  centered within field size
+    =  column mode if strings are greater than field size
+    /  Rough linebreak (break at exactly fieldsize instead of between words)
+    #  table mode, print a list of '\n' separated word (top-to-bottom order)
+    $  Inverse table mode (left-to-right order)
+    n  (where n is a number or *) a number specifies field size
+   .n  set precision
+   :n  set field size & precision
+   ;n  Set column width
+    *  if n is a * then next argument is used
+    ~  get pad string from argument list
+   'X'  Set a pad string. ' cannot be a part of the pad_string (yet)
+    <  Use same arg again
+    ^  repeat this on every line produced
+    @  do this format for each entry in argument array
+    > Put the string at the bottom end of column instead of top
+    _ Set width to size of length of data
+
+ Operators:
+   %% percent
+   %d signed decimal int
+   %u unsigned decimal int
+   %o unsigned octal int
+   %x lowercase unsigned hexadecimal int
+   %X uppercase unsigned hexadecimal int
+   %c char
+   %f float
+   %g heuristically chosen representation of float
+   %e exponential notation float
+   %s string
+   %O any type (prettyprint)
+   %n nop
+   %t type of argument
+   %<modifiers>{format%}  do a format for every index in an array.
+
+   Most flags and operators are combinable in any fashion, but _really_
+   strange results can arise from things like:
+      sprintf("%+080#{%s\n%}",get_dir("/"))
+
+ Callbacks:
+   string _sprintf(int operator, mapping modifiers);
 
  Ideas yet to be implemented:
    Line-break with fill? Lower-case? Upper case? Capitalize?
@@ -30,7 +77,7 @@
 
    A short 'people' function (without sort)
 
-   sprintf("%{%-14s %2d %-30s %s\n%}\n",map(users(),lambda(object x)
+   sprintf("%{%-14s %2d %-30s %s\n%}\n",map_array(users(),lambda(object x)
        {
            return ({x->query_real_name(),
 		x->query_level(),
@@ -46,7 +93,7 @@
    debug-dump
    sprintf("%78*O",width,foo);
 
-   A newspaper
+   A newspape
    sprintf("%-=*s %-=*s\n",width/2,article1,width/2,article2);
 
    A 'dotted-line' pricelist row 
@@ -54,232 +101,9 @@
 
 */
 
-/*! @decl string sprintf(string format, mixed ... args)
- *!
- *!   Print formated output to string.
- *!
- *!   The @[format] string is a string containing a description of how to
- *!   output the data in @[args]. This string should generally speaking
- *!   have one @tt{%@i{<modifiers>@}@i{<operator>@}@} format specifier
- *!   (examples: @tt{%s@}, @tt{%0d@}, @tt{%-=20s@}) for each of the arguments.
- *!
- *!   The following modifiers are supported:
- *!   @int
- *!     @value '0'
- *!       Zero pad numbers (implies right justification).
- *!     @value '!'
- *!       Toggle truncation.
- *!     @value ' '
- *!       Pad positive integers with a space.
- *!     @value '+'
- *!       Pad positive integers with a plus sign.
- *!     @value '-'
- *!       Left adjust within field size (default is right).
- *!     @value '|'
- *!       Centered within field size.
- *!     @value '='
- *!       Column mode if strings are greater than field size.
- *!     @value '/'
- *!       Rough line break (break at exactly field size instead of between
- *!       words).
- *!     @value '#'
- *!       Table mode, print a list of '\n' separated word (top-to-bottom
- *!       order).
- *!     @value '$'
- *!       Inverse table mode (left-to-right order).
- *!     @value 'n'
- *!       (Where n is a number or *) field size specifier.
- *!     @value '.n'
- *!       Precision specifier.
- *!     @value ':n'
- *!       Field size precision specifier.
- *!     @value ';n'
- *!       Column width specifier.
- *!     @value '*'
- *!       If n is a * then next argument is used for precision/field size.
- *!     @value "'"
- *!       Set a pad string. @tt{'@} cannot be a part of the pad string (yet).
- *!     @value '~'
- *!       Get pad string from argument list.
- *!     @value '<'
- *!       Use same argument again.
- *!     @value '^'
- *!       Repeat this on every line produced.
- *!     @value '@'
- *!       Repeat this format for each element in the argument array.
- *!     @value '>'
- *!       Put the string at the bottom end of column instead of top.
- *!     @value '_'
- *!       Set width to the length of data.
- *!     @value '[n]'
- *!       Select argument number @tt{@i{n@}@}. Use @tt{*@} to use the next
- *!       argument as selector.
- *!   @endint
- *!
- *!   The following operators are supported:
- *!   @int
- *!     @value '%'
- *!       Percent.
- *!     @value 'b'
- *!       Signed binary integer.
- *!     @value 'd'
- *!       Signed decimal integer.
- *!     @value 'u'
- *!       Unsigned decimal integer.
- *!     @value 'o'
- *!       Signed octal integer.
- *!     @value 'x'
- *!       Lowercase signed hexadecimal integer.
- *!     @value 'X'
- *!       Uppercase signed hexadecimal integer.
- *!     @value 'c'
- *!       Character. If a fieldsize has been specified this will output
- *!       the low-order bytes of the integer in network (big endian) byte
- *!       order. To get little endian byte order, negate the field size.
- *!     @value 'f'
- *!       Float. (Locale dependent formatting.)
- *!     @value 'g'
- *!       Heuristically chosen representation of float.
- *!       (Locale dependent formatting.)
- *!     @value 'G'
- *!       Like @tt{%g@}, but uses uppercase @tt{E@} for exponent.
- *!     @value 'e'
- *!       Exponential notation float. (Locale dependent output.)
- *!     @value 'E'
- *!       Like @tt{%e@}, but uses uppercase @tt{E@} for exponent.
- *!     @value 'F'
- *!       Binary IEEE representation of float (@tt{%4F@} gives 
- *!       single precision, @tt{%8F@} gives double precision.)
- *!     @value 's'
- *!       String.
- *!     @value 'O'
- *!       Any value, debug style. Do not rely on the exact formatting;
- *!       how the result looks can vary depending on locale, phase of
- *!       the moon or anything else the _sprintf method implementor
- *!       wanted for debugging.
- *!     @value 'n'
- *!       No operation (ignore the argument).
- *!     @value 't'
- *!       Type of the argument.
- *!     @value '{'
- *!     @value '}'
- *!       Perform the enclosed format for every element of the argument array.
- *!   @endint
- *!
- *!   Most modifiers and operators are combinable in any fashion, but some
- *!   combinations may render strange results.
- *!
- *!   If an argument is an object that implements @[lfun::_sprintf()], that
- *!   callback will be called with the operator as the first argument, and
- *!   the current modifiers as the second. The callback is expected to return
- *!   a string.
- *!
- *! @example
- *! Pike v7.4 release 13 running Hilfe v3.5 (Incremental Pike Frontend)
- *! > sprintf("The unicode character %c has character code %04X.", 'A', 'A');
- *! (1) Result: "The unicode character A has character code 0041."
- *! > sprintf("#%@@02X is the HTML code for purple.", Image.Color.purple->rgb());
- *! (2) Result: "#A020F0 is the HTML code for purple."
- *! > int n=4711;
- *! > sprintf("%d = hexadecimal %x = octal %o = %b binary", n, n, n, n);
- *! (3) Result: "4711 = hexadecimal 1267 = octal 11147 = 1001001100111 binary"
- *!
- *! @note
- *!   sprintf style formatting is applied by many formatting functions, such
- *!   @[write()] and @[werror].
- *!
- *! @example
- *! > write(#"Formatting examples:
- *! Left adjusted  [%-10d]
- *! Centered       [%|10d]
- *! Right adjusted [%10d]
- *! Zero padded    [%010d]
- *! ", n, n, n, n);
- *! Formatting examples:
- *! Left adjusted  [4711      ]
- *! Centered       [   4711   ]
- *! Right adjusted [      4711]
- *! Zero padded    [0000004711]
- *! (5) Result: 142
- *! int screen_width=70;
- *! > write("%-=*s\n", screen_width,
- *! >> "This will wordwrap the specified string within the "+
- *! >> "specified field size, this is useful say, if you let "+
- *! >> "users specify their screen size, then the room "+
- *! >> "descriptions will automagically word-wrap as appropriate.\n"+
- *! >> "slosh-n's will of course force a new-line when needed.\n");
- *! This will wordwrap the specified string within the specified field
- *! size, this is useful say, if you let users specify their screen size,
- *! then the room descriptions will automagically word-wrap as
- *! appropriate.
- *! slosh-n's will of course force a new-line when needed.
- *! (6) Result: 355
- *! > write("%-=*s %-=*s\n", screen_width/2,
- *! >> "Two columns next to each other (any number of columns will "+
- *! >> "of course work) independantly word-wrapped, can be useful.",
- *! >> screen_width/2-1,
- *! >> "The - is to specify justification, this is in addherence "+
- *! >> "to std sprintf which defaults to right-justification, "+
- *! >> "this version also supports centre and right justification.");
- *! Two columns next to each other (any The - is to specify justification,
- *! number of columns will of course    this is in addherence to std
- *! work) independantly word-wrapped,   sprintf which defaults to
- *! can be useful.                      right-justification, this version
- *!                                     also supports centre and right
- *!                                     justification.
- *! (7) Result: 426
- *! > write("%-$*s\n", screen_width,
- *! >> "Given a\nlist of\nslosh-n\nseparated\n'words',\nthis option\n"+
- *! >> "creates a\ntable out\nof them\nthe number of\ncolumns\n"+
- *! >> "be forced\nby specifying a\npresision.\nThe most obvious\n"+
- *! >> "use is for\nformatted\nls output.");
- *! Given a          list of          slosh-n
- *! separated        'words',         this option
- *! creates a        table out        of them
- *! the number of    columns          be forced
- *! by specifying a  presision.       The most obvious
- *! use is for       formatted        ls output.
- *! (8) Result: 312
- *! > write("%-#*s\n", screen_width,
- *! >> "Given a\nlist of\nslosh-n\nseparated\n'words',\nthis option\n"+
- *! >> "creates a\ntable out\nof them\nthe number of\ncolumns\n"+
- *! >> "be forced\nby specifying a\npresision.\nThe most obvious\n"+
- *! >> "use is for\nformatted\nls output.");
- *! Given a          creates a        by specifying a
- *! list of          table out        presision.
- *! slosh-n          of them          The most obvious
- *! separated        the number of    use is for
- *! 'words',         columns          formatted
- *! this option      be forced        ls output.
- *! (9) Result: 312
- *! > sample = ([ "align":"left", "valign":"middle" ]);
- *! (10) Result: ([ @xml{/@}* 2 elements *@xml{/@}
- *!          "align":"left",
- *!          "valign":"middle"
- *!        ])
- *! > write("<td%{ %s='%s'%}>\n", (array)sample);
- *! <td valign='middle' align='left'>
- *! (11) Result: 34
- *! >  write("Of course all the simple printf options "+
- *! >> "are supported:\n %s: %d %x %o %c\n",
- *! >> "65 as decimal, hex, octal and a char",
- *! >> 65, 65, 65, 65);
- *! Of course all the simple printf options are supported:
- *!  65 as decimal, hex, octal and a char: 65 41 101 A
- *! (12) Result: 106
- *! > write("%[0]d, %[0]x, %[0]X, %[0]o, %[0]c\n", 75);
- *! 75, 4b, 4B, 113, K
- *! (13) Result: 19
- *! > write("%|*s\n",screen_width, "THE END");
- *!                                THE END
- *! (14) Result: 71
- *!
- *! @seealso
- *!   @[lfun::_sprintf()]
- */
 #include "global.h"
-RCSID("$Id: sprintf.c,v 1.118 2004/03/21 17:14:47 nilsson Exp $");
-#include "pike_error.h"
+RCSID("$Id: sprintf.c,v 1.62 2000/03/29 22:15:46 hubbe Exp $");
+#include "error.h"
 #include "array.h"
 #include "svalue.h"
 #include "stralloc.h"
@@ -294,17 +118,19 @@ RCSID("$Id: sprintf.c,v 1.118 2004/03/21 17:14:47 nilsson Exp $");
 #include "mapping.h"
 #include "builtin_functions.h"
 #include "operators.h"
-#include "opcodes.h"
-#include "cyclic.h"
-#include "module.h"
-#include "pike_float.h"
 #include <ctype.h>
 
-#include "config.h"
+#ifdef PC
+#undef PC
+#endif /* PC */
 
 #include <math.h>
-
-#define sp Pike_sp
+#ifdef HAVE_IEEEFP_H
+#include <ieeefp.h>
+#endif
+#ifdef HAVE_FP_CLASS_H
+#include <fp_class.h>
+#endif
 
 #define FORMAT_INFO_STACK_SIZE 200
 #define RETURN_SHARED_STRING
@@ -316,16 +142,16 @@ struct format_info
   char *fi_free_string;
   struct pike_string *to_free_string;
   PCHARP b;
-  ptrdiff_t len;
-  ptrdiff_t width;
+  int len;
+  int width;
   int precision;
   PCHARP pad_string;
-  ptrdiff_t pad_length;
+  int pad_length;
   int column_entries;
   short flags;
   char pos_pad;
   int column_width;
-  ptrdiff_t column_modulo;
+  int column_modulo;
 };
 
 /* FIXME:
@@ -342,7 +168,7 @@ struct format_stack
 
 #define FIELD_LEFT 1
 #define FIELD_CENTER 2
-#define PAD_POSITIVE 4
+#define PAD_POSITIVE 4 
 #define LINEBREAK 8
 #define COLUMN_MODE 16
 #define ZERO_PAD 32
@@ -452,9 +278,11 @@ INLINE static void low_write_IEEE_float(char *b, double d, int sz)
     e = maxexp;
   else
 #endif
-  if(PIKE_ISNAN(d)) {
+#ifdef HAVE_ISNAN
+  if(isnan(d)) {
     e = maxexp; f = maxf;
   } else
+#endif
 #ifdef HAVE_ISZERO
   if(iszero(d))
     e = 0;
@@ -536,14 +364,14 @@ INLINE static void low_write_IEEE_float(char *b, double d, int sz)
 
 INLINE static void fix_field(struct string_builder *r,
 			     PCHARP b,
-			     ptrdiff_t len,
+			     int len,
 			     int flags,
-			     ptrdiff_t width,
+			     int width,
 			     PCHARP pad_string,
-			     ptrdiff_t pad_length,
+			     int pad_length,
 			     char pos_pad)
 {
-  ptrdiff_t e;
+  int e,d;
   if(!width || width==SPRINTF_UNDECIDED)
   {
     if(pos_pad && EXTRACT_PCHARP(b)!='-') string_builder_putchar(r,pos_pad);
@@ -551,89 +379,75 @@ INLINE static void fix_field(struct string_builder *r,
     return;
   }
 
+  d=0;
   if(!(flags & DO_TRUNC) && len+(pos_pad && EXTRACT_PCHARP(b)!='-')>=width)
   {
     if(pos_pad && EXTRACT_PCHARP(b)!='-') string_builder_putchar(r,pos_pad);
     string_builder_append(r,b,len);
     return;
   }
-
-  if (flags & (ZERO_PAD|FIELD_CENTER|FIELD_LEFT)) {
-    /* Some flag is set. */
-
-    if(flags & ZERO_PAD)		/* zero pad is kind of special... */
+  if(flags & ZERO_PAD)		/* zero pad is kind of special... */
+  {
+    if(EXTRACT_PCHARP(b)=='-')
     {
-      if(EXTRACT_PCHARP(b)=='-')
+      string_builder_putchar(r,'-');
+      INC_PCHARP(b,1);
+      len--;
+      width--;
+    }else{
+      if(pos_pad)
       {
-  	string_builder_putchar(r,'-');
-  	INC_PCHARP(b,1);
-  	len--;
-  	width--;
-      }else{
-  	if(pos_pad)
-  	{
-  	  string_builder_putchar(r,pos_pad);
-  	  width--;
-  	}
+        string_builder_putchar(r,pos_pad);
+        width--;
       }
+    }
 #if 1
-      string_builder_fill(r,width-len,MKPCHARP("0",0),1,0);
+    string_builder_fill(r,width-len,MKPCHARP("0",0),1,0);
 #else
-      for(;width>len;width--) string_builder_putchar(r,'0');
+    for(;width>len;width--) string_builder_putchar(r,'0');
 #endif
-      string_builder_append(r,b,len);
-      return;
-    }
-  
-    if (flags & (FIELD_CENTER|FIELD_LEFT)) {
-      ptrdiff_t d=0;
-  
-      if(flags & FIELD_CENTER)
-      {
-  	e=len;
-  	if(pos_pad && EXTRACT_PCHARP(b)!='-') e++;
-  	e=(width-e)/2;
-  	if(e>0)
-  	{
-  	  string_builder_fill(r, e, pad_string, pad_length, 0);
-  	  width-=e;
-  	}
-      }
-  
-      /* Left adjust */
-      if(pos_pad && EXTRACT_PCHARP(b)!='-')
-      {
-  	string_builder_putchar(r,pos_pad);
-  	width--;
-  	d++;
-      }
+    string_builder_append(r,b,len);
+    return;
+  }
 
-#if 1
-      len = MINIMUM(width, len);
-      if (len) {
-        d += len;
-	string_builder_append(r, b, len);
-	width -= len;
-      }
-#else /* 0 */  
-      d+=MINIMUM(width,len);
-      while(len && width)
-      {
-  	string_builder_putchar(r,EXTRACT_PCHARP(b));
-  	INC_PCHARP(b,1);
-  	len--;
-  	width--;
-      }
-#endif /* 1 */
-  
-      if(width>0)
-      {
-	d%=pad_length;
-	string_builder_fill(r, width, pad_string, pad_length, d);
-      }
-      
-      return;
+  if(flags & FIELD_CENTER)
+  {
+    e=len;
+    if(pos_pad && EXTRACT_PCHARP(b)!='-') e++;
+    e=(width-e)/2;
+    if(e>0)
+    {
+      string_builder_fill(r,e,pad_string, pad_length, 0);
+      width-=e;
     }
+    flags|=FIELD_LEFT;
+  }
+
+  if(flags & FIELD_LEFT)
+  {
+    if(pos_pad && EXTRACT_PCHARP(b)!='-')
+    {
+      string_builder_putchar(r,pos_pad);
+      width--;
+      d++;
+    }
+
+    d+=MINIMUM(width,len);
+    while(len && width)
+    {
+      string_builder_putchar(r,EXTRACT_PCHARP(b));
+      INC_PCHARP(b,1);
+      len--;
+      width--;
+    }
+
+    if(width>0)
+    {
+      d%=pad_length;
+      string_builder_fill(r,width,pad_string,pad_length,d);
+    }
+    
+    return;
   }
 
   /* Right-justification */
@@ -642,7 +456,7 @@ INLINE static void fix_field(struct string_builder *r,
   e=width-len;
   if(e>0)
   {
-    string_builder_fill(r, e, pad_string, pad_length, 0);
+    string_builder_fill(r,e,pad_string, pad_length, 0);
     width-=e;
   }
 
@@ -677,7 +491,7 @@ static void sprintf_error(struct format_stack *fs,
   va_start(args,s);
   free_sprintf_strings(fs);
 
-  sprintf(buf,"sprintf: %s",s);
+  sprintf(buf,"Sprintf: %s",s);
   va_error(buf,args);
   va_end(args);
 }
@@ -692,7 +506,7 @@ INLINE static int do_one(struct format_stack *fs,
 			 struct format_info *f)
 {
   PCHARP rest;
-  ptrdiff_t e, d, lastspace;
+  int e,d,lastspace;
 
   rest.ptr=0;
   if(f->flags & (LINEBREAK|ROUGH_LINEBREAK))
@@ -772,8 +586,7 @@ INLINE static int do_one(struct format_stack *fs,
   }
   else if(f->flags & COLUMN_MODE)
   {
-    ptrdiff_t mod;
-    ptrdiff_t col;
+    int mod,col;
     PCHARP end;
 
     if(f->width==SPRINTF_UNDECIDED)
@@ -915,8 +728,7 @@ INLINE static int do_one(struct format_stack *fs,
      fs->fsp->pad_string=MKPCHARP(" ",0);				\
      fs->fsp->pad_length=1;						\
      fs->fsp->column_width=0;						\
-     fs->fsp->pos_pad=0;						\
-     fs->fsp->flags=0;							\
+     fs->fsp->pos_pad=fs->fsp->flags=0;					\
      fs->fsp->width=fs->fsp->precision=SPRINTF_UNDECIDED;		\
      UNSET_ONERROR(_e);							\
      break;                                                             \
@@ -930,59 +742,51 @@ INLINE static int do_one(struct format_stack *fs,
 	  PEEK_SVALUE(sv);						      \
 	  if(sv->type == T_OBJECT && sv->u.object->prog)		      \
 	  {                                                                   \
-            ptrdiff_t fun=FIND_LFUN(sv->u.object->prog, LFUN__SPRINTF);	      \
-	    if (fun != -1) {						      \
-              DECLARE_CYCLIC();						      \
-              if (!BEGIN_CYCLIC(sv->u.object, fun))			      \
-              {                                                               \
-              	int n=0;						      \
-	      	push_int(EXTRACT_PCHARP(a));				      \
-	      	if (fs->fsp->precision!=SPRINTF_UNDECIDED)		      \
-	      	{							      \
-	      	   push_constant_text("precision");			      \
-	      	   push_int(fs->fsp->precision);			      \
-              	   n+=2;						      \
-	      	}							      \
-	      	if (fs->fsp->width!=SPRINTF_UNDECIDED)			      \
-	      	{							      \
-	      	   push_constant_text("width");	           		      \
-	      	   push_int64(fs->fsp->width);				      \
-              	   n+=2;						      \
-	      	}							      \
-	      	if ((fs->fsp->flags&FIELD_LEFT))			      \
-	      	{							      \
-	      	   push_constant_text("flag_left");	       		      \
-	      	   push_int(1);						      \
-              	   n+=2;						      \
-	      	}							      \
-	      	f_aggregate_mapping(n);					      \
-	      								      \
-	      	SET_CYCLIC_RET(1);					      \
-	      	apply_low(sv->u.object, fun, 2);                              \
+            int fun=FIND_LFUN(sv->u.object->prog, LFUN__SPRINTF);	      \
+	    if (fun != -1)                                                    \
+            {                                                                 \
+              int n=0;							      \
+	      push_int(EXTRACT_PCHARP(a));				      \
+	      if (fs->fsp->precision!=SPRINTF_UNDECIDED)		      \
+	      {								      \
+		 push_constant_text("precision");			      \
+		 push_int(fs->fsp->precision);				      \
+                 n+=2;							      \
+	      }								      \
+	      if (fs->fsp->width!=SPRINTF_UNDECIDED)			      \
+	      {								      \
+		 push_constant_text("width");	           		      \
+		 push_int(fs->fsp->width);				      \
+                 n+=2;							      \
+	      }								      \
+	      if ((fs->fsp->flags&FIELD_LEFT))				      \
+	      {								      \
+		 push_constant_text("flag_left");	       		      \
+		 push_int(1);						      \
+                 n+=2;							      \
+	      }								      \
+	      f_aggregate_mapping(n);					      \
 									      \
-	      	if(sp[-1].type == T_STRING)				      \
-	      	{	                                                      \
-              	  DO_IF_DEBUG( if(fs->fsp->to_free_string)                    \
-              		       Pike_fatal("OOps in sprintf\n"); )                  \
-              	  fs->fsp->to_free_string = (--sp)->u.string;	              \
-	      								      \
-	      	  fs->fsp->b = MKPCHARP_STR(fs->fsp->to_free_string);	      \
-	      	  fs->fsp->len = fs->fsp->to_free_string->len;		      \
-	      								      \
-              	  /* We have to lift one argument from the format stack. */   \
-              	  GET_SVALUE(sv);                                             \
-                  END_CYCLIC();						      \
-	      	  break;						      \
-	      	}							      \
-	      	if(!SAFE_IS_ZERO(sp-1))					      \
-	      	{							      \
-	      	   sprintf_error(fs,"argument %d (object) returned "	      \
-	      			 "illegal value from _sprintf()\n",	      \
-				 argument+1);				      \
-	      	}							      \
-	      	pop_stack();						      \
-              }								      \
-              END_CYCLIC();						      \
+	      apply_low(sv->u.object, fun, 2);                                \
+	      if(sp[-1].type == T_STRING)				      \
+	      {	                                                              \
+                DO_IF_DEBUG( if(fs->fsp->to_free_string)                      \
+                             fatal("OOps in sprintf\n"); )                    \
+                fs->fsp->to_free_string = (--sp)->u.string;	              \
+									      \
+		fs->fsp->b = MKPCHARP_STR(fs->fsp->to_free_string);           \
+		fs->fsp->len = fs->fsp->to_free_string->len;		      \
+	       								      \
+                /* We have to lift one argument from the format stack. */     \
+                GET_SVALUE(sv);                                               \
+		break;							      \
+	      }								      \
+              if(!IS_ZERO(sp-1))					      \
+	      {								      \
+		 sprintf_error(fs,"argument %d (object) returned "	      \
+			       "illegal value from _sprintf()\n",argument+1); \
+	      }								      \
+	      pop_stack();						      \
 	    }								      \
 	  }								      \
 	}
@@ -995,16 +799,16 @@ INLINE static int do_one(struct format_stack *fs,
 static void low_pike_sprintf(struct format_stack *fs,
 			     struct string_builder *r,
 			     PCHARP format,
-			     ptrdiff_t format_len,
+			     int format_len,
 			     struct svalue *argp,
-			     ptrdiff_t num_arg,
+			     int num_arg,
 			     int nosnurkel)
 {
   int argument=0;
-  int tmp,setwhat,d,e;
+  int tmp,setwhat,pos,d,e;
   char buffer[140];
   struct format_info *f,*start;
-  double tf;
+  float tf;
   struct svalue *arg=0;	/* pushback argument */
   struct svalue *lastarg=0;
 
@@ -1016,20 +820,19 @@ static void low_pike_sprintf(struct format_stack *fs,
   {
     int num_snurkel;
 
-    if(fs->fsp-fs->format_info_stack==FORMAT_INFO_STACK_SIZE - 1)
-      sprintf_error(fs, "Sprintf stack overflow.\n");
     fs->fsp++;
 #ifdef PIKE_DEBUG
     if(fs->fsp < fs->format_info_stack)
-      Pike_fatal("sprintf: fs->fsp out of bounds.\n");
+      fatal("sprintf: fs->fsp out of bounds.\n");
 #endif
+    if(fs->fsp-fs->format_info_stack==FORMAT_INFO_STACK_SIZE)
+      sprintf_error(fs, "Sprintf stack overflow.\n");
     fs->fsp->pad_string=MKPCHARP(" ",0);
     fs->fsp->pad_length=1;
     fs->fsp->fi_free_string=0;
     fs->fsp->to_free_string=0;
     fs->fsp->column_width=0;
-    fs->fsp->pos_pad = 0;
-    fs->fsp->flags = 0;
+    fs->fsp->pos_pad=fs->fsp->flags=0;
     fs->fsp->width=fs->fsp->precision=SPRINTF_UNDECIDED;
 
     if(EXTRACT_PCHARP(a)!='%')
@@ -1044,7 +847,7 @@ static void low_pike_sprintf(struct format_stack *fs,
     }
     num_snurkel=0;
     arg=NULL;
-    setwhat=0;
+    setwhat=pos=0;
     begin=a;
 
     for(INC_PCHARP(a,1);;INC_PCHARP(a,1))
@@ -1060,7 +863,7 @@ static void low_pike_sprintf(struct format_stack *fs,
 	}else{
 	  sprintf_error(fs, "Error in format string, \\%o is not a format.\n",EXTRACT_PCHARP(a));
 	}
-	Pike_fatal("Foo, you shouldn't be here!\n");
+	fatal("Foo, you shouldn't be here!\n");
 
         /* First the modifiers */
 
@@ -1083,15 +886,14 @@ static void low_pike_sprintf(struct format_stack *fs,
       got_arg:
 	switch(setwhat)
 	{
-	case 0:
-	case 1:
-	  if(tmp < 0) sprintf_error(fs, "Illegal width %d.\n", tmp);
-	  fs->fsp->width=tmp;
-	  if (!setwhat) break;
+	case 0: fs->fsp->width=tmp; break;
+	case 1: fs->fsp->width=tmp;
 	case 2: fs->fsp->precision=tmp; break;
 	case 3: fs->fsp->column_width=tmp; break;
 	case 4: fs->fsp->precision=-tmp; break;
 	}
+	if(fs->fsp->width!=SPRINTF_UNDECIDED && fs->fsp->width<1)
+	  sprintf_error(fs, "Illegal width.\n");
 	continue;
 
       case ';': setwhat=3; continue;
@@ -1141,10 +943,8 @@ static void low_pike_sprintf(struct format_stack *fs,
       {
 	struct pike_string *s;
 	GET_STRING(s);
-	if (s->len) {
-	  fs->fsp->pad_string=MKPCHARP_STR(s);
-	  fs->fsp->pad_length=s->len;
-	}
+	fs->fsp->pad_string=MKPCHARP_STR(s);
+	fs->fsp->pad_length=s->len;
 	continue;
       }
 
@@ -1154,20 +954,6 @@ static void low_pike_sprintf(struct format_stack *fs,
 	arg=lastarg;
 	continue;
 
-      case '[':
-	INC_PCHARP(a,1);
-	if(EXTRACT_PCHARP(a)=='*') {
-	  GET_INT(tmp);
-	  INC_PCHARP(a,1);
-	} else
-	  tmp=STRTOL_PCHARP(a,&a,10);
-	if(EXTRACT_PCHARP(a)!=']') 
-	  sprintf_error(fs, "Expected ] in format string, not %c.\n",EXTRACT_PCHARP(a));
-	if(tmp >= num_arg)
-	  sprintf_error(fs, "Not enough arguments to [%d].\n",tmp);
-	arg = argp+tmp;
-	continue;
-	
         /* now the real operators */
 
       case '{':
@@ -1180,11 +966,13 @@ static void low_pike_sprintf(struct format_stack *fs,
 	DO_OP();
 	for(e=1,tmp=1;tmp;e++)
 	{
-	  if (!INDEX_PCHARP(a,e) &&
-	      !COMPARE_PCHARP(ADD_PCHARP(a,e),<,format_end)) {
+	  if(!INDEX_PCHARP(a,e))
+	  {
 	    sprintf_error(fs, "Missing %%} in format string.\n");
 	    break;		/* UNREACHED */
-	  } else if(INDEX_PCHARP(a,e)=='%') {
+	  }
+	  if(INDEX_PCHARP(a,e)=='%')
+	  {
 	    switch(INDEX_PCHARP(a,e+1))
 	    {
 	    case '%': e++; break;
@@ -1224,9 +1012,9 @@ static void low_pike_sprintf(struct format_stack *fs,
 	  }
 #ifdef PIKE_DEBUG
 	  if(fs->fsp < fs->format_info_stack)
-	    Pike_fatal("sprintf: fs->fsp out of bounds.\n");
+	    fatal("sprintf: fs->fsp out of bounds.\n");
 	  if(fs->fsp!=fsp_save)
-	    Pike_fatal("sprintf: fs->fsp incorrect after recursive sprintf.\n");
+	    fatal("sprintf: fs->fsp incorrect after recursive sprintf.\n");
 #endif
 	  fs->fsp->b=MKPCHARP_STR(b.s);
 	  fs->fsp->len=b.s->len;
@@ -1262,8 +1050,7 @@ static void low_pike_sprintf(struct format_stack *fs,
 
       case 'c':
       {
-        INT_TYPE tmp;
-	ptrdiff_t l,n;
+        INT32 l,tmp,n;
 	char *x;
         DO_OP();
 	CHECK_OBJECT_SPRINTF()
@@ -1350,7 +1137,7 @@ static void low_pike_sprintf(struct format_stack *fs,
 	if(base)
 	{
 	  char *p = x;
-	  ptrdiff_t l;
+	  int l;
 
 	  if(mask_size || val>=0)
 	  {
@@ -1382,9 +1169,9 @@ static void low_pike_sprintf(struct format_stack *fs,
 	  }
 	}
 	else if(mode == 'u')
-	  sprintf(x, "%"PRINTPIKEINT"u", (unsigned INT_TYPE) val);
+	  sprintf(x, "%u", val);
 	else
-	  sprintf(x, "%"PRINTPIKEINT"d", val);
+	  sprintf(x, "%d", val);
 
 	fs->fsp->len=strlen(x);
 	break;
@@ -1399,51 +1186,27 @@ static void low_pike_sprintf(struct format_stack *fs,
 	char *x;
 	DO_OP();
 	CHECK_OBJECT_SPRINTF()
-	GET_FLOAT(tf);
-
-	/* Special casing for infinity and not a number,
-	 * since many libc's forget about them...
-	 */
-	if (PIKE_ISNAN(tf)) {
-	  /* NaN */
-	  fs->fsp->b = MKPCHARP("nan", 0);
-	  fs->fsp->len = 3;
-	  break;
-	} else if (tf && (tf+tf == tf)) {
-	  /* Infinity. */
-	  if (tf > 0.0) {
-	    fs->fsp->b = MKPCHARP("inf", 0);
-	    fs->fsp->len = 3;
-	  } else {
-	    fs->fsp->b = MKPCHARP("-inf", 0);
-	    fs->fsp->len = 4;
-	  }
-	  break;
-	}
-
 	if (fs->fsp->precision==SPRINTF_UNDECIDED) fs->fsp->precision=3;
 
 	x=(char *)xalloc(100+MAXIMUM(fs->fsp->precision,3));
-	fs->fsp->fi_free_string=x;
 	fs->fsp->b=MKPCHARP(x,0);
 	sprintf(buffer,"%%*.*%c",EXTRACT_PCHARP(a));
+	GET_FLOAT(tf);
 
 	if(fs->fsp->precision<0) {
 	  double m=pow(10.0, (double)fs->fsp->precision);
-	  tf = RINT(tf*m)/m;
+	  tf=RINT(tf*m)/m;
 	} else if (fs->fsp->precision==0) {
-	  tf = RINT(tf);
+	  tf=RINT(tf);
         }
-
-	debug_malloc_touch(x);
+	
 	sprintf(x,buffer,1,fs->fsp->precision<0?0:fs->fsp->precision,tf);
-	debug_malloc_touch(x);
 	fs->fsp->len=strlen(x);
 	
 	/* Make sure that the last digits really are zero. */
 	if(fs->fsp->precision<0)
 	{
-	  ptrdiff_t i, j;
+	  INT32 i, j;
 	  /* Find the ending of the number.  Yes, this can be made
 	     simpler now when the alignment bug for floats is fixed. */
 	  for(i=fs->fsp->len-1; i>=0; i--)
@@ -1456,57 +1219,60 @@ static void low_pike_sprintf(struct format_stack *fs,
 	      break;
 	    }
 	}
+
+	fs->fsp->fi_free_string=x;
 	break;
       }
 
       case 'F':
       {
-        ptrdiff_t l;
+        INT32 l;
 	char *x;
+#if defined(DOUBLE_IS_IEEE_LITTLE) || defined(DOUBLE_IS_IEEE_BIG)
+	double td;
+#endif
         DO_OP();
         l=4;
         if(fs->fsp->width > 0) l=fs->fsp->width;
 	if(l != 4 && l != 8)
-	  sprintf_error(fs, "Invalid IEEE width %ld.\n",
-			PTRDIFF_T_TO_LONG(l));
+	  sprintf_error(fs, "Invalid IEEE width %d.\n", l);
 	x=(char *)alloca(l);
 	fs->fsp->b=MKPCHARP(x,0);
 	fs->fsp->len=l;
 	GET_FLOAT(tf);
 	switch(l) {
 	case 4:
-	  {
 #ifdef FLOAT_IS_IEEE_BIG
-	    float f = DO_NOT_WARN((float)tf);
-	    MEMCPY(x, &f, 4);
+	  MEMCPY(x, &tf, 4);
 #else
 #ifdef FLOAT_IS_IEEE_LITTLE
-	    float f = DO_NOT_WARN((float)tf);
-	    x[0] = ((char *)&f)[3];
-	    x[1] = ((char *)&f)[2];
-	    x[2] = ((char *)&f)[1];
-	    x[3] = ((char *)&f)[0];
+	  x[0] = ((char *)&tf)[3];
+	  x[1] = ((char *)&tf)[2];
+	  x[2] = ((char *)&tf)[1];
+	  x[3] = ((char *)&tf)[0];
 #else
-	    low_write_IEEE_float(x, tf, 4);
+	  low_write_IEEE_float(x, (double)tf, 4);
 #endif
 #endif
-	  }
 	  break;
 	case 8:
 #ifdef DOUBLE_IS_IEEE_BIG
-	  MEMCPY(x, &tf, 8);
+	  td = (double)tf;
+	  MEMCPY(x, &td, 8);
 #else
 #ifdef DOUBLE_IS_IEEE_LITTLE
-	  x[0] = ((char *)&tf)[7];
-	  x[1] = ((char *)&tf)[6];
-	  x[2] = ((char *)&tf)[5];
-	  x[3] = ((char *)&tf)[4];
-	  x[4] = ((char *)&tf)[3];
-	  x[5] = ((char *)&tf)[2];
-	  x[6] = ((char *)&tf)[1];
-	  x[7] = ((char *)&tf)[0];
+	  td = (double)tf;
+
+	  x[0] = ((char *)&td)[7];
+	  x[1] = ((char *)&td)[6];
+	  x[2] = ((char *)&td)[5];
+	  x[3] = ((char *)&td)[4];
+	  x[4] = ((char *)&td)[3];
+	  x[5] = ((char *)&td)[2];
+	  x[6] = ((char *)&td)[1];
+	  x[7] = ((char *)&td)[0];
 #else
-	  low_write_IEEE_float(x, tf, 8);
+	  low_write_IEEE_float(x, (double)tf, 8);
 #endif
 #endif
 	}
@@ -1515,43 +1281,21 @@ static void low_pike_sprintf(struct format_stack *fs,
 
       case 'O':
       {
-	dynamic_buffer save_buf;
-	dynbuf_string s;
+	string s;
 	struct svalue *t;
 	DO_OP();
 	/* No need to do CHECK_OBJECT_SPRINTF() here,
 	   it is checked in describe_svalue. */
 	GET_SVALUE(t);
-	init_buf(&save_buf);
+	init_buf();
 	describe_svalue(t,0,0);
-	s=complex_free_buf(&save_buf);
+	s=complex_free_buf();
 	fs->fsp->b=MKPCHARP(s.str,0);
 	fs->fsp->len=s.len;
 	fs->fsp->fi_free_string=s.str;
 	break;
       }
 
-#if 0
-      /* This can be useful when doing low level debugging. */
-      case 'p':
-      {
-	dynamic_buffer save_buf;
-	dynbuf_string s;
-	char buf[50];
-	struct svalue *t;
-	DO_OP();
-	GET_SVALUE(t);
-	init_buf(&save_buf);
-	sprintf (buf, "%p", t->u.refs);
-	my_strcat (buf);
-	s=complex_free_buf(&save_buf);
-	fs->fsp->b=MKPCHARP(s.str,0);
-	fs->fsp->len=s.len;
-	fs->fsp->fi_free_string=s.str;
-	break;
-      }
-#endif
-
       case 's':
       {
 	struct pike_string *s;
@@ -1577,8 +1321,7 @@ static void low_pike_sprintf(struct format_stack *fs,
     if(((f->flags & INVERSE_COLUMN_MODE) && !f->column_width) ||
        (f->flags & COLUMN_MODE))
     {
-      int max_len,nr;
-      ptrdiff_t columns;
+      int max_len,nr,columns;
       tmp=1;
       for(max_len=nr=e=0;e<f->len;e++)
       {
@@ -1634,7 +1377,7 @@ static void low_pike_sprintf(struct format_stack *fs,
   {
 #ifdef PIKE_DEBUG
     if(fs->fsp < fs->format_info_stack)
-      Pike_fatal("sprintf: fsp out of bounds.\n");
+      fatal("sprintf: fsp out of bounds.\n");
 #endif
     
     if(fs->fsp->fi_free_string)
@@ -1651,14 +1394,9 @@ static void low_pike_sprintf(struct format_stack *fs,
 
 
 /* An C-callable pike_sprintf
-
-** This doesn't seem to work at all.  The prototype for
-** low_pike_sprintf has changed _a lot_ since this code was
-** written.  It should probably be removed.  // Marcus
-
-dynbuf_string pike_sprintf(char *format,struct svalue *argp,int num_arg)
+string pike_sprintf(char *format,struct svalue *argp,int num_arg)
 {
-  dynbuf_string prefix;
+  string prefix;
   prefix.str=0;
   prefix.len=0;
 
@@ -1669,7 +1407,7 @@ dynbuf_string pike_sprintf(char *format,struct svalue *argp,int num_arg)
  */
 
 /* The efun */
-void f_sprintf(INT32 args)
+void f_sprintf(INT32 num_arg)
 {
   ONERROR err_string_builder, err_format_stack;
   struct pike_string *ret;
@@ -1678,28 +1416,12 @@ void f_sprintf(INT32 args)
 
   struct format_stack fs;
 
-  argp=sp-args;
+  argp=sp-num_arg;
   
   fs.fsp = fs.format_info_stack-1;
 
-  if(argp[0].type != T_STRING) {
-    if (argp[0].type == T_OBJECT) {
-      /* Try checking if we can cast it to a string... */
-      ref_push_object(argp[0].u.object);
-      o_cast(string_type_string, PIKE_T_STRING);
-      if (sp[-1].type != T_STRING) {
-	/* We don't accept objects... */
-	Pike_error("sprintf(): Cast to string failed.\n");
-      }
-      /* Replace the original object with the new string. */
-      assign_svalue(argp, sp-1);
-      /* Clean up the stack. */
-      pop_stack();
-    } else {
-      SIMPLE_BAD_ARG_ERROR("sprintf", 1, "string|object");
-    }
-  }
-/*  fprintf(stderr,"SPRINTF: %s\n",argp->u.string->str); */
+  if(argp[0].type != T_STRING)
+    error("Bad argument 1 to sprintf.\n");
 
   init_string_builder(&r,0);
   SET_ONERROR(err_format_stack, free_sprintf_strings, &fs);
@@ -1709,66 +1431,22 @@ void f_sprintf(INT32 args)
 		   MKPCHARP_STR(argp->u.string),
 		   argp->u.string->len,
 		   argp+1,
-		   args-1,
+		   num_arg-1,
 		   0);
   UNSET_ONERROR(err_string_builder);
   UNSET_ONERROR(err_format_stack);
   ret=finish_string_builder(&r);
 
-  pop_n_elems(args);
+  pop_n_elems(num_arg);
   push_string(ret);
 }
 
-static node *optimize_sprintf(node *n)
+void pike_module_init(void)
 {
-  node **arg0 = my_get_arg(&_CDR(n), 0);
-  node **arg1 = my_get_arg(&_CDR(n), 1);
-  node *ret;
-  int num_args=count_args(CDR(n));
-  if(arg0 && arg1 && num_args == 2 &&
-     (*arg0)->token == F_CONSTANT &&
-     (*arg0)->u.sval.type == T_STRING &&
-     (*arg0)->u.sval.u.string->size_shift == 0 &&
-     (*arg0)->u.sval.u.string->len == 2 &&
-     STR0((*arg0)->u.sval.u.string)[0]=='%')
-  {
-    switch(STR0((*arg0)->u.sval.u.string)[1])
-    {
-      case 'c':
-	ADD_NODE_REF2(*arg1,
-		      ret = mkefuncallnode("int2char",*arg1);
-	  );
-	return ret;
-
-      case 't':
-	ADD_NODE_REF2(*arg1,
-		      ret = mkefuncallnode("basetype",*arg1);
-	  );
-	return ret;
-
-      case 'x':
-	ADD_NODE_REF2(*arg1,
-		      ret = mkefuncallnode("int2hex",*arg1);
-	  );
-	return ret;
-
-      default: break;
-    }
-  }
-  return 0;
+  /* function(string, mixed ... : string) */
+  ADD_EFUN("sprintf", f_sprintf,tFuncV(tStr,tMix,tStr), OPT_TRY_OPTIMIZE);
 }
 
-PIKE_MODULE_INIT
-{
-  /* function(string|object, mixed ... : string) */
-  ADD_EFUN2("sprintf", 
-	    f_sprintf,
-	    tFuncV(tOr(tStr, tObj), tMix, tStr),
-	    OPT_TRY_OPTIMIZE,
-	    optimize_sprintf,
-	    0);
-}
-
-PIKE_MODULE_EXIT
+void pike_module_exit(void)
 {
 }
