@@ -5,7 +5,7 @@
 \*/
 
 /*
- * $Id: port.h,v 1.34 2000/12/23 07:34:05 hubbe Exp $
+ * $Id: port.h,v 1.27 2000/06/08 22:10:51 hubbe Exp $
  */
 #ifndef PORT_H
 #define PORT_H
@@ -61,7 +61,7 @@ time_t TIME(time_t *);
 #endif
 
 long STRTOL(char *str,char **ptr,int base);
-PMOD_EXPORT double STRTOD(char * nptr, char **endptr);
+double STRTOD(char * nptr, char **endptr);
 
 #ifndef HAVE_STRCSPN
 int STRCSPN(const char *s,const char * set);
@@ -70,31 +70,19 @@ int STRCSPN(const char *s,const char * set);
 #endif
 
 #ifndef HAVE_STRCASECMP
-PMOD_EXPORT int STRCASECMP(const char *a,const char *b);
+int STRCASECMP(const char *a,const char *b);
 #else
 #  define STRCASECMP strcasecmp
 #endif
 
-#ifndef HAVE_STRNLEN
-PMOD_EXPORT size_t STRNLEN(const char *a,size_t len);
-#else
-#  define STRNLEN strnlen
-#endif
-
-#ifndef HAVE_STRNCMP
-PMOD_EXPORT int STRNCMP(const char *a, const char *b, size_t len);
-#else
-#  define STRNCMP strncmp
-#endif
-
 #ifndef HAVE_MEMSET
-void *MEMSET (void *s,int c,size_t n);
+char *MEMSET (char *s,int c,int n);
 #else
 #  define MEMSET memset
 #endif
 
 #ifdef TRY_USE_MMX
-PMOD_EXPORT void MEMCPY(void *b,const void *a,size_t s);
+void MEMCPY(void *b,const void *a,int s);
 # define __builtin_memcpy MEMCPY
 #else
 # ifndef HAVE_MEMCPY
@@ -102,7 +90,7 @@ PMOD_EXPORT void MEMCPY(void *b,const void *a,size_t s);
 #    define MEMCPY(X,Y,Z) bcopy(Y,X,Z)
 #    define __builtin_memcpy(X,Y,Z) bcopy(Y,X,Z)
 #  else
-void MEMCPY(void *b,const void *a,size_t s);
+void MEMCPY(void *b,const void *a,int s);
 #    define __builtin_memcpy MEMCPY
 #  endif
 # else
@@ -111,28 +99,28 @@ void MEMCPY(void *b,const void *a,size_t s);
 #endif
 
 #ifndef HAVE_MEMMOVE
-PMOD_EXPORT void MEMMOVE(void *b,const void *a,size_t s);
+void MEMMOVE(void *b,const void *a,int s);
 #else
 #  define MEMMOVE memmove
 #endif
 
 #ifndef HAVE_MEMCMP
-PMOD_EXPORT int MEMCMP(const void *b,const void *a,size_t s);
+int MEMCMP(const void *b,const void *a,int s);
 #else
 #  define MEMCMP(X,Y,Z) memcmp((char*)(X),(char*)(Y),(Z))
 #endif
 
 #ifndef HAVE_MEMCHR
-PMOD_EXPORT void *MEMCHR(void *p,char c,size_t e);
+char *MEMCHR(char *p,char c,int e);
 #else
-#  define MEMCHR(X,Y,Z) ((void *)memchr(X,Y,Z))
+#  define MEMCHR(X,Y,Z) ((char *)memchr(X,Y,Z))
 #endif
 
 #ifndef HAVE_STRCHR
 #  ifdef HAVE_INDEX
 #    define STRCHR(X,Y) ((char *)index(X,Y))
 #  else
-PMOD_EXPORT char *STRCHR(char *s,char c);
+char *STRCHR(char *s,char c);
 #  endif
 #else
 #  define STRCHR strchr
@@ -145,20 +133,20 @@ char *STRCHR(char *s,char c);
 #  ifdef HAVE_RINDEX
 #    define STRRCHR(X,Y) ((char *)rindex(X,Y))
 #  else
-PMOD_EXPORT char *STRRCHR(char *s,int c);
+char *STRRCHR(char *s,int c);
 #  endif
 #else
 #  define STRRCHR strrchr
 #endif
 
 #ifndef HAVE_STRSTR
-PMOD_EXPORT char *STRSTR(char *s1,const char *s2);
+char *STRSTR(char *s1,const char *s2);
 #else
 #  define STRSTR strstr
 #endif
 
 #ifndef HAVE_STRTOK
-PMOD_EXPORT char *STRTOK(char *s1,char *s2);
+char *STRTOK(char *s1,char *s2);
 #else
 #  define STRTOK strtok
 #endif
@@ -168,13 +156,13 @@ PMOD_EXPORT char *STRTOK(char *s1,char *s2);
 #endif
 
 #ifndef HAVE_VFPRINTF
-PMOD_EXPORT int VFPRINTF(FILE *f,char *s,va_list args);
+int VFPRINTF(FILE *f,char *s,va_list args);
 #else
 #  define VFPRINTF vfprintf
 #endif
 
 #ifndef HAVE_VSPRINTF
-PMOD_EXPORT int VSPRINTF(char *buf,char *fmt,va_list args);
+int VSPRINTF(char *buf,char *fmt,va_list args);
 #else
 #  define VSPRINTF vsprintf
 #endif
@@ -198,9 +186,9 @@ static INLINE int EXTRACT_CHAR(char *p) { return *p > 0x7f ? *p - 0x100 : *p; }
 #  define EXTRACT_INT(p) (*(INT32 *)(p))
 #else
 #ifdef PIKE_DEBUG
-PMOD_EXPORT unsigned INT16 EXTRACT_UWORD_(unsigned char *p);
-PMOD_EXPORT INT16 EXTRACT_WORD_(unsigned char *p);
-PMOD_EXPORT INT32 EXTRACT_INT_(unsigned char *p);
+unsigned INT16 EXTRACT_UWORD_(unsigned char *p);
+INT16 EXTRACT_WORD_(unsigned char *p);
+INT32 EXTRACT_INT_(unsigned char *p);
 #else
 static INLINE unsigned INT16 EXTRACT_UWORD_(unsigned char *p)
 {
@@ -230,15 +218,7 @@ static INLINE INT32 EXTRACT_INT_(unsigned char *p)
 
 #endif
 
-PMOD_EXPORT unsigned long my_rand(void);
-PMOD_EXPORT void my_srand(long seed);
-
-#ifdef OWN_GETHRTIME
-void own_gethrtime_init(void);
-void own_gethrtime_update(struct timeval *ptr);
-long long gethrtime(void);
-
-#define hrtime_t long long
-#endif
+unsigned long my_rand(void);
+void my_srand(long seed);
 
 #endif
