@@ -1,11 +1,6 @@
 /*
-|| This file is part of Pike. For copyright information see COPYRIGHT.
-|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
-|| for more information.
-|| $Id: preprocessor.h,v 1.66 2004/06/30 17:56:27 grubba Exp $
-*/
-
-/*
+ * $Id: preprocessor.h,v 1.26 2001/05/29 18:10:55 grubba Exp $
+ *
  * Preprocessor template.
  * Based on cpp.c 1.45
  *
@@ -33,7 +28,6 @@
 
 #define lower_cpp		lower_cpp0
 #define find_end_parenthesis	find_end_parenthesis0
-#define find_end_brace		find_end_brace0
 #define PUSH_STRING		PUSH_STRING0
 #define WC_BINARY_FINDSTRING(X, Y)	binary_findstring((char *)X, Y)
 
@@ -68,7 +62,6 @@
 
 #define lower_cpp		lower_cpp1
 #define find_end_parenthesis	find_end_parenthesis1
-#define find_end_brace		find_end_brace1
 #define PUSH_STRING		PUSH_STRING1
 #define WC_BINARY_FINDSTRING	binary_findstring1
 
@@ -97,7 +90,6 @@
 
 #define lower_cpp		lower_cpp2
 #define find_end_parenthesis	find_end_parenthesis2
-#define find_end_brace		find_end_brace2
 #define PUSH_STRING		PUSH_STRING2
 #define WC_BINARY_FINDSTRING	binary_findstring2
 
@@ -119,7 +111,7 @@
 #endif /* SHIFT == 1 */
 
 
-static struct pike_string *WC_BINARY_FINDSTRING(WCHAR *str, ptrdiff_t len)
+static struct pike_string *WC_BINARY_FINDSTRING(WCHAR *str, INT32 len)
 {
   struct pike_string *s;
 #if (SHIFT == 1)
@@ -146,7 +138,7 @@ static struct pike_string *WC_BINARY_FINDSTRING(WCHAR *str, ptrdiff_t len)
  * Generic macros
  */
 #define STRCAT(STR,LEN) do {				\
-  ptrdiff_t x_,len_=(LEN);				\
+  INT32 x_,len_=(LEN);					\
   char *str_=(STR);					\
   if(OUTP())						\
     string_builder_binary_strcat(&this->buf, str_, len_);	\
@@ -157,7 +149,7 @@ static struct pike_string *WC_BINARY_FINDSTRING(WCHAR *str, ptrdiff_t len)
 }while(0)
 
 #define WC_STRCAT(STR,LEN) do {				\
-  ptrdiff_t x_,len_=(LEN);				\
+  INT32 x_,len_=(LEN);					\
   WCHAR *str_=(STR);					\
   if(OUTP())						\
     string_builder_append(&this->buf, MKPCHARP(str_, SHIFT), len_);	\
@@ -185,22 +177,22 @@ static struct pike_string *WC_BINARY_FINDSTRING(WCHAR *str, ptrdiff_t len)
  * Some prototypes
  */
 
-static void PUSH_STRING0(p_wchar0 *, ptrdiff_t, struct string_builder *);
-static void PUSH_STRING1(p_wchar1 *, ptrdiff_t, struct string_builder *);
-static void PUSH_STRING2(p_wchar2 *, ptrdiff_t, struct string_builder *);
+static void PUSH_STRING0(p_wchar0 *, INT32, struct string_builder *);
+static void PUSH_STRING1(p_wchar1 *, INT32, struct string_builder *);
+static void PUSH_STRING2(p_wchar2 *, INT32, struct string_builder *);
 
-static ptrdiff_t calc_0(struct cpp *,p_wchar0 *,ptrdiff_t,ptrdiff_t);
-static ptrdiff_t calc_1(struct cpp *,p_wchar1 *,ptrdiff_t,ptrdiff_t);
-static ptrdiff_t calc_2(struct cpp *,p_wchar2 *,ptrdiff_t,ptrdiff_t);
+static INT32 calc_0(struct cpp *,p_wchar0 *,INT32,INT32);
+static INT32 calc_1(struct cpp *,p_wchar1 *,INT32,INT32);
+static INT32 calc_2(struct cpp *,p_wchar2 *,INT32,INT32);
 
-static ptrdiff_t calc1(struct cpp *,WCHAR *,ptrdiff_t,ptrdiff_t);
+static INT32 calc1(struct cpp *,WCHAR *,INT32,INT32);
 
 /*
  * Functions
  */
 
 #if (SHIFT == 0)
-static void PUSH_STRING_SHIFT(void *str, ptrdiff_t len, int shift,
+static void PUSH_STRING_SHIFT(void *str, INT32 len, int shift,
 			      struct string_builder *buf)
 {
   switch(shift) {
@@ -214,7 +206,7 @@ static void PUSH_STRING_SHIFT(void *str, ptrdiff_t len, int shift,
     PUSH_STRING2((p_wchar2 *)str, len, buf);
     break;
   default:
-    Pike_fatal("cpp(): Bad string shift detected in PUSH_STRING_SHIFT(): %d\n",
+    fatal("cpp(): Bad string shift detected in PUSH_STRING_SHIFT(): %d\n",
 	  shift);
     break;
   }
@@ -222,10 +214,10 @@ static void PUSH_STRING_SHIFT(void *str, ptrdiff_t len, int shift,
 #endif /* SHIFT == 0 */
 
 static void PUSH_STRING(WCHAR *str,
-			ptrdiff_t len,
+			INT32 len,
 			struct string_builder *buf)
 {
-  ptrdiff_t p2;
+  INT32 p2;
   string_builder_putchar(buf, '"');
   for(p2=0; p2<len; p2++)
   {
@@ -293,20 +285,16 @@ static void PUSH_STRING(WCHAR *str,
   string_builder_putchar(buf, '"');
 }
 
-static INLINE ptrdiff_t find_end_parenthesis(struct cpp *this,
-					     WCHAR *data,
-					     ptrdiff_t len,
-					     ptrdiff_t pos)/* pos of first " */
+static INLINE int find_end_parenthesis(struct cpp *this,
+				       WCHAR *data,
+				       INT32 len,
+				       INT32 pos) /* position of first " */
 {
-  INT32 start_line = this->current_line;
   while(1)
   {
     if(pos+1>=len)
     {
-      INT32 save_line = this->current_line;
-      this->current_line = start_line;
       cpp_error(this, "End of file while looking for end parenthesis.");
-      this->current_line = save_line;
       return pos;
     }
 
@@ -317,54 +305,11 @@ static INLINE ptrdiff_t find_end_parenthesis(struct cpp *this,
     case '"':  FIND_END_OF_STRING();  break;
     case '(':  pos=find_end_parenthesis(this, data, len, pos); break;
     case ')':  return pos;
-    case '/':
-      if (data[pos] == '*') {
-	pos++;
-	SKIPCOMMENT();
-      } else if (data[pos] == '/') {
-	pos++;
-	FIND_EOL();
-      }
     }
   }
 }
 
-static INLINE ptrdiff_t find_end_brace(struct cpp *this,
-				       WCHAR *data,
-				       ptrdiff_t len,
-				       ptrdiff_t pos)/* pos of first " */
-{
-  INT32 start_line = this->current_line;
-  while(1)
-  {
-    if(pos+1>=len)
-    {
-      INT32 save_line = this->current_line;
-      this->current_line = start_line;
-      cpp_error(this, "End of file while looking for end brace.");
-      this->current_line = save_line;
-      return pos;
-    }
-
-    switch(data[pos++])
-    {
-    case '\n': PUTNL(); this->current_line++; break;
-    case '\'': FIND_END_OF_CHAR();  break;
-    case '"':  FIND_END_OF_STRING();  break;
-    case '{':  pos=find_end_brace(this, data, len, pos); break;
-    case '}':  return pos;
-    case '/':
-      if (data[pos] == '*') {
-	SKIPCOMMENT();
-      } else if (data[pos] == '/') {
-	FIND_EOL();
-      }
-    }
-  }
-}
-
-static ptrdiff_t calcC(struct cpp *this, WCHAR *data, ptrdiff_t len,
-		       ptrdiff_t pos)
+static INT32 calcC(struct cpp *this,WCHAR *data,INT32 len,INT32 pos)
 {
   FINDTOK();
 
@@ -401,7 +346,7 @@ static ptrdiff_t calcC(struct cpp *this, WCHAR *data, ptrdiff_t len,
     l = STRTOL_PCHARP(p, &p2, 0);
     if(COMPARE_PCHARP(p1,>,p2))
     {
-      push_float(DO_NOT_WARN((FLOAT_TYPE)f));
+      push_float(f);
       pos = ((WCHAR *)p1.ptr) - data;
     }else{
       push_int(l);
@@ -431,14 +376,13 @@ static ptrdiff_t calcC(struct cpp *this, WCHAR *data, ptrdiff_t len,
   }
   
   default:
-    /* fprintf(stderr, "Bad char %c (%d)\n", data[pos], data[pos]); */
+    fprintf(stderr, "Bad char %c (%d)\n", data[pos], data[pos]);
 #ifdef PIKE_DEBUG
     if(WC_ISIDCHAR(data[pos]))
       cpp_error(this, "Syntax error in #if (should not happen).");
 #endif
 
-    cpp_error_sprintf(this, "Syntax error in #if bad character %c (%d).",
-		      data[pos], data[pos]);
+    cpp_error(this, "Syntax error in #if.");
   }
   
 
@@ -458,8 +402,7 @@ static ptrdiff_t calcC(struct cpp *this, WCHAR *data, ptrdiff_t len,
   return pos;
 }
 
-static ptrdiff_t calcB(struct cpp *this, WCHAR *data, ptrdiff_t len,
-		       ptrdiff_t pos)
+static INT32 calcB(struct cpp *this,WCHAR *data,INT32 len,INT32 pos)
 {
   /* DUMPPOS("before calcB"); */
 
@@ -475,8 +418,7 @@ static ptrdiff_t calcB(struct cpp *this, WCHAR *data, ptrdiff_t len,
   return pos;
 }
 
-static ptrdiff_t calcA(struct cpp *this, WCHAR *data, ptrdiff_t len,
-		       ptrdiff_t pos)
+static INT32 calcA(struct cpp *this,WCHAR *data,INT32 len,INT32 pos)
 {
   /* DUMPPOS("before calcA"); */
 
@@ -512,8 +454,7 @@ static ptrdiff_t calcA(struct cpp *this, WCHAR *data, ptrdiff_t len,
   return pos;
 }
 
-static ptrdiff_t calc9(struct cpp *this, WCHAR *data, ptrdiff_t len,
-		       ptrdiff_t pos)
+static INT32 calc9(struct cpp *this,WCHAR *data,INT32 len,INT32 pos)
 {
   /* DUMPPOS("before calc9"); */
 
@@ -544,8 +485,7 @@ static ptrdiff_t calc9(struct cpp *this, WCHAR *data, ptrdiff_t len,
   return pos;
 }
 
-static ptrdiff_t calc8(struct cpp *this, WCHAR *data, ptrdiff_t len,
-		       ptrdiff_t pos)
+static INT32 calc8(struct cpp *this,WCHAR *data,INT32 len,INT32 pos)
 {
   /* DUMPPOS("before calc8"); */
 
@@ -579,8 +519,7 @@ static ptrdiff_t calc8(struct cpp *this, WCHAR *data, ptrdiff_t len,
   return pos;
 }
 
-static ptrdiff_t calc7b(struct cpp *this, WCHAR *data, ptrdiff_t len,
-			ptrdiff_t pos)
+static INT32 calc7b(struct cpp *this,WCHAR *data,INT32 len,INT32 pos)
 {
   /* DUMPPOS("before calc7b"); */
 
@@ -625,8 +564,7 @@ static ptrdiff_t calc7b(struct cpp *this, WCHAR *data, ptrdiff_t len,
   return pos;
 }
 
-static ptrdiff_t calc7(struct cpp *this, WCHAR *data, ptrdiff_t len,
-		       ptrdiff_t pos)
+static INT32 calc7(struct cpp *this,WCHAR *data,INT32 len,INT32 pos)
 {
   /* DUMPPOS("before calc7"); */
 
@@ -659,8 +597,7 @@ static ptrdiff_t calc7(struct cpp *this, WCHAR *data, ptrdiff_t len,
   return pos;
 }
 
-static ptrdiff_t calc6(struct cpp *this, WCHAR *data, ptrdiff_t len,
-		       ptrdiff_t pos)
+static INT32 calc6(struct cpp *this,WCHAR *data,INT32 len,INT32 pos)
 {
   /* DUMPPOS("before calc6"); */
 
@@ -678,8 +615,7 @@ static ptrdiff_t calc6(struct cpp *this, WCHAR *data, ptrdiff_t len,
   return pos;
 }
 
-static ptrdiff_t calc5(struct cpp *this, WCHAR *data, ptrdiff_t len,
-		       ptrdiff_t pos)
+static INT32 calc5(struct cpp *this,WCHAR *data,INT32 len,INT32 pos)
 {
   /* DUMPPOS("before calc5"); */
 
@@ -696,8 +632,7 @@ static ptrdiff_t calc5(struct cpp *this, WCHAR *data, ptrdiff_t len,
   return pos;
 }
 
-static ptrdiff_t calc4(struct cpp *this, WCHAR *data, ptrdiff_t len,
-		       ptrdiff_t pos)
+static INT32 calc4(struct cpp *this,WCHAR *data,INT32 len,INT32 pos)
 {
   /* DUMPPOS("before calc4"); */
 
@@ -714,8 +649,7 @@ static ptrdiff_t calc4(struct cpp *this, WCHAR *data, ptrdiff_t len,
   return pos;
 }
 
-static ptrdiff_t calc3(struct cpp *this, WCHAR *data, ptrdiff_t len,
-		       ptrdiff_t pos)
+static INT32 calc3(struct cpp *this,WCHAR *data,INT32 len,INT32 pos)
 {
   static WCHAR land_[] = { '&', '&' };
 
@@ -729,7 +663,7 @@ static ptrdiff_t calc3(struct cpp *this, WCHAR *data, ptrdiff_t len,
     /* DUMPPOS("inside calc3"); */
 
     check_destructed(Pike_sp-1);
-    if(UNSAFE_IS_ZERO(Pike_sp-1))
+    if(IS_ZERO(Pike_sp-1))
     {
       pos=calc4(this,data,len,pos);
       pop_stack();
@@ -741,8 +675,7 @@ static ptrdiff_t calc3(struct cpp *this, WCHAR *data, ptrdiff_t len,
   return pos;
 }
 
-static ptrdiff_t calc2(struct cpp *this, WCHAR *data, ptrdiff_t len,
-		       ptrdiff_t pos)
+static INT32 calc2(struct cpp *this,WCHAR *data,INT32 len,INT32 pos)
 {
   static WCHAR lor_[] = { '|', '|' };
 
@@ -756,7 +689,7 @@ static ptrdiff_t calc2(struct cpp *this, WCHAR *data, ptrdiff_t len,
     /* DUMPPOS("inside calc2"); */
 
     check_destructed(Pike_sp-1);
-    if(!UNSAFE_IS_ZERO(Pike_sp-1))
+    if(!IS_ZERO(Pike_sp-1))
     {
       pos=calc3(this,data,len,pos);
       pop_stack();
@@ -768,8 +701,7 @@ static ptrdiff_t calc2(struct cpp *this, WCHAR *data, ptrdiff_t len,
   return pos;
 }
 
-static ptrdiff_t calc1(struct cpp *this, WCHAR *data, ptrdiff_t len,
-		       ptrdiff_t pos)
+static INT32 calc1(struct cpp *this, WCHAR *data, INT32 len, INT32 pos)
 {
   /* DUMPPOS("before calc1"); */
 
@@ -785,24 +717,35 @@ static ptrdiff_t calc1(struct cpp *this, WCHAR *data, ptrdiff_t len,
     pos=calc1(this,data,len,pos);
 
     check_destructed(Pike_sp-3);
-    assign_svalue(Pike_sp-3,UNSAFE_IS_ZERO(Pike_sp-3)?Pike_sp-1:Pike_sp-2);
+    assign_svalue(Pike_sp-3,IS_ZERO(Pike_sp-3)?Pike_sp-1:Pike_sp-2);
     pop_n_elems(2);
   }
   return pos;
 }
 
-static ptrdiff_t calc(struct cpp *this, WCHAR *data, ptrdiff_t len,
-		      ptrdiff_t tmp)
+static int calc(struct cpp *this, WCHAR *data, INT32 len, INT32 tmp)
 {
   JMP_BUF recovery;
-  ptrdiff_t pos;
+  INT32 pos;
 
   /* DUMPPOS("Calculating"); */
 
   if (SETJMP(recovery))
   {
-    cpp_handle_exception (this, "Error evaluating expression.");
     pos=tmp;
+    if(throw_value.type == PIKE_T_ARRAY && throw_value.u.array->size)
+    {
+      union anything *a;
+      a=low_array_get_item_ptr(throw_value.u.array, 0, PIKE_T_STRING);
+      if(a)
+      {
+	cpp_error(this, a->string->str);
+      }else{
+	cpp_error(this, "Nonstandard error format.");
+      }
+    }else{
+      cpp_error(this, "Nonstandard error format.");
+    }
     FIND_EOL();
     push_int(0);
   }else{
@@ -816,17 +759,14 @@ static ptrdiff_t calc(struct cpp *this, WCHAR *data, ptrdiff_t len,
   return pos;
 }
 
-static ptrdiff_t lower_cpp(struct cpp *this,
-			   WCHAR *data,
-			   ptrdiff_t len,
-			   int flags,
-			   int auto_convert,
-			   struct pike_string *charset)
+static INT32 lower_cpp(struct cpp *this,
+		       WCHAR *data,
+		       INT32 len,
+		       int flags,
+		       int auto_convert,
+		       struct pike_string *charset)
 {
-  ptrdiff_t pos, tmp, e;
-  int tmp2;
-  INT32 first_line = this->current_line;
-  /* FIXME: What about this->current_file? */
+  INT32 pos, tmp, e, tmp2;
   
   for(pos=0; pos<len;)
   {
@@ -895,7 +835,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
       {
 	struct pike_string *s=0;
 	struct define *d=0;
-	tmp = pos-1;
+	tmp=pos-1;
 	while(WC_ISIDCHAR(data[pos])) pos++;
 
 	if(flags & CPP_DO_IF)
@@ -925,7 +865,6 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	if(d && !(d->inside & 1))
 	{
 	  int arg=0;
-	  INT32 start_line = this->current_line;
 	  struct string_builder tmp;
 	  struct define_argument arguments [MAX_ARGS];
 	  short inside = d->inside;
@@ -935,13 +874,10 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	  if(d->args>=0)
 	  {
 	    SKIPWHITE();
-
+	    
 	    if(!GOBBLE('('))
 	    {
-	      if (s) {
-	        string_builder_shared_strcat(&this->buf,s);
-		free_string(s);
-	      }
+	      cpp_error(this, "Missing '(' in macro call.");
 	      break;
 	    }
 	    
@@ -955,22 +891,12 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 		SKIPWHITE();
 		if(data[pos]==')')
 		{
-		  if((d->varargs && arg + 1 == d->args) ||
-		     (!arg && (d->args == 1))) {
-		    arguments[arg].arg = MKPCHARP(data + pos, SHIFT);
-		    arguments[arg].len=0;
-		    continue;
-		  }else{
-		    if (!d->link.s->size_shift)
-		      cpp_error_sprintf(this,
-					"Too few arguments to macro %s, expected %d.",
-					d->link.s->str, d->args);
-		    else
-		      cpp_error_sprintf(this,
-					"Too few arguments to macro, expected %d.",
-					d->args);
-		    break;
-		  }
+		  char buffer[1024];
+		  sprintf(buffer,
+			  "Too few arguments to macro %.950s, expected %d.",
+			  d->link.s->str, d->args);
+		  cpp_error(this, buffer);
+		  break;
 		}
 	      }
 	      arguments[arg].arg = MKPCHARP(data + pos, SHIFT);
@@ -980,10 +906,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	      {
 		if(pos+1>len)
 		{
-		  INT32 save_line = this->current_line;
-		  this->current_line = start_line;
 		  cpp_error(this, "End of file in macro call.");
-		  this->current_line = save_line;
 		  break;
 		}
 		
@@ -995,17 +918,13 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 		default: continue;
 		  
 		case '"':
-		  /* Note: Strings may contain \-escaped newlines.
-		   *       They must be removed on insertion to
-		   *       avoid being counted twice.
-		   */
 		  if(data[pos-2]!='#') {
 		    FIND_END_OF_STRING();
 		  }else{
 		    FIND_END_OF_STRING2();  /* Newlines allowed */
 		  }
 		  continue;
-
+		  
 		case '\'':
 		  FIND_END_OF_CHAR();
 		  continue;
@@ -1013,16 +932,9 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 		case '(':
 		  pos=find_end_parenthesis(this, data, len, pos);
 		  continue;
-
-		case '{':
-		  pos=find_end_brace(this, data, len, pos);
-		  continue;
 		  
-		case ',':
-		  if(d->varargs && arg+1 == d->args) continue;
-
 		case ')': 
-		  pos--;
+		case ',': pos--;
 		  break;
 		}
 		break;
@@ -1031,14 +943,8 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 		((WCHAR *)(arguments[arg].arg.ptr));
 	    }
 	    SKIPWHITE();
-	    if(!GOBBLE(')')) {
-	      this->current_line = start_line;
-	      if (!d->link.s->size_shift)
-		cpp_error_sprintf(this, "Missing ) in the macro %s.",
-				  d->link.s->str);
-	      else
-		cpp_error(this, "Missing ) in macro.");
-	    }
+	    if(!GOBBLE(')'))
+	      cpp_error(this, "Missing ) in macro call.");
 	  }
 	  
 	  if(d->args >= 0 && arg != d->args)
@@ -1053,7 +959,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	    for(e=0;e<d->num_parts;e++)
 	    {
 	      WCHAR *a;
-	      ptrdiff_t l;
+	      INT32 l;
 	      
 	      if((d->parts[e].argument & DEF_ARG_MASK) < 0 || 
 		 (d->parts[e].argument & DEF_ARG_MASK) >= arg)
@@ -1065,7 +971,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 #ifdef PIKE_DEBUG
 	      if (arguments[d->parts[e].argument&DEF_ARG_MASK].arg.shift !=
 		  SHIFT) {
-		Pike_fatal("cpp(): Bad argument shift %d != %d\n", SHIFT,
+		fatal("cpp(): Bad argument shift %d != %d\n", SHIFT,
 		      arguments[d->parts[e].argument&DEF_ARG_MASK].arg.shift);
 	      }
 #endif /* PIKE_DEBUG */
@@ -1078,53 +984,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	      
 	      if(d->parts[e].argument & DEF_ARG_STRINGIFY)
 	      {
-		/* NOTE: At entry a[0] is non white-space. */
-		int e = 0;
-		string_builder_putchar(&tmp, '"');
-		for(e=0; e<l;) {
-		  if (WC_ISSPACE(a[e]) || a[e]=='"' || a[e]=='\\') {
-		    if (e) {
-		      string_builder_append(&tmp, MKPCHARP(a, SHIFT), e);
-		    }
-		    if (a[e] == '"' || a[e]=='\\') {
-		      /* String or quote. */
-		      string_builder_putchar(&tmp, '\\');
-		      string_builder_putchar(&tmp, a[e]);
-		      if (a[e] == '"') {
-			for (e++; (e < l) && (a[e] != '"'); e++) {
-			  string_builder_putchar(&tmp, a[e]);
-			  if (a[e] == '\\') {
-			    string_builder_putchar(&tmp, '\\');
-			    string_builder_putchar(&tmp, a[++e]);
-			    if (a[e] == '\\') {
-			      string_builder_putchar(&tmp, '\\');
-			    }
-			  }
-			}
-			string_builder_putchar(&tmp, '\\');
-			string_builder_putchar(&tmp, '"');
-			e++;
-		      }
-		    } else {
-		      /* White space. */
-		      while ((e < l) && WC_ISSPACE(a[e])) {
-			e++;
-		      }
-		      if (e != l) {
-			string_builder_putchar(&tmp, ' ');
-		      }
-		    }
-		    a += e;
-		    l -= e;
-		    e = 0;
-		  } else {
-		    e++;
-		  }
-		}
-		if (l) {
-		  string_builder_append(&tmp, MKPCHARP(a, SHIFT), l);
-		}
-		string_builder_putchar(&tmp, '"');
+		PUSH_STRING(a,l,&tmp);
 	      }else{
 		if(DEF_ARG_NOPRESPACE)
 		  while(l && WC_ISSPACE(*a))
@@ -1140,7 +1000,6 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 		}else{
 		  struct string_builder save;
 		  INT32 line=this->current_line;
-		  /* FIXME: Shouldn't we save current_file too? */
 		  save=this->buf;
 		  this->buf=tmp;
 		  d->inside = 2;
@@ -1162,24 +1021,10 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	    }
 	  }
 	  
-	  /* Remove any newlines from the completed expression. */
-	  switch (tmp.s->size_shift) {
-	  case 0:
-	    for(e=0; e< (ptrdiff_t)tmp.s->len; e++)
-	      if(STR0(tmp.s)[e]=='\n')
-		STR0(tmp.s)[e]=' ';
-	    break;
-	  case 1:
-	    for(e=0; e< (ptrdiff_t)tmp.s->len; e++)
-	      if(STR1(tmp.s)[e]=='\n')
-		STR1(tmp.s)[e]=' ';
-	    break;
-	  case 2:
-	    for(e=0; e< (ptrdiff_t)tmp.s->len; e++)
-	      if(STR2(tmp.s)[e]=='\n')
-		STR2(tmp.s)[e]=' ';
-	    break;
-	  }
+	  /* FIXME */
+	  for(e=0;e<(long)tmp.s->len;e++)
+	    if(tmp.s->str[e]=='\n')
+	      tmp.s->str[e]=' ';
 
 	  if(s) d->inside=1;
 	  
@@ -1187,7 +1032,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	  tmp.s->len--;
 #ifdef PIKE_DEBUG
 	  if (tmp.s->size_shift != SHIFT) {
-	    Pike_fatal("cpp(): Bad shift in #if: %d != %d\n",
+	    fatal("cpp(): Bad shift in #if: %d != %d\n",
 		  tmp.s->size_shift, SHIFT);
 	  }
 #endif /* PIKE_DEBUG */
@@ -1318,7 +1163,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 			   this->current_file->len,&this->buf);
 	      break;
 	    default:
-	      Pike_fatal("cpp(): Filename has bad shift %d\n",
+	      fatal("cpp(): Filename has bad shift %d\n",
 		    this->current_file->size_shift);
 	      break;
 	  }
@@ -1373,8 +1218,6 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	  SKIPSPACE();
 	  
 	  check_stack(3);
-
-	  /* FIXME: Ought to macro-expand (Bug 2440). */
 	  
 	  switch(data[pos++])
 	  {
@@ -1392,7 +1235,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 
 	    case '<':
 	      {
-		ptrdiff_t tmp = pos;
+		INT32 tmp=pos;
 		while(data[pos]!='>')
 		{
 		  if(data[pos]=='\n')
@@ -1430,24 +1273,26 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	  {
 	    struct pike_string *new_file;
 
-	    if (!safe_apply_handler ("handle_include",
-				     this->handler, this->compat_handler,
-				     3, BIT_STRING) ||
-		!(new_file=Pike_sp[-1].u.string) ) {
-	      cpp_handle_exception (this, "Couldn't include file.");
+	    SAFE_APPLY_MASTER("handle_include",3);
+	  
+	    if(Pike_sp[-1].type != PIKE_T_STRING)
+	    {
+	      cpp_error(this, "Couldn't include file.");
 	      pop_n_elems(Pike_sp-save_sp);
 	      break;
 	    }
+	    
+	    new_file=Pike_sp[-1].u.string;
 
 	    /* Why not just use ref_push_string(new_file)? */
 	    assign_svalue_no_free(Pike_sp,Pike_sp-1);
 	    Pike_sp++;
-	    dmalloc_touch_svalue(Pike_sp-1);
-
-	    if (!safe_apply_handler ("read_include",
-				     this->handler, this->compat_handler,
-				     1, BIT_STRING)) {
-	      cpp_handle_exception (this, "Couldn't read include file.");
+	    
+	    SAFE_APPLY_MASTER("read_include",1);
+	    
+	    if(Pike_sp[-1].type != PIKE_T_STRING)
+	    {
+	      cpp_error(this, "Couldn't read include file.");
 	      pop_n_elems(Pike_sp-save_sp);
 	      break;
 	    }
@@ -1475,17 +1320,15 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	      }else{
 		/* #include */
 		if (auto_convert) {
-		  struct pike_string *new_str =
-		    recode_string(this, Pike_sp[-1].u.string);
+		  struct pike_string *new_str = recode_string(Pike_sp[-1].u.string);
 		  free_string(Pike_sp[-1].u.string);
 		  Pike_sp[-1].u.string = new_str;
 		} else if (charset) {
 		  ref_push_string(charset);
-		  if (!safe_apply_handler ("decode_charset",
-					   this->handler, this->compat_handler,
-					   2, BIT_STRING)) {
-		    cpp_handle_exception (this,
-					  "Charset decoding failed for included file.");
+		  SAFE_APPLY_MASTER("decode_charset", 2);
+		  if (Pike_sp[-1].type != PIKE_T_STRING) {
+		    cpp_error(this,
+			      "Charset decoding failed for included file.");
 		    pop_n_elems(Pike_sp - save_sp);
 		    break;
 		  }
@@ -1526,21 +1369,20 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 
       if(WGOBBLE2(if_))
       {
-	struct string_builder save, tmp;
-	INT32 nflags = 0;
+	struct string_builder save,tmp;
+	INT32 nflags=CPP_EXPECT_ELSE | CPP_EXPECT_ENDIF;
 #ifdef PIKE_DEBUG
-	ptrdiff_t skip;
+	int skip;
 #endif /* PIKE_DEBUG */
 	
 	if(!OUTP())
-	  nflags = CPP_REALLY_NO_OUTPUT;
+	  nflags|=CPP_REALLY_NO_OUTPUT;
 	
 	save=this->buf;
 	init_string_builder(&this->buf, SHIFT);
 #ifdef PIKE_DEBUG
 	/* DUMPPOS("#if before lower_cpp()"); */
-	skip = lower_cpp(this, data+pos, len-pos,
-			 nflags | CPP_END_AT_NEWLINE | CPP_DO_IF,
+	skip = lower_cpp(this, data+pos, len-pos, CPP_END_AT_NEWLINE|CPP_DO_IF,
 			 auto_convert, charset);
 	/* fprintf(stderr, "shift:%d, pos:%d, len:%d, skip:%d\n",
 	   SHIFT, pos, len, skip); */
@@ -1555,54 +1397,49 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	/* fprintf(stderr, "\"\n"); */
 	/* fflush(stderr); */
 #else /* !PIKE_DEBUG */
-	pos += lower_cpp(this, data+pos, len-pos,
-			 nflags | CPP_END_AT_NEWLINE | CPP_DO_IF,
-			 auto_convert, charset);
+	pos+=lower_cpp(this, data+pos, len-pos, CPP_END_AT_NEWLINE|CPP_DO_IF,
+		       auto_convert, charset);
 #endif /* PIKE_DEBUG */
 	tmp=this->buf;
 	this->buf=save;
 	
 	string_builder_putchar(&tmp, 0);
 	tmp.s->len--;
-
-	if (!nflags) {
-	  switch(tmp.s->size_shift) {
-	  case 0:
-	    calc_0(this, (p_wchar0 *)tmp.s->str, tmp.s->len, 0);
-	    break;
-	  case 1:
-	    calc_1(this, (p_wchar1 *)tmp.s->str, tmp.s->len, 0);
-	    break;
-	  case 2:
-	    calc_2(this, (p_wchar2 *)tmp.s->str, tmp.s->len, 0);
-	    break;
-	  default:
-	    Pike_fatal("cpp(): Bad shift: %d\n", tmp.s->size_shift);
-	    break;
-	  }
-	  if(SAFE_IS_ZERO(Pike_sp-1)) nflags|=CPP_NO_OUTPUT;
-	  pop_stack();
+	
+	switch(tmp.s->size_shift) {
+	case 0:
+	  calc_0(this, (p_wchar0 *)tmp.s->str, tmp.s->len, 0);
+	  break;
+	case 1:
+	  calc_1(this, (p_wchar1 *)tmp.s->str, tmp.s->len, 0);
+	  break;
+	case 2:
+	  calc_2(this, (p_wchar2 *)tmp.s->str, tmp.s->len, 0);
+	  break;
+	default:
+	  fatal("cpp(): Bad shift: %d\n", tmp.s->size_shift);
+	  break;
 	}
 	free_string_builder(&tmp);
-	pos += lower_cpp(this, data+pos, len-pos,
-			 nflags | CPP_EXPECT_ELSE | CPP_EXPECT_ENDIF,
+	if(IS_ZERO(Pike_sp-1)) nflags|=CPP_NO_OUTPUT;
+	pop_stack();
+	pos += lower_cpp(this, data+pos, len-pos, nflags,
 			 auto_convert, charset);
 	break;
       }
 
       if(WGOBBLE2(ifdef_))
 	{
-	  INT32 nflags;
-	  ptrdiff_t namestart;
+	  INT32 namestart,nflags;
 	  struct pike_string *s;
 	  SKIPSPACE();
 
 	  if(!WC_ISIDCHAR(data[pos]))
 	    cpp_error(this, "#ifdef what?");
 
-	  namestart = pos;
+	  namestart=pos;
 	  while(WC_ISIDCHAR(data[pos])) pos++;
-	  nflags = CPP_EXPECT_ELSE | CPP_EXPECT_ENDIF | CPP_NO_OUTPUT;
+	  nflags=CPP_EXPECT_ELSE | CPP_EXPECT_ENDIF | CPP_NO_OUTPUT;
 
 	  if(!OUTP())
 	    nflags|=CPP_REALLY_NO_OUTPUT;
@@ -1618,8 +1455,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 
       if(WGOBBLE2(ifndef_))
 	{
-	  INT32 nflags;
-	  ptrdiff_t namestart;
+	  INT32 namestart,nflags;
 	  struct pike_string *s;
 	  SKIPSPACE();
 
@@ -1683,17 +1519,17 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	
 	flags|=CPP_EXPECT_ENDIF;
 	
-	if((flags & (CPP_NO_OUTPUT | CPP_REALLY_NO_OUTPUT)) == CPP_NO_OUTPUT)
+	if(flags & CPP_NO_OUTPUT)
 	{
 	  struct string_builder save,tmp;
 	  save=this->buf;
 	  init_string_builder(&this->buf, 0);
 	  pos += lower_cpp(this, data+pos, len-pos,
-			   CPP_END_AT_NEWLINE | CPP_DO_IF,
+			   CPP_END_AT_NEWLINE|CPP_DO_IF,
 			   auto_convert, charset);
 	  tmp=this->buf;
 	  this->buf=save;
-
+	  
 	  string_builder_putchar(&tmp, 0);
 	  tmp.s->len--;
 	  
@@ -1708,23 +1544,22 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	    calc_2(this, (p_wchar2 *)tmp.s->str, tmp.s->len,0);
 	    break;
 	  default:
-	    Pike_fatal("cpp(): Bad shift: %d\n", tmp.s->size_shift);
+	    fatal("cpp(): Bad shift: %d\n", tmp.s->size_shift);
 	    break;
 	  }
 	  free_string_builder(&tmp);
-	  if(!SAFE_IS_ZERO(Pike_sp-1)) flags&=~CPP_NO_OUTPUT;
+	  if(!IS_ZERO(Pike_sp-1)) flags&=~CPP_NO_OUTPUT;
 	  pop_stack();
-	} else {
+	}else{
 	  FIND_EOL();
-	  flags |= CPP_NO_OUTPUT | CPP_REALLY_NO_OUTPUT;
+	  flags|= CPP_NO_OUTPUT | CPP_REALLY_NO_OUTPUT;
 	}
 	break;
       }
 
       if(WGOBBLE2(error_))
 	{
-          ptrdiff_t foo;
-
+          INT32 foo;
           SKIPSPACE();
           foo=pos;
           FIND_EOL();
@@ -1753,11 +1588,9 @@ static ptrdiff_t lower_cpp(struct cpp *this,
       if(WGOBBLE2(define_))
 	{
 	  struct string_builder str;
-	  INT32 argno=-1;
-	  ptrdiff_t tmp3, namestart, nameend;
+	  INT32 namestart, tmp3, nameend, argno=-1;
 	  struct define *def;
 	  struct svalue *partbase,*argbase=Pike_sp;
-	  int varargs=0;
 
 	  SKIPSPACE();
 
@@ -1781,7 +1614,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 
 	      while(data[pos]!=')')
 		{
-		  ptrdiff_t tmp2;
+		  INT32 tmp2;
 		  if(argno)
 		    {
 		      if(!GOBBLE(','))
@@ -1789,8 +1622,6 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 				  "Expecting comma in macro definition.");
 		      SKIPWHITE();
 		    }
-		  if(varargs)
-		    cpp_error(this,"Expected ) after ...");
 		  tmp2=pos;
 
 		  if(!WC_ISIDCHAR(data[pos]))
@@ -1821,13 +1652,6 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 		    pop_stack();
 		    argno--;
 		  }
-
-		  if(data[pos]=='.' && data[pos+1]=='.' && data[pos+2]=='.')
-		  {
-		    varargs=1;
-		    pos+=3;
-		    SKIPWHITE();
-		  }
 		}
 
 	      if(!GOBBLE(')'))
@@ -1884,7 +1708,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 		{
 #ifdef PIKE_DEBUG
 		  if(Pike_sp[-1].type != PIKE_T_INT)
-		    Pike_fatal("Internal error in CPP\n");
+		    fatal("Internal error in CPP\n");
 #endif
 		  Pike_sp[-1].u.integer|=DEF_ARG_NOPOSTSPACE;
 		}
@@ -1912,7 +1736,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 			check_stack(2);
 			push_string(finish_string_builder(&str));
 			init_string_builder(&str, 0);
-			push_int(DO_NOT_WARN(e | extra));
+			push_int(e | extra);
 			extra=0;
 			break;
 		      }
@@ -1981,16 +1805,15 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 #endif /* SHIFT == 0 */
 	    copy_shared_string(def->first, partbase->u.string);
 	    def->args=argno;
-	    def->varargs=varargs;
 	    
 	    for(e=0;e<def->num_parts;e++)
 	    {
-#ifdef PIKE_DEBUG
+#if 1
 	      if(partbase[e*2+1].type != PIKE_T_INT)
-		Pike_fatal("Cpp internal error, expected integer!\n");
+		fatal("Cpp internal error, expected integer!\n");
 	      
 	      if(partbase[e*2+2].type != PIKE_T_STRING)
-		Pike_fatal("Cpp internal error, expected string!\n");
+		fatal("Cpp internal error, expected string!\n");
 #endif
 	      def->parts[e].argument=partbase[e*2+1].u.integer;
 	      copy_shared_string(def->parts[e].postfix,
@@ -2000,7 +1823,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 #ifdef PIKE_DEBUG
 	    if(def->num_parts==1 &&
 	       (def->parts[0].argument & DEF_ARG_MASK) > MAX_ARGS)
-	      Pike_fatal("Internal error in define\n");
+	      fatal("Internal error in define\n");
 #endif	  
 	    {
 	      struct define *d;
@@ -2027,7 +1850,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	/* NOTE: Reuses undefine_ for undef_ */
       if(WGOBBLE2(undefine_) || WGOBBLE2(undef_))
 	{
-	  ptrdiff_t tmp;
+	  INT32 tmp;
 	  struct pike_string *s;
 
 	  SKIPSPACE();
@@ -2059,7 +1882,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	static WCHAR charset_[] = { 'c', 'h', 'a', 'r', 's', 'e', 't' };
 
       if (WGOBBLE2(charset_)) {
-	ptrdiff_t p;
+	INT32 p;
 	struct pike_string *s;
 
 	if (flags & (CPP_EXPECT_ENDIF | CPP_EXPECT_ELSE)) {
@@ -2093,9 +1916,11 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	  MEMCPY(s->str, data + p, pos - p);
 	  push_string(end_shared_string(s));
 
-	  if (!safe_apply_handler ("decode_charset", this->handler, this->compat_handler,
-				   2, BIT_STRING)) {
-	    cpp_handle_exception (this, NULL);
+	  SAFE_APPLY_MASTER("decode_charset", 2);
+
+	  if (Pike_sp[-1].type != PIKE_T_STRING) {
+	    pop_stack();
+	    cpp_error(this, "Unknown charset.");
 	  } else {
 	    low_cpp(this, Pike_sp[-1].u.string->str, Pike_sp[-1].u.string->len,
 		    Pike_sp[-1].u.string->size_shift, flags,
@@ -2127,66 +1952,11 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	}
 	if(WGOBBLE2(pike_))
 	{
-	  
-	  if(OUTP())
-	  {
-	    int major, minor;
-	    ptrdiff_t tmp;
-	    PCHARP ptr;
-
-	    STRCAT("#pike", 5);
-	    tmp= this->buf.s->len;
-	    pos +=  lower_cpp(this, data+pos, len-pos,
-			      CPP_END_AT_NEWLINE | CPP_DO_IF,
-			      auto_convert, charset);
-
-	    ptr=MKPCHARP(this->buf.s->str, this->buf.s->size_shift);
-	    INC_PCHARP(ptr, tmp);
-
-	    major=STRTOL_PCHARP(ptr, &ptr, 10);
-	    if(INDEX_PCHARP(ptr,0) == '.')
-	    {
-	      INC_PCHARP(ptr, 1);
-	      minor=STRTOL_PCHARP(ptr, &ptr, 10);
-	      cpp_change_compat(this, major, minor);
-	    }else{
-	      cpp_error(this, "Missing '.' in #pike.");
-	      this->compat_minor=0;
-	    }
-	  }
-	  else
-	    FIND_EOL();
+	  /* FIXME */
+	  FIND_EOL();
 	  break;
 	}
       }
-    case 'w': /* warning */
-      {
-	static WCHAR warning_[] = { 'w', 'a', 'r', 'n', 'i', 'n', 'g' };
-
-	if(WGOBBLE2(warning_))
-	{
-	  ptrdiff_t foo;
-
-          SKIPSPACE();
-          foo=pos;
-          FIND_EOL();
-	  if(OUTP())
-	  {
-#if (SHIFT == 0)
-	    push_string(make_shared_binary_string((char *)data+foo, pos-foo));
-#else /* SHIFT != 0 */
-#if (SHIFT == 1)
-	    push_string(make_shared_binary_string1(data+foo, pos-foo));
-#else /* SHIFT != 1 */
-	    push_string(make_shared_binary_string2(data+foo, pos-foo));
-#endif /* SHIFT == 1 */
-#endif /* SHIFT == 0 */
-	    cpp_warning(this, Pike_sp[-1].u.string->str);
-	  }
-	  break;
-	}
-	goto unknown_preprocessor_directive;
-      }
     default:
     unknown_preprocessor_directive:
       {
@@ -2205,12 +1975,8 @@ static ptrdiff_t lower_cpp(struct cpp *this,
     }
   }
 
-  if(flags & CPP_EXPECT_ENDIF) {
-    INT32 saved_line = this->current_line;
-    this->current_line = first_line;
+  if(flags & CPP_EXPECT_ENDIF)
     cpp_error(this, "End of file while searching for #endif.");
-    this->current_line = saved_line;
-  }
 
   return pos;
 }
@@ -2229,7 +1995,6 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 
 #undef lower_cpp
 #undef find_end_parenthesis
-#undef find_end_brace
 
 #undef calc
 #undef calc1
@@ -2245,3 +2010,4 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 #undef calcA
 #undef calcB
 #undef calcC
+
